//**************************************************************************************************
//						БИБЛИОТЕКА RT_HW_BASE.h
// *************************************************************************************************
#ifndef   RT_HW_BASE_h
#define   RT_HW_BASE_h
#define   RT_HW_BASE_VER 		401									//--Номер версии библиотеки;
#define   RT_HW_BASE_VER_NAME 	"4.0.1"								//--Строка с версией библиотеки;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//						1.Библиотеки привязки к Arduino
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#if defined(ARDUINO) && (ARDUINO >= 100) 
#include <Arduino.h>
#else
#include <WProgram.h>
#endif
#include <stdint.h>
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//						2.Библиотеки для всех архитектур
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#include <Wire.h>												//--Подключение библиотеки i2c;	   	
#include <SPI.h>												//--Подключение библиотеки SPI
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//							3.Макросы для RT_HW_BASE
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#define RT_HW_MICROS				micros()	//--Макрос измерения времени в mcs;
#define RT_HW_MILLIS				millis()	//--Макрос измерения времени в ms;
#define RT_HW_CODE_CHAR_BREAK		'~'			//--Код "Запрещающего" символа;
#define RT_HW_CODE_CHAR_BLANK		's'			//--Символ заполнения='пробел' (для удобства применения в блоках FLProg);
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//						3.Подключение дополнительных файлов 1
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#include "RT_HW_BASE_01_01_DEFINED_BOARD.hpp"	    //--Идентификаторы контроллеров;
#include "RT_HW_BASE_02_00_BOARD_ARCH_FOR_ALL.hpp"	//--Архитектура для всех контроллеров;
#include "RT_HW_BASE_02_01_BOARD_ARCH_AVR.hpp"	    //--Контроллеры архитектуры AVR и DUE;
#include "RT_HW_BASE_02_02_BOARD_ARCH_ESP8266.hpp"	//--Контроллеры архитектуры ESP8266;
#include "RT_HW_BASE_02_03_BOARD_ARCH_ESP32.hpp"	//--Контроллеры архитектуры ESP32;
#include "RT_HW_BASE_02_04_BOARD_ARCH_OTHERS.hpp"	//--Контроллеры архитектуры SAM и другие;
#include "RT_HW_BASE_02_05_BOARD_ARCH_STM32F1.hpp"	//--Контроллеры архитектуры STM32F1;
#include "RT_HW_BASE_02_06_BOARD_ARCH_STM32F4.hpp"	//--Контроллеры архитектуры STM32F4;
#include "RT_HW_BASE_02_07_BOARD_ARCH_STM32F7.hpp"	//--Контроллеры архитектуры STM32F7(H7);
#include "RT_HW_BASE_02_99_BOARD_ARCH_ANON.hpp"		//--Архитектура ANON (последний в перечни);
#include "RT_HW_BASE_03_00_DEFINED_STRUCT.hpp"		//--Структуры для использования в RT_HW_BASE.h;
//=================================================================================================
//								КЛАСС RT_HW_BASE.h
//=================================================================================================
class RT_HW_BASE{
public:
//-----Данные--------------------------------------------------------------------------------------
	RT_HW_STRUCT_SHED		shed;
	RT_HW_STRUCT_DEVICE 	device;		//--Данные по устройствам;
	RT_HW_STRUCT_CONSOLE	console;	//--Данные консоли;
//-------------------------------------------------------------------------------------------------
uint32_t buff32;															//--Рабочий регистр;
//-----Методы--------------------------------------------------------------------------------------
RT_HW_BASE();																	//--Конструктор
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//						1.Функции работы со временем
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
uint32_t getPastTime(uint32_t &timeBegin, uint32_t timeEnd);					//--Возвращает разницу между двумя временными точками. 
uint32_t getPastMs 	(uint32_t &timeBegin);										//--Возвращает прошедшее время в мс;
uint32_t getPastMcs	(uint32_t &timeBegin);										//--Возвращает прошедшее время в мкс; 
uint8_t	 timeOut	(uint32_t &timeBegin, uint32_t timeEnd, uint16_t timeMax);	//--Контроль тайм-аута;
uint8_t  timeOutMs	(uint32_t &timeBegin, uint16_t timeMax);					//--Контроль тайм-аута мс;
uint8_t  timeOutMcs	(uint32_t &timeBegin, uint16_t timeMax);					//--Контроль тайм-аута мкс;
uint8_t  period		(uint32_t &timeBegin, uint32_t timeEnd, uint32_t period);	//--Генерация события с точным поддержания периода и защитой от пропуска шага.
void 	 sheduler();															//--Диспетчер задач;
void 	 divideEvent(RT_HW_DIVIDE_EVENT &id, uint8_t num=5, bool EN=1);			//--Делитель событий;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//						2.Базовые функции работы с консолью
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void consoleBegin(int8_t numUart,bool permit=1);//--Инициализация консоли;
bool consoleCheck();							//--Проверка готовности консоли;
void consoleWrite(uint8_t val);					//--Вывод байта;
void consolePrint(String str);					//--Вывод строки;
void consoleSuff (char suff);					//--Вывод суффикса: ='~' нет вывода; ='Y' перевод строки; =другой символ-вывод 'этого символа;
//----Вывод с добавлением суффикса-----------------------------------------------------------------
void consoleCR   (char suff='~');										//--Перевод строки;
void consoleChar (char val, char suff='~');								//--Вывод символа; 
void consoleBool (bool v,char sign0='-',char sign1='+',char suff='~'); 	//--Вывод bool
void consoleBlank(uint8_t n=1,char blank=' ',char suff='~');			//--Вывод n заполнителей blank;

void consoleInt8 (int8_t  v,  char mode='I', char suff='~');			//--Вывод int8_t  (mode=char->I,U);
void consoleInt16(int16_t v,  char mode='I', char suff='~');			//--Вывод int16_t (mode=char->I,U);
void consoleInt32(int32_t v,  char mode='I', char suff='~');			//--Вывод int32_t (mode=char->I,U);
void consoleFloat(float   v,  char mode='1', char suff='~');			//--Вывод float   (mode=1,2,3,4);	
void consoleStr  (String &v,                 char suff='~');			//--Вывод строковой переменной по адресу переменной;
void consoleText (String  v,                 char suff='~');			//--Вывод строковой переменной;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//						3.Форматированный вывод на консоль
// mode -вариант вывода; sz-длина поля, suff-суффикс;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//----Суффикса, Val int32_t(mode:D,C,S,T,I,U,H,h,B,b,0,1,2,3,4); float((mode:0,1,2,3,4); String(mode: char->L,C,R);
void ConsoleLine   (uint8_t n=80,char blank='-');						//--Вывод горизонтальной линии; 
//void ConsoleSuff   (char suff);											
void ConsoleVal    (char mode, uint8_t sz, char suff, int32_t val);
void ConsoleFloat  (char mode, uint8_t sz, char suff, float   val);
void ConsoleText   (char mode, uint8_t sz, char suff, char blank, String str);
//----Форматированный вывод до 4-х int8_t,int16_t,int32_t,float-------------------------------------
void ConsoleV8x4   (char mode, uint8_t sz, char suff, uint8_t qnt, int8_t  v1=0, int8_t  v2=0, int8_t  v3=0, int8_t  v4=0);
void ConsoleV16x4  (char mode, uint8_t sz, char suff, uint8_t qnt, int16_t v1=0, int16_t v2=0, int16_t v3=0, int16_t v4=0);
void ConsoleV32x4  (char mode, uint8_t sz, char suff, uint8_t qnt, int32_t v1=0, int32_t v2=0, int32_t v3=0, int32_t v4=0);	
void ConsoleFloatx4(char mode, uint8_t sz, char suff, uint8_t qnt, float   v1=0, float   v2=0, float   v3=0, float   v4=0);	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//						4.Вывод на консоль параметров
//  	mode='A' ->  вывод всех элементов в формате int8_t 	с форматированием;
//		mode='B' ->  вывод всех элементов в формате uint8_t c форматированием;
//		mode='U' ->  вывод всех элементов в формате uint8_t c форматированием до трех знаков (вывод байтов);
//		mode='C' ->  вывод элементов 0-127, вместо остальных элементов выводятся заполнители;
//      mode='D' ->  вывод элементов 0-127, остальные элементы не выводятся;
//  	mode='S' ->  вывод всех элементов в формате int8_t без форматирования;
//      mode='P' ->  вывод в формате пинов(для ESP8266,STM32 в текстовой форме),вместо отсутствующих пины(=-1) выводится заполнитель;
//      mode='N' ->  ничего не выводится;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
void ConsoleDataTime();							//--Вывод даты и времени компиляции;
void ConsoleNameBoard();						//--Вывод наименования и параметров контроллера;
void ConsoleDepthBoard();						//--Вывод разрядности контроллера;
void ConsoleParamWire();						//--Вывод параметров i2c;
void ConsoleParamSPI();							//--Вывод параметров SPI;
void ConsoleParamUART();						//--Вывод параметров UART (UHS,USS);
void ConsoleParamCAN();							//--Вывод параметров CAN;
void ConsoleParamBTH();							//--Вывод параметров встроенного BlueTooth;
void ConsoleParamETH();							//--Вывод параметров Ethernet;
void ConsoleParamWFI();							//--Вывод параметров Wi-Fi;
void ConsoleSysParam (char mode='B');			//--Вывод массива системных параметров;
void ConsoleSysPins	 (char mode='P');			//--Вывод массива системных пинов;	
void ConsoleLinkPins (char mode='P');			//--Вывод массива пинов интерфейсов;	
void ConsoleNetParam (char mode='U');			//--Вывод массива пинов сетевых интерфейсов;
void ConsolePinsBoard(char mode='P');			//--Вывод массива пинов контроллера;	
void ConsolePinsTest (char mode='P');			//--Вывод массива тестовых пинов;
void ConsoleAdrTest	 (char mode='A');			//--Вывод массива тестовых адресов;
void ConsoleNamePins();							//--Вывод наименование пинов для ESP8266,STM32, по другим контроллерам не выводится ничего;
void ConsoleScanWire(uint8_t bus);				//--Сканирование шины i2c;

void ConsoleArrPGM  (uint8_t kind, char mode);  //--Вывод данных типа kind в формате mode(A,B,U,C,D,S,P,N);
void ConsoleTablePinsESP8266();					//--Вывод таблицы с номерами пинов для ESP8266;
void ConsoleTablePinsSTM32();					//--Вывод таблицы с номерами пинов для STM32;
//--------------------------------------------------Вспомогательные функции---------------------------
bool 	checkPinx4(uint8_t qnt, int8_t v1, int8_t v2, int8_t v3, int8_t v4); //--Контроль на наличие до 4-х пинов;
void 	ConsolePinx4(uint8_t qnt, int8_t v1, int8_t v2, int8_t v3, int8_t v4); //--Вывод до 4-х пинов;

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//						5.Вспомогательные  функции работы с консолью
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool 	checkChange(uint16_t* buff,uint16_t v1,uint16_t v2,uint16_t v3,uint16_t v4);//--Проверка изменения 4-х значений;
char    boolToChar (bool val, char sign_0='-',char sign_1='+');					//--Конвертация bool в char; 
uint8_t getLenBegin(uint8_t len, uint8_t src, char alig);						//--Кол-во заполнителей спереди; 
uint8_t getLenEnd  (uint8_t len, uint8_t src, char alig);						//--Кол-во заполнителей сзади;
uint8_t getLenVal  (char mode, int32_t val);									//--
uint8_t getLenPGM  (const char *text, uint8_t lenMax);							//--Получение длины строки в PROGMEM;
uint8_t getLenStr  (String &str,      uint8_t lenMax);							//--Получение длины строки в оперативной памяти;
String  getNamePin (int8_t pin);												//--Получение символьное значение пина;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// 						6.Вспомогательные  функции разные
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
uint8_t  getFistBitUint16(uint16_t &reg);										//--Возвращает первый свободный бит регистра reg;
uint16_t maxBitDepth(uint8_t  v);  												//--Возвращает макс.число числа разрядности v;
uint16_t normADC  	(uint16_t v);												//--Нормализация прочитанного ADC значения;
uint16_t normPWM  	(uint16_t v);												//--Нормализация выводимого   PWM значения;
uint16_t normDAC  	(uint16_t v);												//--Нормализация выводимого   DAC значения;
uint16_t makeDelta  (uint16_t v1, uint16_t v2, uint8_t delta);					//--Получения значения при выходе за гистерезис
uint8_t  getNumberVal(uint32_t val, char mode, uint8_t len, uint8_t n);			//--Получение цифры из числа (D,H,B);
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
//						7.Служебные функции работы с пинами
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
uint8_t  readArrPGM   (uint8_t kind, uint8_t n); //--Чтение       из массива PGM значения с индексом n;
uint8_t  getLenFullPGM(uint8_t kind);			 //--Получение длины массива PGM;		
uint8_t  getLenPosPGM (uint8_t kind);			 //--Получение длины массива PGM только для значений в диапазоне 0-127;
bool     checkPinPGM  (uint8_t kind, uint8_t n); //--Проверяет наличия пина в массиве PGM;
int8_t   getIntPGM	  (uint8_t kind, uint8_t n); //--Возвращает значение int8_t  из массива PGM по индексу. Если n  >значения массива,то возвращается -1;
uint8_t  getUintPGM   (uint8_t kind, uint8_t n); //--Возвращает значение uint8_t  из массива PGM по индексу. Если n  >значения массива,то возвращается 0;
uint16_t getValPGM	  (uint8_t kind, uint8_t n); //--Возвращает значение int16_t из массива PGM по индексу. Если n+1>значения массива,то возвращается -1;
String	 getNameKindPin(uint8_t kind);			 //--Возвращает символьное обозначение типа пинов.
String 	 getNamePinESP8266(int8_t pin);			 //--Возвращает символьное обозначение номера пина для ESP8266
String 	 getNamePinSTM32(int8_t pin=-1);		 //--Возвращает символьное обозначение номера пина для STM32
int8_t	 checkPinSTM32  (char reg, uint8_t num); //--Возвращает номер пина по обозначению регистра ('A'-'G') и биту в регистре(0-15);
int8_t	 getPinFromString(String str,uint8_t num);//--Возвращает номер пина из строки, разделенной запятыми; 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		Работа с пинами чрез прямой доступ к регистрам ввода/вывода
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
void 	 directPinSetID	(RT_HW_PIN_DIR_ID &id, int8_t pin); 					//--Настройка параметров пина - адрес регистра,маска,режим использования;
void 	 directPinDigitalWrite(RT_HW_PIN_DIR_ID &id, int8_t pin, uint8_t val);
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		Работа с пинами средствами Wiring
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
void 	 setPinID       (RT_HW_PIN_DIR_ID &id, int8_t pin, uint8_t kind, char mode='S',uint16_t freq=1000);	//--Инициализация пина для быстрой работы;
bool 	 pinDigitalRead (RT_HW_PIN_DIR_ID &id);					//--Быстрое дискретное чтение пина;
uint16_t pinAnalogRead  (RT_HW_PIN_DIR_ID &id, uint16_t &ago, char mode='F');		//--Аналоговое чтение пина;	
uint16_t pinTouchRead	(RT_HW_PIN_DIR_ID &id, 				  char mode='F');
void     pinDigitalWrite(RT_HW_PIN_DIR_ID &id, uint8_t val);	//--Быстрая дискретная запись пина;
int16_t  pinReadHALL();
uint16_t pinWritePWM	(RT_HW_PIN_DIR_ID &id, uint16_t val, uint16_t freq=1000, char mode='N');
uint16_t pinWriteDAC	(RT_HW_PIN_DIR_ID &id, uint16_t val);				//--Запись DAC;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		Базовые функции работы с UART (n: =0-6 -аппаратные UART; =7-программный UART  
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool	uartCheckNum (uint8_t num);						//--Проверка наличия в контроллере UART c номером n; 
void 	uartBegin    (uint8_t n, uint32_t speed);		//--Инициализация UART;
bool	uartSerial   (uint8_t n);						//--Ожидание готовности  UART;
uint8_t uartAvailable(uint8_t n);						//--Получение кол-ва байт в буфере чтения;
uint8_t uartAvailableForWrite(uint8_t n);				//--Получение кол-ва байт в буфере записи;
uint8_t uartRead     (uint8_t n);						//--Чтение байта         из буфера чтения;
void 	uartWrite    (uint8_t n, uint8_t val);			//--Запись байта в буфер записи;
void 	uartPrint    (uint8_t n, String str);			//--Вывод строки в буфер записи;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		LINK: РАБОТА С ИНТЕРФЕЙСАМИ
//clockFrequency: the value (in Hertz) of desired communication clock. 
//		Accepted values are 100000 (standard mode) and 400000 (fast //mode). 
//		Some processors also support 10000 (low speed mode), 1000000 (fast mode plus) and 3400000 (high speed mode). 
//		Please refer to //the specific processor documentation to make sure the desired mode is supported.
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
uint32_t convertModeFreqWire(uint8_t mode);	//--Конвертация кода скорости в значение скорости i2c;
void    wireBegin(int8_t sda=-1, int8_t scl=-1, uint8_t slaveAdr=0, uint8_t bus=0); //--Инициализация библиотеки и подключение к i2c;
void    wireBeginTransmission(int8_t adr, 		uint8_t bus=0);	//--Инициирование команды отправки по указанному адресу;
uint8_t wireEndTransmission(					uint8_t bus=0);	//--Завершение процесса передачи, инциированный функцией
void    wireSetClock(uint8_t speed, 			uint8_t bus=0);	//--Установка частоты i2c;

void    wireWrite(uint8_t val,            		uint8_t bus=0);	//--Передача байта;
void    wireWrite(String str,   				uint8_t bus=0);	//--Передача последовательности байт;
void    wireWrite(uint8_t data, uint8_t length, uint8_t bus=0);	//--Передача байтов из массива data в кол-ве length;
void	wireRequestFrom(int8_t adr,uint8_t qnt, uint8_t bus=0);	
bool    findDeviceWire(RT_HW_STRUCT_FIND_I2C &id, uint8_t adr, uint8_t bus=0);	//--Поиск устройства с заданным адресом на шине i2c;
void    scanDeviceWire(uint8_t bus=0);							//--Сканирование шины i2c с номеров bus;
//-------------------------------------------------------------------------------------------------
private:
};  //---END 
//-------------------------------------------------------------------------------------------------


//=================================================================================================
extern RT_HW_BASE 	RT_HW_Base; //--Создание объекта RT_HW_base;
//HardwareSerial Serial2(USART2);
//HardwareSerial Serial3(USART3);
//=================================================================================================
#endif //---RT_HW_BASE_h

