
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//						Макросы корректировки прерываний для архитектур ESP8266
// Для совместимостью с ESP8266 перед каждой функцией прерываний следует включить RT_HW_ICACHE_RAM_ATTR
// Пример: void RT_HW_ICACHE_RAM_ATTR handleInterrupt(){}; 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#if defined(RT_HW_ARCH_ESP8266)
#define RT_HW_ICACHE_RAM_ATTR		ICACHE_RAM_ATTR				
#else
#define RT_HW_ICACHE_RAM_ATTR		
#endif

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//						1.Архитектура AVR
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#if defined(RT_HW_ARCH_AVR)
//----------------------1.1.Основные параметры-----------------------------------------------------
#define RT_HW_ARCH_CODE			RT_HW_ARCH_AVR_ID				//--Код архитектуры;
#define RT_HW_ARCH_NAME			"AVR"							//--Наименование архитектуры;
#define RT_HW_DATE_TYPE 		uint8_t							//--Тип базовой переменной при прямом управлении пинами;
#define RT_HW_PIN_TXT_LEN 		2								//--Размер поля при выводе на консоль номеров пинов; 
#define RT_HW_PIN_QNT_LINE 		24								//--Кол-во пинов в стройке при выводе таблицы на консоль; 
struct  RT_HW_PIN_DIR_ID {uint8_t pin; uint8_t dir; uint16_t mode; uint8_t bit; volatile uint8_t *base;};							
//----------------------1.2.Дополнительные библиотеки----------------------------------------------
#include <SoftwareSerial.h>										//--Библиотека программного UART;
#include <avr/pgmspace.h>										//--Библиотека для работы с PROGMEM;	
//----------------------Корректировка атрибутов для MEGA2560---------------------------------------
#if defined(RT_HW_BOARD_AVR_MEGA2560)	
  #ifdef  PROGMEM
  #undef  PROGMEM
  #endif
#define PROGMEM __attribute__(( section(".progmem.data") ))
#endif
//----------------------1.3.Макросы для хранения в PROGMEM-----------------------------------------
#define RT_HW_BUILD_BYTE_PGM(a) 	const byte    a[] PROGMEM  	//--Создание в PROGMEM массива byte 
#define RT_HW_BUILD_INT8_PGM(a) 	const int8_t  a[] PROGMEM  	//--Создание в PROGMEM массива int8_t 
#define RT_HW_READ_BYTE_PGM(arr,n) 	pgm_read_byte(&arr[n])		//--Чтение из массива arr байта с номером n(для <64 kB);
//----------------------1.4.Макросы для работы c пинами через функции WIRING-----------------------  
#define RT_HW_PIN_MODE_INPUT  				INPUT				//--Дискретный вход;
#define RT_HW_PIN_MODE_OUTPUT  				OUTPUT				//--Дискретный выход;
#define RT_HW_PIN_MODE_INPUT_PULLUP  		INPUT_PULLUP 		//--Дискретный вход с подтяжкой к +Vcc;
//----------------------1.5.Макросы для совместимости с STM32--------------------------------------
#define RT_HW_PIN_MODE_INPUT_FLOATING  		INPUT				//--Дискретный вход - аналог INPUT;
#define RT_HW_PIN_MODE_INPUT_PULLDOWN  		INPUT				//--Дискретный вход с подтяжкой к GND;
#define RT_HW_PIN_MODE_INPUT_ANALOG  		INPUT				//--Аналоговый вход;
#define RT_HW_PIN_MODE_OUTPUT_OPEN_DRAIN  	OUTPUT				//--Дискретный выход с открытым выходом;
#define RT_HW_PIN_MODE_OUTPUT_PWM				OUTPUT			//--ШИМ выход;
#define RT_HW_PIN_MODE_OUTPUT_PWM_OPEN_DRAIN	OUTPUT			//--ШИМ выход с открытым коллектором;
//----------------------1.6.Макросы прямого управления пинами--------------------------------------
#define RT_HW_PIN_DIR_SET_ID(id,p) 	    	id.pin=p; id.bit=digitalPinToBitMask(p); id.base=portInputRegister(digitalPinToPort(p));
#define RT_HW_PIN_DIR_MODE_INPUT(id)        *((id.base)+(1)) &=~id.bit; *((id.base)+(2))&=~id.bit; 
#define RT_HW_PIN_DIR_MODE_PULLUP(id) 	    *((id.base)+(1)) &=~id.bit;  *((id.base)+(2))|= id.bit;
#define RT_HW_PIN_DIR_MODE_OUTPUT(id) 	    *((id.base)+(1)) |= id.bit;
#define RT_HW_PIN_DIR_READ(id)           (((*((id.base)+(0))) & id.bit) ? (1) : (0)); 
#define RT_HW_PIN_DIR_WRITE(id,val) if(val){*((id.base)+(2)) |= id.bit;} else{*((id.base)+(2)) &= ~id.bit;};
#define RT_HW_PIN_DIR_WRITE_LOW(id)         *((id.base)+(2)) &= ~id.bit;
#define RT_HW_PIN_DIR_WRITE_HIGH(id)        *((id.base)+(2)) |= id.bit;
//-------------------------------------------------------------------------------------------------	
#endif			
//-------------------------------------------------------------------------------------------------	

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//						2.Архитектура ESP8266
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#if defined(RT_HW_ARCH_ESP8266)
//----------------------2.1.Основные параметры-----------------------------------------------------
#define RT_HW_ARCH_CODE 		RT_HW_ARCH_ESP8266_ID			//--Код архитектуры;
#define RT_HW_ARCH_NAME			"ESP8266" 						//--Наименование архитектуры;
#define RT_HW_DATE_TYPE 		uint32_t		 				//--Тип базовой переменной при прямом управлении пинами;
#define RT_HW_PIN_TXT_LEN 		3 								//--Размер поля при выводе на консоль номеров пинов;
#define RT_HW_PIN_QNT_LINE 		18 								//--Кол-во пинов в стройке при выводе таблицы на консоль; 
struct  RT_HW_PIN_DIR_ID{uint32_t pin;       uint32_t dir;       uint16_t mode;      uint32_t GPFFS_pin; 
						 uint32_t LEFT_PIN;  uint32_t LEFT_GPCI; uint32_t LEFT_GPCD; uint32_t LEFT_GPFPU;};
//----------------------2.2.Дополнительные библиотеки----------------------------------------------
#include <SoftwareSerial.h>										//--Библиотека программного UART;
#include <avr/pgmspace.h>										//--Работа с PROGMEM;
//----------------------2.3.Макросы для хранения в PROGMEM-----------------------------------------
#define RT_HW_BUILD_BYTE_PGM(a) 	const byte    a[] PROGMEM  	//--Создание в PROGMEM массива byte 
#define RT_HW_BUILD_INT8_PGM(a) 	const int8_t  a[] PROGMEM  	//--Создание в PROGMEM массива int8_t 
#define RT_HW_READ_BYTE_PGM(arr,n) 	pgm_read_byte(arr+n)		//--Чтение из массива arr байта с номером n
//----------------------2.4.Макросы для работы c пинами через функции WIRING----------------------- 
#define RT_HW_PIN_MODE_INPUT  				INPUT				//--Дискретный вход;
#define RT_HW_PIN_MODE_OUTPUT  				OUTPUT				//--Дискретный выход;
#define RT_HW_PIN_MODE_INPUT_PULLUP  		INPUT_PULLUP 		//--Дискретный вход с подтяжкой к +Vcc;
//----------------------2.5.Макросы для совместимости с STM32--------------------------------------
#define RT_HW_PIN_MODE_INPUT_FLOATING  		INPUT				//--Дискретный вход - аналог INPUT;
#define RT_HW_PIN_MODE_INPUT_PULLDOWN  		INPUT				//--Дискретный вход с подтяжкой к GND;
#define RT_HW_PIN_MODE_INPUT_ANALOG  		INPUT				//--Аналоговый вход;
#define RT_HW_PIN_MODE_OUTPUT_OPEN_DRAIN  	OUTPUT				//--Дискретный выход с открытым выходом;
#define RT_HW_PIN_MODE_OUTPUT_PWM				OUTPUT			//--ШИМ выход;
#define RT_HW_PIN_MODE_OUTPUT_PWM_OPEN_DRAIN	OUTPUT			//--ШИМ выход с открытым коллектором;
//----------------------2.6.Макросы прямого управления пинами--------------------------------------
static inline __attribute__((always_inline))		//--Чтение дискретного пина
bool RT_HW_BASE_digitalReadFast(RT_HW_PIN_DIR_ID &id) {if(id.pin < 16){return GPIP(id.pin);} else if(id.pin == 16){return GP16I & 0x01;} return 0;};
#define RT_HW_PIN_DIR_SET_ID(id,p)    id.pin=p; id.GPFFS_pin=GPFFS(GPFFS_GPIO(id.pin)); id.LEFT_PIN=(1<<id.pin); id.LEFT_GPCI=(0xF<<GPCI);\
                                                id.LEFT_GPCD=(1<<GPCD); id.LEFT_GPFPU=(1<<GPFPU);
#define RT_HW_PIN_DIR_MODE_INPUT(id)  GPF(id.pin) =id.GPFFS_pin; GPEC=id.LEFT_PIN; GPC(id.pin)=(GPC(id.pin) & id.LEFT_GPCI) | (id.LEFT_GPCD);
#define RT_HW_PIN_DIR_MODE_PULLUP(id) GPF(id.pin) =id.GPFFS_pin; GPEC=id.LEFT_PIN; GPC(id.pin)=(GPC(id.pin) & id.LEFT_GPCI) | (id.LEFT_GPCD);\
                                      GPF(id.pin)|=id.LEFT_GPFPU;
#define RT_HW_PIN_DIR_MODE_OUTPUT(id) GPF(id.pin)=id.GPFFS_pin; GPC(id.pin)=GPC(id.pin) & id.LEFT_GPCI; GPES = id.LEFT_PIN;
#define RT_HW_PIN_DIR_READ(id)        RT_HW_BASE_digitalReadFast(id);
#define RT_HW_PIN_DIR_WRITE(id,val)   if(val) GPOS=id.LEFT_PIN; else GPOC=id.LEFT_PIN;
#define RT_HW_PIN_DIR_WRITE_LOW(id,val)                              GPOC=id.LEFT_PIN;
#define RT_HW_PIN_DIR_WRITE_HIGH(id,val)      GPOS=id.LEFT_PIN;
//-------------------------------------------------------------------------------------------------
#endif
//-------------------------------------------------------------------------------------------------

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//						3.Архитектура ESP32
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#if defined(RT_HW_ARCH_ESP32) 
//----------------------3.1.Основные параметры-----------------------------------------------------
#define RT_HW_ARCH_CODE 		RT_HW_ARCH_ESP32_ID 			//--Код архитектуры;
#define RT_HW_ARCH_NAME			"ESP32" 						//--Наименование архитектуры;
#define RT_HW_DATE_TYPE 		uint32_t						//--Тип базовой переменной при прямом управлении пинами;
#define RT_HW_PIN_TXT_LEN 		2 								//--Размер поля при выводе на консоль номеров пинов; 
#define RT_HW_PIN_QNT_LINE 		20								//--Кол-во пинов в стройке при выводе таблицы на консоль; 
struct  RT_HW_PIN_DIR_ID {uint32_t pin; uint32_t dir; uint32_t mode; uint32_t bit; uint32_t num; uint32_t freq;}; //--Структура для прямого управления пинами
//----------------------3.2.Подключение библиотек--------------------------------------------------
#include "esp32-hal-ledc.h"										//--Библиотека для работы с PWM и LED;	
#include <driver/rtc_io.h>										//--
#include <pgmspace.h>											//--Работа с PROGMEM;
//----------------------3.3.Макросы для хранения в PROGMEM-----------------------------------------
#define RT_HW_BUILD_BYTE_PGM(a) 	const byte    a[] PROGMEM  	//--Создание в PROGMEM массива byte 
#define RT_HW_BUILD_INT8_PGM(a) 	const int8_t  a[] PROGMEM  	//--Создание в PROGMEM массива int8_t 
#define RT_HW_READ_BYTE_PGM(arr,n) 	pgm_read_byte(arr+n)		//--Чтение из массива arr байта с номером n
//----------------------Макросы корректировки прерываний для архитектуры ESP32---------------------
#ifdef  interrupts
#undef  interrupts
#endif
#ifdef  noInterrupts
#undef  noInterrupts
#endif
#define noInterrupts() {portMUX_TYPE mux = portMUX_INITIALIZER_UNLOCKED; portENTER_CRITICAL(&mux)
#define interrupts() portEXIT_CRITICAL(&mux);}
//----------------------3.4.Макросы для работы c пинами через функции WIRING----------------------- 
#define RT_HW_PIN_MODE_INPUT  				INPUT				//--Дискретный вход;
#define RT_HW_PIN_MODE_OUTPUT  				OUTPUT				//--Дискретный выход;
#define RT_HW_PIN_MODE_INPUT_PULLUP  		INPUT_PULLUP 		//--Дискретный вход с подтяжкой к +Vcc;
//----------------------3.5.Макросы для совместимости с STM32--------------------------------------
#define RT_HW_PIN_MODE_INPUT_FLOATING  		INPUT				//--Дискретный вход - аналог INPUT;
#define RT_HW_PIN_MODE_INPUT_PULLDOWN  		INPUT_PULLDOWN		//--Дискретный вход с подтяжкой к GND;
#define RT_HW_PIN_MODE_INPUT_ANALOG  		INPUT				//--Аналоговый вход;
#define RT_HW_PIN_MODE_OUTPUT_OPEN_DRAIN  	OUTPUT_OPEN_DRAIN	//--Дискретный выход с открытым выходом;
#define RT_HW_PIN_MODE_OUTPUT_PWM				OUTPUT			//--ШИМ выход;
#define RT_HW_PIN_MODE_OUTPUT_PWM_OPEN_DRAIN	OUTPUT			//--ШИМ выход с открытым коллектором;
//----------------------3.6.Макросы прямого управления пинами--------------------------------------
#define RT_HW_PIN_DIR_SET_ID(id,p) 	        id.pin=p; id.bit=id.pin; 
#define RT_HW_PIN_DIR_MODE_INPUT(id)        pinMode(id.pin,INPUT);
#define RT_HW_PIN_DIR_MODE_PULLUP(id) 	    pinMode(id.pin,INPUT_PULLUP);
#define RT_HW_PIN_DIR_MODE_OUTPUT(id) 	    pinMode(id.pin,OUTPUT);
#define RT_HW_PIN_DIR_READ(id)              digitalRead(id.pin);
#define RT_HW_PIN_DIR_WRITE(id,val) 		digitalWrite(id.pin,val);
#define RT_HW_PIN_DIR_WRITE_LOW(id)         digitalWrite(id.pin,0);
#define RT_HW_PIN_DIR_WRITE_HIGH(id)        digitalWrite(id.pin,1);
//-------------------------------------------------------------------------------------------------
#endif
//-------------------------------------------------------------------------------------------------

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//						4.Архитектура SAM
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#if defined(RT_HW_ARCH_SAM)
//----------------------4.1.Основные параметры-----------------------------------------------------
#define RT_HW_ARCH_CODE			RT_HW_ARCH_SAM_ID				//--Код архитектуры;
#define RT_HW_ARCH_NAME			"SAM"							//--Наименование архитектуры;
#define RT_HW_DATE_TYPE 		uint32_t						//--Тип базовой переменной при прямом управлении пинами;
#define RT_HW_PIN_TXT_LEN 		2 								//--Размер поля при выводе на консоль номеров пинов;
#define RT_HW_PIN_QNT_LINE 		24  							//--Кол-во пинов в стройке при выводе таблицы на консоль; 
struct  RT_HW_PIN_DIR_ID {uint32_t pin; uint32_t dir; uint32_t mode; uint32_t bit; volatile uint32_t *base;}; 
//----------------------4.2.Подключение библиотек--------------------------------------------------
#include <avr/pgmspace.h>										//--Работа с PROGMEM;
//----------------------4.3.Макросы для хранения в PROGMEM-----------------------------------------
#define RT_HW_BUILD_BYTE_PGM(a) 	const byte    a[] PROGMEM  	//--Создание в PROGMEM массива byte 
#define RT_HW_BUILD_INT8_PGM(a) 	const int8_t  a[] PROGMEM  	//--Создание в PROGMEM массива int8_t 
#define RT_HW_READ_BYTE_PGM(arr,n) 	pgm_read_byte(arr+n)		//--Чтение из массива arr байта с номером n
//----------------------4.4.Макросы для работы c пинами через функции WIRING----------------------- 
#define RT_HW_PIN_MODE_INPUT  				INPUT				//--Дискретный вход;
#define RT_HW_PIN_MODE_OUTPUT  				OUTPUT				//--Дискретный выход;
#define RT_HW_PIN_MODE_INPUT_PULLUP  		INPUT_PULLUP 		//--Дискретный вход с подтяжкой к +Vcc;
//----------------------4.5.Макросы для совместимости с STM32--------------------------------------
#define RT_HW_PIN_MODE_INPUT_FLOATING  		INPUT				//--Дискретный вход - аналог INPUT;
#define RT_HW_PIN_MODE_INPUT_PULLDOWN  		INPUT				//--Дискретный вход с подтяжкой к GND;
#define RT_HW_PIN_MODE_INPUT_ANALOG  		INPUT				//--Аналоговый вход;
#define RT_HW_PIN_MODE_OUTPUT_OPEN_DRAIN  	OUTPUT				//--Дискретный выход с открытым выходом;
//----------------------4.6.Макросы прямого управления пинами--------------------------------------
#define RT_HW_PIN_DIR_SET_ID(id,p) 	    	id.pin=p; id.bit=digitalPinToBitMask(p); id.base=(&(digitalPinToPort(p)->PIO_PER));
#define RT_HW_PIN_DIR_MODE_INPUT(id)        pinMode(id.pin,INPUT); 				// *((id.base)+(5))= id.bit;   
#define RT_HW_PIN_DIR_MODE_PULLUP(id) 	    pinMode(id.pin,INPUT_PULLUP);		// *((id.base)+(6))= id.bit;  
#define RT_HW_PIN_DIR_MODE_OUTPUT(id) 	    *((id.base)+(4))= id.bit;
#define RT_HW_PIN_DIR_READ(id)           	(((*((id.base)+(15))) & id.bit) ? (1) : (0)); 
#define RT_HW_PIN_DIR_WRITE(id,val) 		if(val){*((id.base)+(12))= id.bit;} else{*((id.base)+(13))= id.bit;};
#define RT_HW_PIN_DIR_WRITE_LOW(id)         *((id.base)+(13))= id.bit;
#define RT_HW_PIN_DIR_WRITE_HIGH(id)        *((id.base)+(12))= id.bit;
//-------------------------------------------------------------------------------------------------
#endif
//-------------------------------------------------------------------------------------------------

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//						4.Архитектура SEEEDUINO
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#if defined(RT_HW_ARCH_SEEEDUINO)
//----------------------4.1.Основные параметры-----------------------------------------------------
#define RT_HW_ARCH_CODE			RT_HW_ARCH_SEEEDUINO_ID			//--Код архитектуры;
#define RT_HW_ARCH_NAME			"SEEEDUINO"						//--Наименование архитектуры;
#define RT_HW_DATE_TYPE 		uint32_t						//--Тип базовой переменной при прямом управлении пинами;
#define RT_HW_PIN_TXT_LEN 		2 								//--Размер поля при выводе на консоль номеров пинов;
#define RT_HW_PIN_QNT_LINE 		24  							//--Кол-во пинов в стройке при выводе таблицы на консоль; 
struct  RT_HW_PIN_DIR_ID {uint32_t pin; uint32_t dir; uint32_t mode; uint32_t bit; volatile uint32_t *base;}; 
//----------------------4.2.Подключение библиотек--------------------------------------------------
#include <avr/pgmspace.h>										//--Работа с PROGMEM;
//----------------------4.3.Макросы для хранения в PROGMEM-----------------------------------------
#define RT_HW_BUILD_BYTE_PGM(a) 	const byte    a[] PROGMEM  	//--Создание в PROGMEM массива byte 
#define RT_HW_BUILD_INT8_PGM(a) 	const int8_t  a[] PROGMEM  	//--Создание в PROGMEM массива int8_t 
#define RT_HW_READ_BYTE_PGM(arr,n) 	pgm_read_byte(arr+n)		//--Чтение из массива arr байта с номером n
//----------------------4.4.Макросы для работы c пинами через функции WIRING----------------------- 
#define RT_HW_PIN_MODE_INPUT  				INPUT				//--Дискретный вход;
#define RT_HW_PIN_MODE_OUTPUT  				OUTPUT				//--Дискретный выход;
#define RT_HW_PIN_MODE_INPUT_PULLUP  		INPUT_PULLUP 		//--Дискретный вход с подтяжкой к +Vcc;
//----------------------4.5.Макросы для совместимости с STM32--------------------------------------
#define RT_HW_PIN_MODE_INPUT_FLOATING  		INPUT				//--Дискретный вход - аналог INPUT;
#define RT_HW_PIN_MODE_INPUT_PULLDOWN  		INPUT				//--Дискретный вход с подтяжкой к GND;
#define RT_HW_PIN_MODE_INPUT_ANALOG  		INPUT				//--Аналоговый вход;
#define RT_HW_PIN_MODE_OUTPUT_OPEN_DRAIN  	OUTPUT				//--Дискретный выход с открытым выходом;
#define RT_HW_PIN_MODE_OUTPUT_PWM				OUTPUT			//--ШИМ выход;
#define RT_HW_PIN_MODE_OUTPUT_PWM_OPEN_DRAIN	OUTPUT			//--ШИМ выход с открытым коллектором;
//----------------------4.6.Макросы прямого управления пинами--------------------------------------
#define RT_HW_PIN_DIR_SET_ID(id,p) 	    	id.pin=p; 
#define RT_HW_PIN_DIR_MODE_INPUT(id)        pinMode(id.pin,INPUT); 				
#define RT_HW_PIN_DIR_MODE_PULLUP(id) 	    pinMode(id.pin,INPUT_PULLUP);
#define RT_HW_PIN_DIR_MODE_OUTPUT(id) 	    pinMode(id.pin,OUTPUT);
#define RT_HW_PIN_DIR_READ(id)           	digitalRead(id.pin); 
#define RT_HW_PIN_DIR_WRITE(id,val) 		digitalWrite(id.pin,val);
#define RT_HW_PIN_DIR_WRITE_LOW(id)         digitalWrite(id.pin,0);
#define RT_HW_PIN_DIR_WRITE_HIGH(id)        digitalWrite(id.pin,1);
//-------------------------------------------------------------------------------------------------
#endif
//-------------------------------------------------------------------------------------------------


//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//						5,6.Архитектура STM32F1
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#if defined(RT_HW_ARCH_STM32F1) 
//----------------------5.1.Основные параметры-----------------------------------------------------
#define RT_HW_ARCH_CODE 		RT_HW_ARCH_STM32F1_ID 			//--Код архитектуры;
#define RT_HW_ARCH_NAME			"STM32F1" 						//--Наименование архитектуры;
#define RT_HW_DATE_TYPE 		uint32_t 						//--Тип базовой переменной при прямом управлении пинами;
#define RT_HW_PIN_TXT_LEN 		4  								//--Размер поля при выводе на консоль номеров пинов;
#define RT_HW_PIN_QNT_LINE 		16 								//--Кол-во пинов в стройке при выводе таблицы на консоль; 
struct  RT_HW_PIN_DIR_ID {uint32_t pin; uint32_t dir; uint32_t mode; uint32_t bit; volatile uint32_t *base;}; //--Структура для прямого управления пинами
//----------------------5.2.Подключение библиотек--------------------------------------------------
#include <avr/pgmspace.h>										//--Работа с PROGMEM;
//-------------------------------------------------------------------------------------------------
#if defined(RT_HW_ARCH_STM32DUINO)
#include "usbd_cdc_if.h"										//--Для работы с USB_CDC в среде STM32duino
#endif
//----------------------5.3.Макросы для хранения в PROGMEM-----------------------------------------
#define RT_HW_BUILD_BYTE_PGM(a) 	const byte    a[] PROGMEM  	//--Создание в PROGMEM массива byte 
#define RT_HW_BUILD_INT8_PGM(a) 	const int8_t  a[] PROGMEM  	//--Создание в PROGMEM массива int8_t 
#define RT_HW_READ_BYTE_PGM(arr,n) 	pgm_read_byte(arr+n)		//--Чтение из массива arr байта с номером n
//----------------------5.4.Макросы для работы c пинами через функции WIRING-----------------------
#define RT_HW_PIN_MODE_INPUT  				INPUT				//--Дискретный вход;
#define RT_HW_PIN_MODE_OUTPUT  				OUTPUT				//--Дискретный выход;
#define RT_HW_PIN_MODE_INPUT_PULLUP  		INPUT_PULLUP 		//--Дискретный вход с подтяжкой к +Vcc;
//----------------------5.5.Макросы для совместимости с STM32--------------------------------------
#define RT_HW_PIN_MODE_INPUT_FLOATING  		INPUT				//--Дискретный вход - аналог INPUT;
#define RT_HW_PIN_MODE_INPUT_PULLDOWN  		INPUT_PULLDOWN		//--Дискретный вход с подтяжкой к GND;
#define RT_HW_PIN_MODE_INPUT_ANALOG  		INPUT_ANALOG		//--Аналоговый вход;
#define RT_HW_PIN_MODE_OUTPUT_OPEN_DRAIN  	OUTPUT_OPEN_DRAIN	//--Дискретный выход с открытым выходом;
#define RT_HW_PIN_MODE_OUTPUT_PWM				PWM				//--ШИМ выход;
#define RT_HW_PIN_MODE_OUTPUT_PWM_OPEN_DRAIN	PWM_OPEN_DRAIN	//--ШИМ выход с открытым коллектором;
//----------------------5.6.Макросы прямого управления пинами--------------------------------------
#define RT_HW_PIN_DIR_SET_ID(id,p)	  id.pin=p; id.bit=id.pin; 
#define RT_HW_PIN_DIR_MODE_INPUT(id)  pinMode(id.pin,INPUT);		//pin_function((PinName)id.pin, STM_PIN_DATA(STM_MODE_INPUT,     GPIO_NOPULL,0))
#define RT_HW_PIN_DIR_MODE_PULLUP(id) pinMode(id.pin,INPUT_PULLUP);	//pin_function((PinName)id.pin, STM_PIN_DATA(STM_MODE_INPUT,     GPIO_NOPULL,0))
#define RT_HW_PIN_DIR_MODE_OUTPUT(id) pinMode(id.pin,OUTPUT);		//pin_function((PinName)id.pin, STM_PIN_DATA(STM_MODE_OUTPUT_PP, GPIO_NOPULL,0))
#define RT_HW_PIN_DIR_READ(id)        digitalRead(id.pin);			//digitalReadFast((PinName)id.pin);  //digitalRead(id.pin);
#define RT_HW_PIN_DIR_WRITE(id,val)   digitalWrite(id.pin,val);		//if(val){digitalWriteFast((PinName)id.pin,HIGH);} else {digitalWriteFast((PinName)id.pin,LOW);} 
#define RT_HW_PIN_DIR_WRITE_LOW(id)   digitalWrite(id.pin,0);		//digitalWriteFast((PinName)id.pin, LOW); 	//digitalWrite(id.pin,0);
#define RT_HW_PIN_DIR_WRITE_HIGH(id)  digitalWrite(id.pin,l);		//digitalWriteFast((PinName)id.pin, HIGH);	//digitalWrite(id.pin,1);
//-------------------------------------------------------------------------------------------------
#endif
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//						7,8.Архитектура STM32F4
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#if defined(RT_HW_ARCH_STM32F4)
//----------------------7.1.Основные параметры-----------------------------------------------------
#define RT_HW_ARCH_CODE 		RT_HW_ARCH_STM32F4_ID 			//--Код архитектуры;
#define RT_HW_ARCH_NAME			"STM32F4"  						//--Наименование архитектуры;
#define RT_HW_DATE_TYPE 		uint32_t 						//--Тип базовой переменной при прямом управлении пинами;
#define RT_HW_PIN_TXT_LEN 		4  								//--Размер поля при выводе на консоль номеров пинов;
#define RT_HW_PIN_QNT_LINE 		16 								//--Кол-во пинов в стройке при выводе таблицы на консоль; 
struct  RT_HW_PIN_DIR_ID {uint32_t pin; uint32_t dir; uint32_t mode; uint32_t bit; volatile uint32_t *base;};



//----------------------7.2.Подключение библиотек--------------------------------------------------
#include <avr/pgmspace.h>										//--Работа с PROGMEM;
//-------------------------
#if defined(RT_HW_ARCH_STM32DUINO)
#include "usbd_cdc_if.h"										//--Для работы с USB_CDC в среде STM32duino
#endif
//----------------------7.3.Макросы для хранения в PROGMEM-----------------------------------------
#define RT_HW_BUILD_BYTE_PGM(a) 	const byte    a[] PROGMEM  	//--Создание в PROGMEM массива byte 
#define RT_HW_BUILD_INT8_PGM(a) 	const int8_t  a[] PROGMEM  	//--Создание в PROGMEM массива int8_t 
#define RT_HW_READ_BYTE_PGM(arr,n) 	pgm_read_byte(arr+n)		//--Чтение из массива arr байта с номером n
//----------------------7.4.Макросы для работы c пинами через функции WIRING-----------------------
#define RT_HW_PIN_MODE_INPUT  				INPUT				//--Дискретный вход;
#define RT_HW_PIN_MODE_OUTPUT  				OUTPUT				//--Дискретный выход;
#define RT_HW_PIN_MODE_INPUT_PULLUP  		INPUT_PULLUP 		//--Дискретный вход с подтяжкой к +Vcc;
//----------------------7.5.Макросы для совместимости с STM32--------------------------------------
#define RT_HW_PIN_MODE_INPUT_FLOATING  		INPUT				//--Дискретный вход - аналог INPUT;
#define RT_HW_PIN_MODE_INPUT_PULLDOWN  		INPUT_PULLDOWN		//--Дискретный вход с подтяжкой к GND;
#define RT_HW_PIN_MODE_INPUT_ANALOG  		INPUT_ANALOG		//--Аналоговый вход;
#define RT_HW_PIN_MODE_OUTPUT_OPEN_DRAIN  	OUTPUT_OPEN_DRAIN	//--Дискретный выход с открытым выходом;
#define RT_HW_PIN_MODE_OUTPUT_PWM				PWM				//--ШИМ выход;
#define RT_HW_PIN_MODE_OUTPUT_PWM_OPEN_DRAIN	PWM_OPEN_DRAIN	//--ШИМ выход с открытым коллектором;
//----------------------7.6.Макросы прямого управления пинами--------------------------------------
#define RT_HW_PIN_DIR_SET_ID(id,p) 	  id.pin=p; id.bit=id.pin; 
#define RT_HW_PIN_DIR_MODE_INPUT(id)  pinMode(id.pin,INPUT);		//pin_function((PinName)id.pin, STM_PIN_DATA(STM_MODE_INPUT, GPIO_NOPULL,0))
#define RT_HW_PIN_DIR_MODE_PULLUP(id) pinMode(id.pin,INPUT_PULLUP);	//pin_function((PinName)id.pin, STM_PIN_DATA(STM_MODE_INPUT, GPIO_NOPULL,0))
#define RT_HW_PIN_DIR_MODE_OUTPUT(id) pinMode(id.pin,OUTPUT);		//pin_function((PinName)id.pin, STM_PIN_DATA(STM_MODE_OUTPUT_PP, GPIO_NOPULL,0))
#define RT_HW_PIN_DIR_READ(id)        digitalRead(id.pin);			//digitalReadFast((PinName)id.pin);  //digitalRead(id.pin);
#define RT_HW_PIN_DIR_WRITE(id,val)   digitalWrite(id.pin,val);		//if(val){digitalWriteFast((PinName)id.pin, HIGH);} else {digitalWriteFast((PinName)id.pin,LOW);} 
#define RT_HW_PIN_DIR_WRITE_LOW(id)   digitalWrite(id.pin,0);		//digitalWriteFast((PinName)id.pin, LOW); 	//digitalWrite(id.pin,0);
#define RT_HW_PIN_DIR_WRITE_HIGH(id)  digitalWrite(id.pin,l);		//digitalWriteFast((PinName)id.pin, HIGH);	//digitalWrite(id.pin,1);
//-------------------------------------------------------------------------------------------------
#endif
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//						9.Архитектура STM32F7
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#if defined(RT_HW_ARCH_STM32F7)
//----------------------9.1.Основные параметры-----------------------------------------------------
#define RT_HW_ARCH_CODE 			RT_HW_ARCH_STM32F7_ID 		//--Код архитектуры;
#define RT_HW_ARCH_NAME				"STM32F7" 					//--Наименование архитектуры;
#define RT_HW_DATE_TYPE 			uint32_t 					//--Тип базовой переменной при прямом управлении пинами;
#define RT_HW_PIN_TXT_LEN 			4  							//--Размер поля при выводе на консоль номеров пинов;
#define RT_HW_PIN_QNT_LINE 			16 	                        //--Кол-во пинов в стройке при выводе таблицы на консоль; 
struct  RT_HW_PIN_DIR_ID {uint32_t pin; uint32_t dir; uint32_t mode; uint32_t bit; volatile uint32_t *base;};
//----------------------9.2.Подключение библиотек--------------------------------------------------
#include <avr/pgmspace.h>										//--Работа с PROGMEM;
#if defined(RT_HW_ARCH_STM32DUINO)
#include "usbd_cdc_if.h"										//--Для работы с USB_CDC
#endif
//----------------------9.3.Макросы для хранения в PROGMEM-----------------------------------------
#define RT_HW_BUILD_BYTE_PGM(a) 	const byte    a[] PROGMEM  	//--Создание в PROGMEM массива byte 
#define RT_HW_BUILD_INT8_PGM(a) 	const int8_t  a[] PROGMEM  	//--Создание в PROGMEM массива int8_t 
#define RT_HW_READ_BYTE_PGM(arr,n) 	pgm_read_byte(arr+n)		//--Чтение из массива arr байта с номером n

//----------------------9.4.Макросы для работы c пинами через функции WIRING-----------------------
#define RT_HW_PIN_MODE_INPUT  				INPUT				//--Дискретный вход;
#define RT_HW_PIN_MODE_OUTPUT  				OUTPUT				//--Дискретный выход;
#define RT_HW_PIN_MODE_INPUT_PULLUP  		INPUT_PULLUP 		//--Дискретный вход с подтяжкой к +Vcc;
//----------------------9.5.Макросы для совместимости с STM32--------------------------------------
#define RT_HW_PIN_MODE_INPUT_FLOATING  		INPUT				//--Дискретный вход - аналог INPUT;
#define RT_HW_PIN_MODE_INPUT_PULLDOWN  		INPUT_PULLDOWN		//--Дискретный вход с подтяжкой к GND;
#define RT_HW_PIN_MODE_INPUT_ANALOG  		INPUT_ANALOG		//--Аналоговый вход;
#define RT_HW_PIN_MODE_OUTPUT_OPEN_DRAIN    OUTPUT_OPEN_DRAIN	//--Дискретный выход с открытым выходом;
#define RT_HW_PIN_MODE_OUTPUT_PWM				PWM				//--ШИМ выход;
#define RT_HW_PIN_MODE_OUTPUT_PWM_OPEN_DRAIN	PWM_OPEN_DRAIN	//--ШИМ выход с открытым коллектором;
//----------------------9.6.Макросы прямого управления пинами--------------------------------------
#define RT_HW_PIN_DIR_SET_ID(id,p) 	        id.pin=p; id.bit=id.pin; 
#define RT_HW_PIN_DIR_MODE_INPUT(id)        pinMode(id.pin,INPUT);			//pin_function((PinName)id.pin, STM_PIN_DATA(STM_MODE_INPUT,     GPIO_NOPULL,0))
#define RT_HW_PIN_DIR_MODE_PULLUP(id) 	    pinMode(id.pin,INPUT_PULLUP);	//pin_function((PinName)id.pin, STM_PIN_DATA(STM_MODE_INPUT,     GPIO_NOPULL,0))
#define RT_HW_PIN_DIR_MODE_OUTPUT(id) 	    pinMode(id.pin,OUTPUT);			//pin_function((PinName)id.pin, STM_PIN_DATA(STM_MODE_OUTPUT_PP, GPIO_NOPULL,0))
#define RT_HW_PIN_DIR_READ(id)              digitalRead(id.pin);			//digitalReadFast((PinName)id.pin);  //digitalRead(id.pin);
#define RT_HW_PIN_DIR_WRITE(id,val) 		digitalWrite(id.pin,val);		//if(val){digitalWriteFast((PinName)id.pin, HIGH);} else {digitalWriteFast((PinName)id.pin,LOW);} 
#define RT_HW_PIN_DIR_WRITE_LOW(id)         digitalWrite(id.pin,0);			//digitalWriteFast((PinName)id.pin, LOW); 	//digitalWrite(id.pin,0);
#define RT_HW_PIN_DIR_WRITE_HIGH(id)        digitalWrite(id.pin,l);			//digitalWriteFast((PinName)id.pin, HIGH);	//digitalWrite(id.pin,1);
//-------------------------------------------------------------------------------------------------
#endif
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//							90. Идентификация контроллера ANON (неопознанный контроллер)
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#if !defined(RT_HW_ARCH_CODE)  									//--Плата ANON;
//----------------------90.1.Основные параметры----------------------------------------------------
#define RT_HW_ARCH_ANON											//--Идентификация архитектуры платы;
#define RT_HW_ARCH_CODE				RT_HW_ARCH_ANON_ID
#define RT_HW_ARCH_NAME				"ARCH ANON" 				//--Наименование архитектуры;
#define RT_HW_DATE_TYPE 			uint16_t 					//--Наименование архитектуры;
#define RT_HW_PIN_TXT_LEN 			2 							//--Размер поля при выводе на консоль номеров пинов;
#define RT_HW_PIN_QNT_LINE 			24 	                     	//--Кол-во пинов в стройке при выводе таблицы на консоль; 
struct  RT_HW_PIN_DIR_ID {uint16_t pin; uint16_t dir; uint16_t mode; uint16_t bit; volatile uint16_t *base;}; 
//----------------------90.2.Подключение библиотек-------------------------------------------------
#include <avr/pgmspace.h>										//--Работа с PROGMEM;
//----------------------90.3.Макросы для хранения в PROGMEM----------------------------------------
#define RT_HW_BUILD_BYTE_PGM(a) 	const byte    a[] PROGMEM  	//--Создание в PROGMEM массива byte 
#define RT_HW_BUILD_INT8_PGM(a) 	const int8_t  a[] PROGMEM  	//--Создание в PROGMEM массива int8_t 
#define RT_HW_READ_BYTE_PGM(arr,n) 	pgm_read_byte(&arr[n])		//--Чтение из массива arr байта с номером n(для <64 kB);
//----------------------90.4.Макросы для работы c пинами через функции WIRING---------------------- 
#define RT_HW_PIN_MODE_INPUT  				INPUT				//--Дискретный вход;
#define RT_HW_PIN_MODE_OUTPUT  				OUTPUT				//--Дискретный выход;
#define RT_HW_PIN_MODE_INPUT_PULLUP  		INPUT_PULLUP 		//--Дискретный вход с подтяжкой к +Vcc;
//----------------------90.5.Макросы для совместимости с STM32-------------------------------------
#define RT_HW_PIN_MODE_INPUT_FLOATING  		INPUT				//--Дискретный вход - аналог INPUT;
#define RT_HW_PIN_MODE_INPUT_PULLDOWN  		INPUT				//--Дискретный вход с подтяжкой к GND;
#define RT_HW_PIN_MODE_INPUT_ANALOG  		INPUT				//--Аналоговый вход;
#define RT_HW_PIN_MODE_OUTPUT_OPEN_DRAIN  	 OUTPUT				//--Дискретный выход с открытым выходом;
#define RT_HW_PIN_MODE_OUTPUT_PWM				OUTPUT			//--ШИМ выход;
#define RT_HW_PIN_MODE_OUTPUT_PWM_OPEN_DRAIN	OUTPUT			//--ШИМ выход с открытым коллектором;
//----------------------90.6.Макросы прямого управления пинами-------------------------------------	
#define RT_HW_PIN_DIR_SET_ID(id,p) 	        id.pin=p; id.bit=id.pin; 
#define RT_HW_PIN_DIR_MODE_INPUT(id)        pinMode(id.pin,INPUT);			
#define RT_HW_PIN_DIR_MODE_PULLUP(id) 	    pinMode(id.pin,INPUT_PULLUP);
#define RT_HW_PIN_DIR_MODE_OUTPUT(id) 	    pinMode(id.pin,OUTPUT);
#define RT_HW_PIN_DIR_READ(id)              digitalRead(id.pin);
#define RT_HW_PIN_DIR_WRITE(id,val) 		digitalWrite(id.pin,val); 
#define RT_HW_PIN_DIR_WRITE_LOW(id)         digitalWrite(id.pin,0);
#define RT_HW_PIN_DIR_WRITE_HIGH(id)        digitalWrite(id.pin,l);
//-------------------------------------------------------------------------------------------------
#endif






//#if defined(RT_HW_ARCH_STM32) 
//#include "usbd_cdc_if.h"
//#endif
//**************************************************************************************************
/*
#define INPUT         0x0
#define OUTPUT              0x1
#define INPUT_PULLUP        0x2
///STM32 extension ///
#define INPUT_FLOATING      INPUT
#define INPUT_PULLDOWN      0x3
#define INPUT_ANALOG        0x4
#define OUTPUT_OPEN_DRAIN   0x5
*/

//=================================================================================================