#include "RT_HW_BASE.h"
//=================================================================================================
//							СИСТЕМНЫЕ ФУНКЦИИ
//=================================================================================================
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
//	 			Возвращает первый свободный бит регистра reg (используется для PWM ESP32);
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
uint8_t  RT_HW_BASE::getFistBitUint16(uint16_t &reg){for(uint8_t i=0;i<=15;i++){if(!bitRead(reg,i)) return i;} return -1;};	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
//	 			Возвращает максимальнок число соответствующее его разрядности(кол-во=v);
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
uint16_t RT_HW_BASE::maxBitDepth(uint8_t  v){if(v>=16) {return 0xFFFF;} if(v==0){return 0;} return ((1<<v)-1);};	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
//	 			Нормализация значения ADC до системного значения;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
uint16_t RT_HW_BASE::normADC  	(uint16_t v){return map(v,0,maxBitDepth(device.depth.adc),0,maxBitDepth(device.depth.sys));};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
//	 			Нормализация системного значения до значения записываемого в PWM;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
uint16_t RT_HW_BASE::normPWM  	(uint16_t v){return map(v,0,maxBitDepth(device.depth.sys),0,maxBitDepth(device.depth.pwm));};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
//	 			Нормализация истемного значения до значения записываемого в DAC;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
uint16_t RT_HW_BASE::normDAC  	(uint16_t v){return map(v,0,maxBitDepth(device.depth.sys),0,maxBitDepth(device.depth.dac));};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
//	Возвращает входное значение с поправкой на гистерезис
//  v1 - предыдущее значение;
//  v2 - новое значение
//  delta - допустимое отклонение;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
uint16_t RT_HW_BASE:: makeDelta(uint16_t v1, uint16_t v2, uint8_t delta){
if(v2>v1) {if((v2-v1)>delta) {return v2;}} 
if(v1>v2) {if((v1-v2)>delta) {return v2;}} 
return v1;}  //--Гистерезис двух величин;

//=================================================================================================
//							ФУНКЦИИ ДЛЯ РАБОТЫ С ПИНАМИ
//=================================================================================================
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
//  Получение текстового обозначения типа массива (массивы с перечнем основных пинов);
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
String RT_HW_BASE:: getNameKindPin(uint8_t kind){
//-------------------------------------------------------------------------------------------------
if(kind==RT_HW_PGM_SYS_PAR_ID) {return String(F("par"));}	
if(kind==RT_HW_PGM_SYS_ADC_ID) {return String(F("adc"));}
if(kind==RT_HW_PGM_SYS_TCH_ID) {return String(F("tch"));}	
if(kind==RT_HW_PGM_SYS_PWM_ID) {return String(F("pwm"));}
if(kind==RT_HW_PGM_SYS_DAC_ID) {return String(F("dac"));}
//-------------------------------------------------------------------------------------------------
if(kind==RT_HW_PGM_SYS_PIN_ID) {return String(F("pin"));}
if(kind==RT_HW_PGM_PIN_I2C_ID) {return String(F("I2C"));}
if(kind==RT_HW_PGM_PIN_SPI_ID) {return String(F("SPI"));}
if(kind==RT_HW_PGM_PIN_UHS_ID) {return String(F("UHS"));}	
if(kind==RT_HW_PGM_PIN_USS_ID) {return String(F("USS"));}	
if(kind==RT_HW_PGM_PIN_CAN_ID) {return String(F("CAN"));}
if(kind==RT_HW_PGM_PIN_BTH_ID) {return String(F("BTH"));}
if(kind==RT_HW_PGM_PIN_ETH_ID) {return String(F("ETH"));}
if(kind==RT_HW_PGM_PIN_WFI_ID) {return String(F("WFI"));}
//-------------------------------------------------------------------------------------------------
if(kind==RT_HW_PGM_PIN_ALL_ID) {return String(F("ALL"));}	
if(kind==RT_HW_PGM_PIN_DIN_ID) {return String(F("DIN"));}
if(kind==RT_HW_PGM_PIN_ADC_ID) {return String(F("ADC"));}
if(kind==RT_HW_PGM_PIN_TCH_ID) {return String(F("TCH"));}	
if(kind==RT_HW_PGM_PIN_DOT_ID) {return String(F("DOT"));}
if(kind==RT_HW_PGM_PIN_PWM_ID) {return String(F("PWM"));}
if(kind==RT_HW_PGM_PIN_DAC_ID) {return String(F("DAC"));}
if(kind==RT_HW_PGM_PIN_INT_ID) {return String(F("INT"));} 
if(kind==RT_HW_PGM_PIN_N5V_ID) {return String(F("N5V"));}
//-------------------------------------------------------------------------------------------------
if(kind==RT_HW_PGM_PIN_DBG_ID) {return String(F("DBG"));}
if(kind==RT_HW_PGM_PIN_OUT_ID) {return String(F("OUT"));}
if(kind==RT_HW_PGM_PIN_INP_ID) {return String(F("INP"));} 
if(kind==RT_HW_PGM_PIN_DCT_ID) {return String(F("DCT"));}
if(kind==RT_HW_PGM_PIN_DEV_ID) {return String(F("DEV"));}
if(kind==RT_HW_PGM_PIN_CSS_ID) {return String(F("CSS"));}	
if(kind==RT_HW_PGM_PIN_LC6_ID) {return String(F("LC6"));}	
if(kind==RT_HW_PGM_PIN_SSP_ID) {return String(F("SSP"));}
if(kind==RT_HW_PGM_ADR_I2C_ID) {return String(F("ADR"));}
//-------------------------------------------------------------------------------------------------	
return String(F(" - "));  };
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
//  	Получение значения из массива PGM;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
uint8_t RT_HW_BASE::readArrPGM(uint8_t kind, uint8_t n){
//-----Системные параметры-------------------------------------------------------------------------
if(kind==RT_HW_PGM_SYS_PAR_ID)	{return RT_HW_READ_BYTE_PGM(RT_HW_PGM_SYS_PAR,n);}
if(kind==RT_HW_PGM_SYS_ADC_ID)	{return RT_HW_READ_BYTE_PGM(RT_HW_PGM_SYS_ADC,n);}
if(kind==RT_HW_PGM_SYS_TCH_ID)	{return RT_HW_READ_BYTE_PGM(RT_HW_PGM_SYS_TCH,n);}
if(kind==RT_HW_PGM_SYS_PWM_ID)	{return RT_HW_READ_BYTE_PGM(RT_HW_PGM_SYS_PWM,n);}
if(kind==RT_HW_PGM_SYS_DAC_ID)	{return RT_HW_READ_BYTE_PGM(RT_HW_PGM_SYS_DAC,n);}
//-----Системные пины------------------------------------------------------------------------------
if(kind==RT_HW_PGM_SYS_PIN_ID)	{return RT_HW_READ_BYTE_PGM(RT_HW_PGM_SYS_PIN,n);}
//-----Пины интерфейсов связи----------------------------------------------------------------------
if(kind==RT_HW_PGM_PIN_I2C_ID)	{return RT_HW_READ_BYTE_PGM(RT_HW_PGM_PIN_I2C,n);}
if(kind==RT_HW_PGM_PIN_SPI_ID)	{return RT_HW_READ_BYTE_PGM(RT_HW_PGM_PIN_SPI,n);}
if(kind==RT_HW_PGM_PIN_UHS_ID)	{return RT_HW_READ_BYTE_PGM(RT_HW_PGM_PIN_UHS,n);}
if(kind==RT_HW_PGM_PIN_USS_ID)	{return RT_HW_READ_BYTE_PGM(RT_HW_PGM_PIN_USS,n);}
if(kind==RT_HW_PGM_PIN_CAN_ID)	{return RT_HW_READ_BYTE_PGM(RT_HW_PGM_PIN_CAN,n);}
if(kind==RT_HW_PGM_PIN_BTH_ID)	{return RT_HW_READ_BYTE_PGM(RT_HW_PGM_PIN_BTH,n);}
if(kind==RT_HW_PGM_PIN_ETH_ID)	{return RT_HW_READ_BYTE_PGM(RT_HW_PGM_PIN_ETH,n);}
if(kind==RT_HW_PGM_PIN_WFI_ID)	{return RT_HW_READ_BYTE_PGM(RT_HW_PGM_PIN_WFI,n);}
//-----Пины контроллера----------------------------------------------------------------------------
if(kind==RT_HW_PGM_PIN_ALL_ID)	{return RT_HW_READ_BYTE_PGM(RT_HW_PGM_PIN_ALL,n);}
if(kind==RT_HW_PGM_PIN_DIN_ID)	{return RT_HW_READ_BYTE_PGM(RT_HW_PGM_PIN_DIN,n);}
if(kind==RT_HW_PGM_PIN_ADC_ID)	{return RT_HW_READ_BYTE_PGM(RT_HW_PGM_PIN_ADC,n);}
if(kind==RT_HW_PGM_PIN_TCH_ID)	{return RT_HW_READ_BYTE_PGM(RT_HW_PGM_PIN_TCH,n);}
if(kind==RT_HW_PGM_PIN_DOT_ID)	{return RT_HW_READ_BYTE_PGM(RT_HW_PGM_PIN_DOT,n);}
if(kind==RT_HW_PGM_PIN_PWM_ID)	{return RT_HW_READ_BYTE_PGM(RT_HW_PGM_PIN_PWM,n);}
if(kind==RT_HW_PGM_PIN_DAC_ID)	{return RT_HW_READ_BYTE_PGM(RT_HW_PGM_PIN_DAC,n);}
if(kind==RT_HW_PGM_PIN_INT_ID)	{return RT_HW_READ_BYTE_PGM(RT_HW_PGM_PIN_INT,n);}
if(kind==RT_HW_PGM_PIN_N5V_ID)	{return RT_HW_READ_BYTE_PGM(RT_HW_PGM_PIN_N5V,n);}
//-----Тестовые пины-------------------------------------------------------------------------------
if(kind==RT_HW_PGM_PIN_DBG_ID)	{return RT_HW_READ_BYTE_PGM(RT_HW_PGM_PIN_DBG,n);}
if(kind==RT_HW_PGM_PIN_OUT_ID)	{return RT_HW_READ_BYTE_PGM(RT_HW_PGM_PIN_OUT,n);}
if(kind==RT_HW_PGM_PIN_INP_ID)	{return RT_HW_READ_BYTE_PGM(RT_HW_PGM_PIN_INP,n);}
if(kind==RT_HW_PGM_PIN_DCT_ID)	{return RT_HW_READ_BYTE_PGM(RT_HW_PGM_PIN_DCT,n);}
if(kind==RT_HW_PGM_PIN_DEV_ID)	{return RT_HW_READ_BYTE_PGM(RT_HW_PGM_PIN_DEV,n);}
if(kind==RT_HW_PGM_PIN_CSS_ID)	{return RT_HW_READ_BYTE_PGM(RT_HW_PGM_PIN_CSS,n);}
if(kind==RT_HW_PGM_PIN_LC6_ID)	{return RT_HW_READ_BYTE_PGM(RT_HW_PGM_PIN_LC6,n);}
if(kind==RT_HW_PGM_PIN_SSP_ID)	{return RT_HW_READ_BYTE_PGM(RT_HW_PGM_PIN_SSP,n);}
//-----Тестовые адреса устройств i2c---------------------------------------------------------------
if(kind==RT_HW_PGM_ADR_I2C_ID)	{return RT_HW_READ_BYTE_PGM(RT_HW_PGM_ADR_I2C,n);}
//-------------------------------------------------------------------------------------------------
return 0;}	
//-------------------------------------------------------------------------------------------------

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//   Проверяет наличие пина в массиве PGM
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool RT_HW_BASE:: checkPinPGM(uint8_t kind, uint8_t n){int8_t k=0;
for(uint8_t i=0;i<=127;i++){k=readArrPGM(kind,i); 
                            if(k==RT_HW_PGM_END){return  0;} 
                            if(k==n){return 1;}} 
return 0;};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//   Возвращает значение int8_t  из массива PGM по индексу. Если n>длины массива,то возвращается -1;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
int8_t RT_HW_BASE:: getIntPGM	 (uint8_t kind, uint8_t n){uint8_t len;
len=getLenFullPGM(kind); if(n>=len) {return -1;}  
return (int8_t)readArrPGM(kind,n);} 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//   Возвращает значение int8_t  из массива PGM по индексу. Если n>длины массива,то возвращается 0;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
uint8_t RT_HW_BASE:: getUintPGM   (uint8_t kind, uint8_t n){uint8_t len;
len=getLenFullPGM(kind); if(n>=len) {return -0;} 
return (uint8_t)readArrPGM(kind,n);};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//   Возвращает значение uint16_t  из массива PGM по индексу. Если n>длины массива,то возвращается 0;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
uint16_t RT_HW_BASE:: getValPGM	 (uint8_t kind, uint8_t n){uint8_t len;
len=getLenFullPGM(kind); if((n+1)>=len) {return 0;} 
return (((uint8_t)readArrPGM(kind,n)<<8) + ((uint8_t)readArrPGM(kind,n+1)));} 
//-------------------------------------------------------------------------------------------------


//=================================================================================================
//								ДОПОЛНИТЕЛЬНЫЕ ФУНКЦИИ
//=================================================================================================
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//   Возвращает размер массива PGM
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
uint8_t RT_HW_BASE:: getLenFullPGM(uint8_t kind){int8_t k; uint8_t cnt=0;
for(uint8_t i=0;i<=127;i++){k=(int8_t)readArrPGM(kind,i); if(k==RT_HW_PGM_END){break;}           cnt++;} return cnt;};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//   Возвращает кол-ва положительных  значений(0-127) в массиве PGM;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
uint8_t RT_HW_BASE:: getLenPosPGM(uint8_t kind){int8_t k; uint8_t cnt=0;
for(uint8_t i=0;i<=127;i++){k=(int8_t)readArrPGM(kind,i); if(k==RT_HW_PGM_END){break;} if(k>=0) {cnt++;}} return cnt;};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	var_ID: Возвращает цифру  из выводимой переменной начиная со старшей позиции
//  val - значение числа; len - число значащих цифр; mode -формат D,H,B; 
//  n   - номер позиции с младшей цифры 1,2...10;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
uint8_t RT_HW_BASE:: getNumberVal(uint32_t val, char mode, uint8_t len, uint8_t n){uint32_t buff32=0;
if((n>len) || (n>10)) n=0; 
if(mode=='B') {buff32=(val>>(len-n))&1;}
if(mode=='H') {buff32=(val>>(len-n)*4)&0xf;}
if(mode=='D') {n=len-n+1; while(n!=0) {buff32=val; val/=10; buff32=buff32-(val*10); n--;};}
//if(buff32<=9) {buff32=buff32 +'0';} else {buff32=buff32-10+(upper ? 'A' : 'a') ;}
return (uint8_t)buff32;}; 
//-------------------------------------------------------------------------------------------------