#include "RT_HW_BASE.h"	
//=================================================================================================
/**************************************************************************************************
1. Форматированный вывода int32_t (форматы сhar->D,C,S,T,I,U,H,B,1,2,3,4)
   Функция printVal  (char mode, uint8_t sz, char suff,  int32_t val);
   1.1.mode - формат вывода int32_t (форматы сhar->D,C,S,T,I,U,H,B,1,2,3,4):
	'D' - добавление впереди пробела, если число <9;
	'C'	- преобразование числа в символ;
	'S'	- вывод числа как есть (Serial.print(val);
	'T'	- вывод числа           с добавлением    нулей до размера поля sz;
	'I' - вывод числа со знаком с добавлением пробелов до размера поля sz;
	'U' - вывод числа без знака с добавлением пробелов до размера поля sz;
	'H' - вывод HEX числа с префиксом 0x и добав.нулей до размера поля sz;
	'h' - вывод HEX числа с префиксом 0x и добав.нулей до размера поля 2 (младшие два байта val);
	'B' - вывод BIN числа с префиксом b  и добав.нулей до размера поля sz;
	'b' - вывод BIN числа с префиксом b  и добав.нулей до размера поля 8 (младшие два байта val);;
	'0' - вывод числа	    в формате float без дробной части;
	'1' - вывод числа/10    в формате float c 1-м  знаком  после запятой;
	'2' - вывод числа/100   в формате float c 2-мя знаками после запятой;
	'3' - вывод числа/1000  в формате float c 3-мя  знаком после запятой;
	'4' - вывод числа/10000 в формате float c 4-мя знаками после запятой;
   1.2.sz - размер выводимого поля. Применяется в форматах T,I,U,H,B,0,1,2,3,4.
	   Если sz=0, то устанавливаются следующие значения:
	   'T'->2, 'I'->6, 'U'->5, 'H'->4, 'B'->8, '0','1','2','3','4'->6;
   1.3.suff - суффикс в конце вывода переменной. Вывод осуществляется функцией printSuff(char):
       '~' - ничего не выводится;	's' - выводится пробел; 'Y' - выполняется перевод строки;
	   'любой другой символ' - выводится этот символ.
   1.4. val - переменная формата int32.
		При форматах вывода 'U','T','H','B' выполняется 
		внутреннее преобразование входной переменной в uint32_t.

2. Форматированный вывод текста из PROGMEM (String(F("Текст"))с поддержкой кодировки UTF-8(в т.ч.русский шрифт).
   Функция printTextPGM(const char *text, uint8_t len, char alig, char blank, char suff);
	2.1.const char *text - указатель на текст определенный в памяти типа PROGMEM;
	    Если первый символ текста `~', то вывод заголовка не осуществляется.
	2.2.len - длина выводимого поля. 
		Выводимый текст обрезается сзади, или дополняется заполнителями до длины len.
		Если len=0, текст выводится как есть.
	2.3.alig - выравнивание текста в выводимом поле:
		'L' - влево, 'C' - влево, 'R' - вправо.
	2.4.blank - символ заполнителя поля;
	2.4.suff - суффикс в конце вывода текста. Вывод осуществляется функцией printSuff(char):
       '~' - ничего не выводится;	's' - выводится пробел; 'Y' - выполняется перевод строки;
	   'любой другой символ' - выводится этот символ.   	
***************************************************************************************************/
//=================================================================================================
//		1. Инициализация консоли
//=================================================================================================
void RT_HW_BASE:: consoleBegin(int8_t numUart, bool permit) {
if(console.dir==3) {return;}
if(console.dir==0) {if(!permit){return;}			
  console.numUart=uint8_t(numUart);
  if(!uartCheckNum(console.numUart)){console.dir=3; return;}; //--Выход, не доступный номер UART;	
  if(console.numUart==9) {uartBegin (console.numUart,RT_HW_UART_SPEED_SS);} //--Установка скорости для программного UART;
	else				 {uartBegin (console.numUart,RT_HW_UART_SPEED_HS);} //--Установка скорости для паппаратного UART;
  console.cnt=0; 
  console.dir++; }											//--Сброс параметров, переход на след.этап;
if(console.dir==1) {if(uartSerial(console.numUart)) { console.ready=console.head=1; 
					consoleCR();
					//for(uint8_t i=0; i<50;i++){consoleCR();}						
					console.dir++; }} //--Периодическая проверка соединения с UART 
if(console.dir==2) {if(console.resetHead || (++console.cnt>10)){console.head=0; console.mess=1; console.dir++;}};
}; 		//--Ожидание сброса или 10 циклов loop(); 		
//=================================================================================================
//		2. Базовые функции вывода на консоль
//=================================================================================================
bool RT_HW_BASE::consoleCheck() 		  {return console.ready;};								//--Проверка готовности коносли;
void RT_HW_BASE::consoleWrite(uint8_t val){if(console.ready){uartWrite(console.numUart,val);}}; //--Вывод байта;
void RT_HW_BASE::consolePrint(String str) {if(console.ready){uartPrint(console.numUart,str);}}; //--Вывод строки;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		3. Вывод суффикса	
// ~	 				- ничего не выводится; 
//		 N(nothing)	    - ничего не выводится; 
// ;	 S(semicolon)	- точка с запятой;
// .	 D(dot) 		- точка;
// ,	 V(virgale) 	- запятая;
//пробел G(gap)			- пробел;
// -	 H(hyphen)		- дефис;
// 		 L(line)		- перевод строки;
// 		 E(end)			- точка с запятой с последующим переводом строки;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_BASE::consoleSuff (char suff)  {if(consoleCheck()){
	if(suff=='~'){											return;} 
	if(suff=='G'){consoleWrite(' ');  						return;} 
	if(suff=='L'){consoleWrite('\n'); 						return;}; 			
	if(suff=='E'){consoleWrite(';'); consoleWrite('\n'); 	return;} 
	consoleWrite(suff);
}};										   
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		4.1. Перевод строки
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_BASE::consoleCR  (char suff)       {consoleWrite('\n'); consoleSuff(suff);};										  
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		4.2. Вывод символа	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_BASE::consoleChar(char v,char suff){if(consoleCheck()){consoleWrite(v);consoleSuff(suff);}}; 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		4.5. Вывод bool	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_BASE::consoleBool(bool v,char sign0,char sign1,char suff){if(v){consoleWrite(sign1);} else{consoleWrite(sign0);} consoleSuff(suff);};	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		4.6. Вывод n символов (заполнителей)	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_BASE:: consoleBlank(uint8_t n,char blank,char suff) {if(blank==RT_HW_CODE_CHAR_BLANK) {blank=' ';} 
	for(int i=0; i<n; i++){consoleWrite(blank);} consoleSuff(suff);}; 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		4.6. Вывод горизонтальной линии	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_BASE:: ConsoleLine(uint8_t n,char blank) {for(int i=0; i<n; i++){consoleWrite(blank);} consoleCR();}; 	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		4.7. Вывод числа int
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_BASE:: consoleInt8 (int8_t  v, char mode, char suff) {if(mode=='I'){consolePrint(String(v));} else {consolePrint(String((uint8_t)v));}  consoleSuff(suff);};
void RT_HW_BASE:: consoleInt16(int16_t v, char mode, char suff) {if(mode=='I'){consolePrint(String(v));} else {consolePrint(String((uint16_t)v));} consoleSuff(suff);};
void RT_HW_BASE:: consoleInt32(int32_t v, char mode, char suff) {if(mode=='I'){consolePrint(String(v));} else {consolePrint(String((uint32_t)v));} consoleSuff(suff);};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		4.8. Вывод числа float
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_BASE:: consoleFloat(float v, char mode, char suff) {
if(mode=='0') {consolePrint(String(v,0));} 
if(mode=='1') {consolePrint(String(v,1));} 
if(mode=='2') {consolePrint(String(v,2));} 
if(mode=='3') {consolePrint(String(v,3));} 	
if(mode=='4') {consolePrint(String(v,4));} 
consoleSuff(suff);};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		4.9. Вывод строковой переменной
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_BASE:: consoleStr (String &v,char suff){consolePrint(v); consoleSuff(suff);};
void RT_HW_BASE:: consoleText(String  v,char suff){consolePrint(v); consoleSuff(suff);};
//=================================================================================================
//		5. Форматированный вывод
//=================================================================================================
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		5.1. Форматированный вывод int32_t (форматы сhar->D,C,S,T,I,U,H,B,1,2,3,4) 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_BASE:: ConsoleVal(char mode, uint8_t sz, char suff, int32_t val){uint8_t len; String str;
//------Вывод с добавление впереди пробела до 2-х знаков-------------------------------------------
if(mode=='D'){if((val>=0)&&(val<10)){consoleChar(' ');} consolePrint(String(val));}
//------Вывод одного символа-----------------------------------------------------------------------
if(mode=='C') {consoleWrite(char(val&0xFF));} 	
//------Простой вывод числа------------------------------------------------------------------------
if(mode=='S') {consolePrint(String(val));}
//------Вывод в формате времени последних (до sz значащих знаков) с добавлением ' ' до длины sz----
if(mode=='T') {	if(sz==0){sz=2;} if(sz>4){sz=4;} str=String((uint16_t)val); 
				len=str.length(); if(len>sz){str=str.substring(len-sz,len); len=sz;}	                     
				consoleBlank((sz-len),'0');  consolePrint(str);}
//------Вывод в INT-формате последних (до sz значащих знаков) с добавлением ' ' до длины sz-------
if(mode=='I') {	if(sz==0){sz=6;} if(sz>10){sz=10;} str=String( (int32_t)val); 
				len=str.length(); if(len>sz){str=str.substring(len-sz,len); len=sz;}	                      
				consoleBlank((sz-len),' '); consolePrint(str);} 
 //------Вывод в UINT-формате последних (до sz значащих знаков) с добавлением ' ' до длины sz-------
if(mode=='U') {	if(sz==0){sz=5;} if(sz>10){sz=10;} str=String((uint32_t)val); 
				len=str.length(); if(len>sz){str=str.substring(len-sz,len); len=sz;}	                      
				consoleBlank((sz-len),' '); consolePrint(str);} 
 //------Вывод в HEX-формате последних (до sz значащих знаков) с добавлением '0' до длины sz-------
if(mode=='H') {	if(sz==0){sz=4;} if(sz> 8){sz= 8;} str=String((uint32_t)val,HEX); str.toUpperCase(); len=str.length(); 
				if(len>sz){str=str.substring(len-sz,len); len=sz;}	
				consoleChar('0','x'); 
				consoleBlank((sz-len),'0'); consolePrint(str);} 
 //------Вывод в HEX-формате последних (до sz значащих знаков) с добавлением '0' до длины sz-------
if(mode=='h') {	sz= 4; str=String(((uint32_t)val&&0xFF),HEX); str.toUpperCase(); len=str.length(); 
				if(len>sz){str=str.substring(len-sz,len); len=sz;}	
				consoleChar('0','x'); 
				consoleBlank((sz-len),'0'); consolePrint(str);} 				
//------Вывод в BIN-формате. Вывод первых знаков('S'=32, 's'=16, 'B'=8, 'b'=4)---------------------
if(mode=='B') {	if(sz==0){sz=4;} if(sz>32){sz=32;} str=String((uint32_t)val,BIN); str.toUpperCase(); len=str.length(); 
				if(len>sz) {str=str.substring(len-sz,len); len=sz;}	
				consoleChar('b');     
				consoleBlank((sz-len),'0');  consolePrint(str);} 
//------Вывод в BIN-формате. Вывод первых знаков('S'=32, 's'=16, 'B'=8, 'b'=4)---------------------
if(mode=='b') {	sz=8;  str=String(((uint32_t)val&&0xFF),BIN); str.toUpperCase(); len=str.length(); 
				if(len>sz) {str=str.substring(len-sz,len); len=sz;}	
				consoleChar('b');     
				consoleBlank((sz-len),'0');  consolePrint(str);} 				
//------Вывод в формате с точкой-------------------------------------------------------------------
if((mode=='0') || (mode=='1') || (mode=='2') || (mode=='3') || (mode=='4')){str=String(float(val));
				if(mode=='0') {str=String(float(val),0);} 	
				if(mode=='1') {str=String(float(val)/10,1);} 									
				if(mode=='2') {str=String(float(val)/100,2);}
				if(mode=='3') {str=String(float(val)/1000,3);} 									
				if(mode=='4') {str=String(float(val)/10000,4);}
				if(sz==0){sz=6;} if(sz>32){sz=32;} 
				len=str.length(); if(len>sz){str=str.substring(len-sz,len); len=sz;}	                     
				consoleBlank((sz-len),'0');  consolePrint(str);}					
//-------Завершение вывода-------------------------------------------------------------------------
consoleSuff(suff);};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		5.3.Форматированный вывод float (форматы сhar->1,2,3,4) 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_BASE:: ConsoleFloat(char mode, uint8_t sz, char suff, float val){uint8_t len; String str;
if((mode=='0') || (mode=='1') || (mode=='2') || (mode=='3') || (mode=='4')){str=String(float(val));
				if(mode=='0') {str=String(float(val),0);} 	
				if(mode=='1') {str=String(float(val),1);} 									
				if(mode=='2') {str=String(float(val),2);}
				if(mode=='3') {str=String(float(val),3);} 									
				if(mode=='4') {str=String(float(val),4);}}
		else	{str=String(val,2);}
				if(sz==0){sz=6;} if(sz>32){sz=32;} 
				len=str.length(); if(len>sz){str=str.substring(len-sz,len); len=sz;}	                     
				consoleBlank((sz-len),'0');  consolePrint(str);					
//-------Завершение вывода-------------------------------------------------------------------------
consoleSuff(suff);};		//--Форматированный вывод суффикса;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		5.4. Форматированный вывод строковой переменной
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//void ConsoleText   (char mode, uint8_t sz, char suff, char blank, String str); 
void RT_HW_BASE:: ConsoleText(char mode, uint8_t sz, char suff, char blank, String str){uint8_t curLen=0;  
if(sz>160){sz=160;} 
if(sz>0) {str.substring(0,sz);} curLen= getLenStr(str, sz);               
consoleBlank(getLenBegin(sz,curLen,mode),blank); 
consolePrint(str);
consoleBlank(getLenEnd(sz,curLen,mode),blank); 
//-------Завершение вывода-------------------------------------------------------------------------
consoleSuff(suff);};		//--Форматированный вывод суффикса;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  	5.5. Форматированный вывод до 4-х int8_t.
//	Если suff='C' (continue), в конце добавляется разделитель запятая, перевод строки не осуществляется.
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void  RT_HW_BASE:: ConsoleV8x4(char mode, uint8_t sz, char suff, uint8_t qnt, int8_t v1, int8_t v2, int8_t v3, int8_t v4){//if(mode!='I') {mode='U';}
if(qnt>0){                  ConsoleVal(mode,sz,'~',v1);} 	if(qnt>1){consoleChar(','); ConsoleVal(mode,sz,'~',v2);} 
if(qnt>2){consoleChar(','); ConsoleVal(mode,sz,'~',v3);} 	if(qnt>3){consoleChar(','); ConsoleVal(mode,sz,'~',v4);} 	
if(suff=='C') {consoleChar(','); return;}	//--Вывод символа ',' и выход для продолжения вывода на текущей строке;
//-------Завершение вывода-------------------------------------------------------------------------
consoleSuff(suff);};		//--Форматированный вывод суффикса;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  	5.6. Форматированный вывод до 4-х int16_t
//	Если suff='C' (continue), в конце добавляется разделитель запятая, перевод строки не осуществляется.
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void  RT_HW_BASE:: ConsoleV16x4(char mode, uint8_t sz, char suff, uint8_t qnt, int16_t v1, int16_t v2, int16_t v3, int16_t v4){//if(mode!='I') {mode='U';}
if(qnt>0){                  ConsoleVal(mode,sz,'~',v1);} 	if(qnt>1){consoleChar(','); ConsoleVal(mode,sz,'~',v2);} 
if(qnt>2){consoleChar(','); ConsoleVal(mode,sz,'~',v3);} 	if(qnt>3){consoleChar(','); ConsoleVal(mode,sz,'~',v4);} 	
if(suff=='C') {consoleChar(','); return;}	//--Вывод символа ',' и выход для продолжения вывода на текущей строке;
//-------Завершение вывода-------------------------------------------------------------------------
consoleSuff(suff);};		//--Форматированный вывод суффикса;	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  	5.7. Форматированный вывод до 4-х int32_t
//	Если suff='C' (continue), в конце добавляется разделитель запятая, перевод строки не осуществляется.
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void  RT_HW_BASE:: ConsoleV32x4(char mode, uint8_t sz, char suff, uint8_t qnt, int32_t v1, int32_t v2, int32_t v3, int32_t v4){//if(mode!='I') {mode='U';}
if(qnt>0){                  ConsoleVal(mode,sz,'~',v1);} 	if(qnt>1){consoleChar(','); ConsoleVal(mode,sz,'~',v2);} 
if(qnt>2){consoleChar(','); ConsoleVal(mode,sz,'~',v3);} 	if(qnt>3){consoleChar(','); ConsoleVal(mode,sz,'~',v4);} 	
if(suff=='C') {consoleChar(','); return;}	//--Вывод символа ',' и выход для продолжения вывода на текущей строке;
//-------Завершение вывода-------------------------------------------------------------------------
consoleSuff(suff);};		//--Форматированный вывод суффикса;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		5.8. Форматированный вывод до 4-х float	c 2-знаками после запятой
//	Если suff='C' (continue), в конце добавляется разделитель запятая, перевод строки не осуществляется.
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void  RT_HW_BASE:: ConsoleFloatx4(char mode, uint8_t sz, char suff, uint8_t qnt,float v1, float v2, float v3, float v4){
if(qnt>0){                  ConsoleFloat(mode,sz,'~',v1);} 	if(qnt>1){consoleChar(','); ConsoleFloat(mode,sz,'~',v2);} 
if(qnt>2){consoleChar(','); ConsoleFloat(mode,sz,'~',v3);} 	if(qnt>3){consoleChar(','); ConsoleFloat(mode,sz,'~',v4);} 	
//-------Завершение вывода-------------------------------------------------------------------------
consoleSuff(suff);};		//--Форматированный вывод суффикса;	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	Вывод на консоль отформатированного вывод пина с суффиксом по его номеру
//		для ESP8266 - в формате D0-D8,RX,TX,A0;
//		для STM32	- в формату Pxx (PA0,PA15,PB0,PB15 и т.д.)
//	Если номер pin не допустим для контроллера, выводится символ '*'.
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
String  RT_HW_BASE:: getNamePin(int8_t pin){
#if defined(RT_HW_ARCH_ESP8266)	
return getNamePinESP8266(pin);			 
#elif defined(RT_HW_ARCH_STM32)			 
return getNamePinSTM32(pin);
#else
return String(pin);	
#endif		
};
//=================================================================================================
//		4. ВСПОМОГАТЕЛЬНЫЕ ФУНКЦИИ
//=================================================================================================
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		4.1. Конвертация bool в char------------------------------------------------------------ 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
char    RT_HW_BASE::boolToChar (bool val,char sign_0,char sign_1){if(val) return sign_1; else return sign_0;};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		4.2. Получение кол-ва заполнителей спереди (для форматированного вывода)----------------
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
uint8_t RT_HW_BASE::getLenBegin(uint8_t len,uint8_t src,char alig){if(len<=src) return 0;
		switch(alig){default:return 0; case 'C':return (len-src-(len-src)/2); case 'R':return (len-src);}};  
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		4.3. Получение кол-ва заполнителей сзади (для форматированного вывода)
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
uint8_t RT_HW_BASE::getLenEnd  (uint8_t len,uint8_t src,char alig){if(len<=src) return 0;
switch(alig){default:return 0; case 'C':return (len-src-getLenBegin(len,src,alig)); case 'L':return (len-src);}}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++		
//		4.4 Получение длины текста String (utf-8, блокирующий символ RT_HW_CODE_CHAR_BREAK '~')
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
uint8_t RT_HW_BASE:: getLenStr(String &str, uint8_t lenMax){char buff; uint8_t LEN=0; 
const char * data=str.c_str();	
for(uint8_t i=0; i<=160; i++){buff=data[i]; 
if((buff=='\0') || (buff==RT_HW_CODE_CHAR_BREAK) || (LEN>=lenMax)) {break;}
if(((uint8_t)buff)<0xC0) {LEN++;}}		   
return LEN;};

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++												   
//		4.5.Получение длины текста PROGMEM (utf-8, блокирующий символ RT_HW_CODE_CHAR_BREAK '~')
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
uint8_t RT_HW_BASE:: getLenPGM(const char *text, uint8_t lenMax){uint8_t buff,j,LEN; if(lenMax==0) {lenMax=80;} j=LEN=0;
while (1) {buff=pgm_read_byte(text+j);	j++;
		   if((buff==0)   || (buff==RT_HW_CODE_CHAR_BREAK) || (LEN>=lenMax) || (j>160)) {break;}
		   if( buff>=0xC0){j++; buff=pgm_read_byte(text+j);} LEN++;} return LEN;}	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		4.6.Получение кол-ва знаков в числах  с оcнованиями 10,16,2 (DEC(I,U),HEX,BIN)
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
uint8_t RT_HW_BASE:: getLenVal(char mode, int32_t val){
uint32_t v, buff; uint8_t len=0; uint8_t bs;
if(mode=='I') {if(val<0) {v=-val;} else {v=val;}	
								buff=bs=10;  len=1; for(int8_t i=0;i<10;i++) {if(v<buff){break;} buff=buff*bs; len++;} if(val<0) {len++;}} 
if(mode=='U') {v=(uint32_t)val; buff=bs=10;  len=1; for(int8_t i=0;i<10;i++) {if(v<buff){break;} buff=buff*bs; len++;}} 
if(mode=='H') {v=(uint32_t)val;	buff=bs=0xF; len=1; for(int8_t i=0;i<10;i++) {if(v<buff){break;} buff=buff*bs; len++;}} 
if(mode=='B') {v=(uint32_t)val;	buff=bs=0x1; len=1; for(int8_t i=0;i<10;i++) {if(v<buff){break;} buff=buff*bs; len++;}} 
return len;};	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
//  Вывод данных по массиву по его типу (kind) и выбранному режиму (mode).
//  Признак окончания массива RT_HW_PIN_PGM_END(=-2). 
//  Если последовательном переборе данных массива встретится специальное значение (признак онкончанич массива =-2, 254, 0xfe), 
//  то последующий перебор данных прекращается.
//  Режимы (A,B,U,C,D,S,P,N):
//   mode='A' ->  вывод всех элементов в формате int8_t  с форматированием;
//	 mode='B' ->  вывод всех элементов в формате uint8_t c форматированием;
//	 mode='U' ->  вывод всех элементов в формате uint8_t c форматированием до трех знаков (вывод байтов);
//	 mode='C' ->  вывод элементов 0-127, вместо остальных элементов выводятся заполнители;
//	 mode='D' ->  вывод элементов 0-127, остальные элементы не выводятся;
//   mode='S' ->  вывод всех элементов в формате int8_t без форматирования;
//   mode='P' ->  вывод в формате пинов(для ESP8266,STM32 в текстовой форме),вместо отсутствующих пинов(=-1) выводится заполнитель(*);
//   mode='N' ->  ничего не выводится;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
void RT_HW_BASE:: ConsoleArrPGM  (uint8_t kind, char mode){int8_t k; uint8_t len,sz; uint8_t cnt=0; uint8_t delim=0;
if(kind>RT_HW_PGM_QNT_ID) 	{return;}	//--Выход по несуществующему типу массива; 
//----Проверка допустимости символа kind-----------------------------------------------------------
char  bf[7]={'A','B','C','D','S','P','U'};
len=0; for(uint8_t i=0; i<7;i++){if(mode==bf[i]){len++;}} if(len==0) {return;}
//----Вычисление длины массива---------------------------------------------------------------------
len=getLenFullPGM(kind); 									//--Получение общей длины массива;
sz =getLenPosPGM(kind); 									//--Получение кол-ва положительных (>=0) значений в массиве;
if(len==0) 					{return;}						//--Выход, если нет массив пуст;
if(((mode=='D') || (mode=='P'))  && (sz==0)){return;}		//--Выход по свойствам режим 'D';
String str=getNameKindPin(kind); consoleStr(str); 			//--Вывод обозначения массива;
if(( mode=='D') || (mode=='P')) {k=sz;} else {k=len;}		//--Установка кол-ва рассматриваемых элементов в массиве;
consoleChar('['); ConsoleVal('D',0,']',k); consoleChar(':',' ');	//--Вывод размера массива;
for(uint8_t i=0;i<len;i++){									//--Цикл
	if(delim==1) {consoleChar(',');} 
	if(delim==2) {consoleChar(',');}						//--Вывод разделителя;
	k=(int8_t)readArrPGM(kind,i); 							//--Получения элемента по индексу i;	
	if(mode=='A') {           ConsoleVal('I',RT_HW_PIN_TXT_LEN,'~',         k);  delim=1; cnt++;}
	if(mode=='B') {           ConsoleVal('U',RT_HW_PIN_TXT_LEN,'~',(uint8_t)k);  delim=1; cnt++;}
	if(mode=='U') {           ConsoleVal('U',				 3,'~',(uint8_t)k);  delim=1; cnt++;}
	if(mode=='C') {if(k>=0)  {ConsoleVal('I',RT_HW_PIN_TXT_LEN,'~',         k);} else {consoleBlank((RT_HW_PIN_TXT_LEN-1),' ','*');} delim=1; cnt++;}
	if(mode=='D') {if(k>=0)  {ConsoleVal('I',RT_HW_PIN_TXT_LEN,'~',         k);  delim=1; cnt++;} else {delim=0;}}
	if(mode=='S') {           ConsoleVal('S',				   0,'~',       k);  delim=1; cnt++;}
	if(mode=='P') {//consoleText(getNamePin(k)); 
	ConsoleText('R',RT_HW_PIN_TXT_LEN,'~',' ',getNamePin(k)); 
	if(k>=0) {delim=1;} else {delim=2;} cnt++;}
	//if(k>=0)  {printVal('I',RT_HW_PIN_TXT_LEN,'~',         k);} else {printBlank(RT_HW_PIN_TXT_LEN,'-');} delim=1; cnt++;
	//-------------------------------------------
	if(mode=='D') {if((cnt>=RT_HW_PIN_QNT_LINE) || ((i+1)>=sz))  {cnt=delim=0; consoleChar(';'); consoleCR(); if((i+1)<sz) {consoleBlank(9);}}}
	else		  {if((cnt>=RT_HW_PIN_QNT_LINE) || ((i+1)>=len)) {cnt=delim=0; consoleChar(';'); consoleCR(); if((i+1)<len){consoleBlank(9);}}}
}};

					
/**************************************************************************************************
						ТЕКСТ ДЛЯ БЛОКОВ FLProg
Форматированный вывода int16_t c форматированным заголовком.(
1.Перед переменной выводится заголовок с форматом, определяемые параметрами пользователя:
	1.1.ЗАГОЛОВОК - текст UTF-8(в т.ч.русский шрифт), который размещается в памяти PROGMEM.
	    Если первый символ текста `~', то вывод заголовка не осуществляется.
	1.2.ДЛИНА_ЗАГОЛОВКА - длина выводимого поля вместе текстом ЗАГОЛОВОК. 
		Выводимый текст обрезается сзади, или дополняется сзади пробелами до этой длины.
		Если ДЛИНА_ЗАГОЛОВКА=0, текст ЗАГОЛОВКА выводится как есть.
	1.3.СУФФИКС_ЗАГОЛОВКА - действие после вывода поля с ЗАГОЛОВКОМа:
		'~'-ничего не выводится; 's'-выводится пробел; 'Y'-выполняется перевод строки;
	  	'любой другой символ'-выводится этот символ.
2.После заголовка выводится переменная  форматом, определяемые параметрами пользователя:
	2.1.ФОРМАТ_ПЕРЕМЕННОЙ - (D,C,S,T,I,U,H,B,1,2,3,4):
	'D' - добавление впереди пробела, если число <9;
	'C'	- преобразование числа в символ;
	'S'	- вывод числа как есть (Serial.print(val);
	'T'	- вывод числа           с добавлением    нулей до размера поля sz;
	'I' - вывод числа со знаком с добавлением пробелов до размера поля sz;
	'U' - вывод числа без знака с добавлением пробелов до размера поля sz;
	'H' - вывод HEX числа с префиксом 0x и добав.нулей до размера поля sz;
	'h' - вывод HEX числа с префиксом 0x и добав.нулей до размера поля 2 (младшие два байта val);
	'B' - вывод BIN числа с префиксом b  и добав.нулей до размера поля sz;
	'b' - вывод BIN числа с префиксом b  и добав.нулей до размера поля 8 (младшие два байта val);;
	'0' - вывод числа	    в формате float без дробной части;
	'1' - вывод числа/10    в формате float c 1-м  знаком  после запятой;
	'2' - вывод числа/100   в формате float c 2-мя знаками после запятой;
	'3' - вывод числа/1000  в формате float c 3-мя  знаком после запятой;
	'4' - вывод числа/10000 в формате float c 4-мя знаками после запятой;	
   2.2.РАЗМЕР_ПОЛЯ_ПЕРЕМЕННОЙ - размер выводимого поля вместе с переменной.
	   Применяется в форматах T,I,U,H,B. Если РАЗМЕР_ПОЛЯ_ПЕРЕМЕННОЙ =0, 
	   то устанавливаются следующие значения: 'T'->2, 'I'->6, 'U'->5, 'H'->4, 'B'->8.
   2.3.СУФФИКС_ПЕРЕМЕННОЙ - суффикс в конце вывода переменной. 
       '~'-ничего не выводится; 's'-выводится пробел; 'Y'-выполняется перевод строки;
	   'любой другой символ' - выводится этот символ.
   2.4. val - переменная формата int32.
		При форматах вывода 'U','T','H','B' выполняется 
		внутреннее преобразование входной переменной в uint32_t.
3.Разрешение вывода регулируется флагами консоли, параметром пользователя, внешним разрешением EN.
	3.1.флаг head - устанавливается консолью после инициализации UART.
		Это особенно важно для контроллеров DUE,STM32 без аппаратного чипа UART-USB.
	    Флаг head сбрасывается через программную установку флага reset, 
		или самостоятельно через 10 циклов loop().
	3.2.флаг mess - уставливается после установки и сброса флага head.
	3.3.параметр пользователя УСЛОВИЕ ВЫВОДА:
		'C'-вывод осуществляется по флагу mess при первом вхождении в блок, 
			  далее по изменению значения переменной.
		'P'-вывод осуществляется по флагу mess при каждом вхождении в блок.
		'H'-вывод осуществляется по флагу head.		
	3.4.вывод осуществляется только при разрешение EN=1.
	    Вход EN можно использовать для циклического вывода при УСЛОВИЕ ВЫВОДА='P', для чего
		на вход EN периодически подается "Событие" (HIGH в течении одного цикла loop()). 
---------------------------------------------------------------------------------------------------
ecoins@mail.ru 19.11.2020	   	
***************************************************************************************************/
