#include "RT_HW_LCD_HD44780.h"
//=====================================================================================================================
// 						I.ФУНКЦИИ ДЛЯ РАБОТЫ С УСТРОЙСТВОМ
//=====================================================================================================================
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	1.1. Конструктор
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
RT_HW_LCD_HD44780::RT_HW_LCD_HD44780(){device.qnt=0; device.i2c_ok=device.spi_ok=false;}; 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	1.2. Настройка параметров устройства
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
uint8_t RT_HW_LCD_HD44780:: setDevice(uint8_t num, char mode, int8_t addr, uint8_t cols, uint8_t rows, uint8_t period, 
                                       int8_t pRS, int8_t pWR,int8_t pDB4, int8_t pDB5, int8_t pDB6, int8_t pDB7){
if(num!=15) 								 {return num;};			//--Выход-устройство уже настроено; 
if(device.qnt>=RT_HW_LCD_HD44780_MAX_DEVICE)  {num=14; return num;}  //--Выход-нет свободных номеров (все устройства заняты);
//if (addr<0) 								 {num=13; return num;}; //--Недопустимый адрес (номер пина);
if((mode=='I') || (mode=='S') || (mode=='B') || (mode=='P')) {num=device.qnt++; lcd[num].num=num;	lcd[num].custom=1;} else {num=12; return num;}	//--Установка номера устройства или неверный код интерфейса; 
lcd[num].addr=addr; //--Сохранение 1)адреса для i2c; 2)ss для SPI; 3)первый пин 4-х битовой шины (RS,WR,D4,D5,D6,D7); 4)три пина через HC595;
if(cols>RT_HW_LCD_HD44780_MAX_COLS) {lcd[num].cols=RT_HW_LCD_HD44780_MAX_COLS;} else {lcd[num].cols=cols;}	//--Установка кол-во колонок;	
if(rows>RT_HW_LCD_HD44780_MAX_ROWS) {lcd[num].rows=RT_HW_LCD_HD44780_MAX_ROWS;} else {lcd[num].rows=rows;}	//--Установка кол-во строк;					
lcd[num].period=period;																						//--Установка периода обновления  lcd;
lcd[num].stepInit=lcd[num].stepDirect=lcd[num].stepFind=lcd[num].stepSend=0;								//--Очистка шагов;	
//----Настройка интерфейса i2c--------------------------------------------
if(mode=='I'){if(addr<0) {num=13; return num;}; lcd[num].mode=1;  if(!device.i2c_ok) {device.i2c_ok=true; Wire.begin();}}
//----Настройка интерфейса SPI--------------------------------------------
if(mode=='S'){if(addr<0) {num=13; return num;}; lcd[num].mode=2;  digitalWrite(lcd[num].addr,true); pinMode(lcd[num].addr,OUTPUT);
			  if(!device.spi_ok) {device.spi_ok=true; SPI.begin(); digitalWrite(lcd[num].addr,false); SPI.transfer(0); digitalWrite(lcd[num].addr,true);}}
//----Настройка интерфейса - параллельный через 6 пинов-------------------
if(mode=='B'){if(addr<0) {num=13; return num;}; lcd[num].mode=3; 
    for(uint8_t i=0; i<6; i++) {lcd[num].pins[i]=lcd[num].addr+i; 
							    digitalWrite(lcd[num].pins[i],0); pinMode(lcd[num].pins[i],OUTPUT);}}
//----Настройка интерфейса - параллельный через 6 пинов-------------------
if(mode=='P'){lcd[num].mode=4; lcd[num].pins[0]=pRS; lcd[num].pins[1]=pWR; lcd[num].pins[2]=pDB4; lcd[num].pins[3]=pDB5; lcd[num].pins[4]=pDB6; lcd[num].pins[5]=pDB7;
   for(uint8_t i=0; i<6; i++) {for(uint8_t j=0; j<6; j++){if((i!=j) && (lcd[num].pins[i]==lcd[num].pins[j])) {num=13; return num;}}}
   for(uint8_t i=0; i<6; i++) {digitalWrite(lcd[num].pins[i],0); pinMode(lcd[num].pins[i],OUTPUT);}}
//------------------------------------------------------------------------
return num;};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
//	1.3. Генератор меандра 50мс и флага перезапуска устройства (periodReboot в тиках=50мс)
//   	Вход: lcd[i].period -период перезапуска устройства (если =0, то не перезапускается);
//   	Выход: 1)меандр 50мс(getState(num,bitBLINK); 2)флаг перезапуска устройства(getState(num,bitREBOOT); 
//  	Счетчик lcd[i].cnt используется  только когда устройство в работе(lcd[i].permit=1)'
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
void RT_HW_LCD_HD44780:: genEvent(){device.event=device.eventSec=false;
if(RT_HW_LCD_HD44780_MILLIS>=device.timeStart) {
device.event=true; device.blink=!device.blink; device.timeStart=RT_HW_LCD_HD44780_MILLIS+RT_HW_LCD_HD44780_PERIOD_TICK; 
if(++device.cntSec>20){device.cntSec=1; device.eventSec=1; device.blinkSec=!device.blinkSec;} 	
for(uint8_t i=0; i<device.qnt; i++){if(device.qnt>RT_HW_LCD_HD44780_MAX_DEVICE){device.qnt=RT_HW_LCD_HD44780_MAX_DEVICE;}
lcd[i].event=device.event; lcd[i].blink=device.blink;
if(device.eventSec && lcd[i].permit && lcd[i].period){if(lcd[i].cnt++ > lcd[i].period) {lcd[i].cnt=0; lcd[i].reboot=1;}}}}};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	1.4. Управление устройством
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_LCD_HD44780:: directDevice(uint8_t num){if(num>RT_HW_LCD_HD44780_MAX_DEVICE) {return;}
if(num==0) {genEvent();}	//--Генератор устройств. Вызывается только устройством с номером 1;
if(lcd[num].stepDirect==0) {if(lcd[num].custom) {lcd[num].timeDelay=RT_HW_LCD_HD44780_MILLIS; lcd[num].busy=1; 		//--Настройка таймера, флагов
                                                 lcd[num].permit=0; lcd[num].reboot=0; lcd[num].stepDirect++;}} 	//       и переход на следующий этап; 
if(lcd[num].stepDirect==1) {if(((RT_HW_LCD_HD44780_MILLIS-lcd[num].timeDelay)>50)) {lcd[num].stepDirect++;			//--Задержка 50мс (для нормализации питания устройства);
															  if(lcd[num].mode!=1) {lcd[num].stepDirect++;}}}		//--Пропуск этапа, если интерфейс не i2c; 
if(lcd[num].stepDirect==2) {if(findDeviceI2C(num,20)) {lcd[num].stepDirect++;}}										//--Поиск устройства на шине i2c;
if(lcd[num].stepDirect==3) {init(num);	     if(!lcd[num].link)  {lcd[num].stepDirect=0; return;}					//--Выполнение одного шага блока инициализации. Если потеря связи, переход на полную переинициализацию;  
							if( lcd[num].initOk){lcd[num].busy=0; lcd[num].permit=1; lcd[num].stepDirect++;}}		//--Переход на рабочий режим;
if(lcd[num].stepDirect==4) {if(!lcd[num].link) 	{lcd[num].busy=1; lcd[num].permit=0; lcd[num].stepDirect=0; return;}//--Если потеря связи, переход на полную переинициализацию; 				
	  if( lcd[num].reboot){lcd[num].busy=1; lcd[num].permit=0; lcd[num].reboot=0; lcd[num].stepDirect=3; return;}}};//--Если перезапуск, переход на частичную
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	1.5. Поиск устройства на шине i2c
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool RT_HW_LCD_HD44780:: findDeviceI2C(uint8_t num, uint8_t cntMax){
if(lcd[num].stepFind==0){lcd[num].cnt=0; lcd[num].timeDelay=RT_HW_LCD_HD44780_MILLIS; lcd[num].stepFind=1;}
if(lcd[num].stepFind==1){if(RT_HW_LCD_HD44780_MILLIS< lcd[num].timeDelay){return 0;} if(++lcd[num].cnt>cntMax) {lcd[num].cnt=cntMax;}
						 Wire.beginTransmission(lcd[num].addr); 
						 if(Wire.endTransmission()) {lcd[num].timeDelay=RT_HW_LCD_HD44780_MILLIS+100*lcd[num].cnt; return 0;}
						 if(lcd[num].cnt<2)         {lcd[num].timeDelay=RT_HW_LCD_HD44780_MILLIS+50;	           return 0;} 	//--Выход для повторного поиска (для надежности) или успешное завершение;
						 lcd[num].stepFind=0;	  														      	   return 1;}; return 0;};	//--Успешное завершение;			
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	1.6. Запись в устройство. 	mode:'D'-данные,'C'- байтовая команда,'T'-тетрадная (4-х битная) команда;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool RT_HW_LCD_HD44780:: sendDevice(uint8_t num, uint8_t val, uint8_t mode, uint16_t time){uint8_t cmnd;
//---ЭТАП==0-----------------------------------------------------------------------------------
if(lcd[num].stepSend==0){if(mode==DATA) {cmnd=1;} else {cmnd=0;}
//--Интерфейс i2c------------------------------------------------------------------------------
if(lcd[num].mode==1){
	if(!(mode==NIBBLE)) {device.buff8=(val &0xf0)| cmnd;} else {device.buff8= val <<4;}
	device.buffSend[0] = device.buff8          | BACKLIGHT_ON;	//--Повтор -> необходим импульс не менее >450ns
	device.buffSend[1] = device.buff8 |  FL_EN | BACKLIGHT_ON;	//--Повтор -> необходим импульс не менее >450ns
	device.buffSend[2] = device.buff8          | BACKLIGHT_ON;	//-- 
	device.buffSend[3] =(device.buff8 & ~FL_EN)| BACKLIGHT_ON; 	//--			
	if(!(mode==NIBBLE)) {device.buff8=(val<<4 & 0xf0) | cmnd;	
	device.buffSend[4] = device.buff8          | BACKLIGHT_ON;	//--Повтор -> необходим импульс не менее >450ns
	device.buffSend[5] = device.buff8 |  FL_EN | BACKLIGHT_ON;	//--Повтор -> необходим импульс не менее >450ns
	device.buffSend[6] = device.buff8          | BACKLIGHT_ON;	 
	device.buffSend[7] =(device.buff8 & ~FL_EN)| BACKLIGHT_ON;}		
	if(!(mode==NIBBLE)) {device.buff8=8;} else {device.buff8=4;}//--Установка длины посылки;			
	Wire.beginTransmission(lcd[num].addr); Wire.write(device.buffSend,device.buff8); 
	device.buff8=Wire.endTransmission(); 						//--Отправка на шину i2c
	if(device.buff8!=0) {lcd[num].link=0;}};					//--Установка флага ошибки
//--Интерфейс SPI------------------------------------------------------------------------------	
if(lcd[num].mode==2){
	if(!(mode==NIBBLE)){device.buff8=val>>4;} else {device.buff8= val &0x0f;}
						device.buff8=(device.buff8<<2); bitWrite(device.buff8,0,cmnd); 
						bitWrite(device.buff8,1,1); digitalWrite(lcd[num].addr,0); SPI.transfer(device.buff8); digitalWrite(lcd[num].addr,1);
						bitWrite(device.buff8,1,0); digitalWrite(lcd[num].addr,0); SPI.transfer(device.buff8); digitalWrite(lcd[num].addr,1);
	if(!(mode==NIBBLE)){device.buff8=val; 
						device.buff8=(device.buff8<<2); bitWrite(device.buff8,0,cmnd); 
						bitWrite(device.buff8,1,1); digitalWrite(lcd[num].addr,0); SPI.transfer(device.buff8); digitalWrite(lcd[num].addr,1);
						bitWrite(device.buff8,1,0); digitalWrite(lcd[num].addr,0); SPI.transfer(device.buff8); digitalWrite(lcd[num].addr,1);}}		
//--Интерфейс - параллельное подключение через 6 пинов------------------------------------------
if((lcd[num].mode==3) || (lcd[num].mode==4)){
	digitalWrite(lcd[num].pins[0],cmnd); 
	if(!(mode==NIBBLE)){device.buff8=val>>4;}	else {device.buff8=val & 0x0f;} 
						for(int i=2; i<6; i++) {digitalWrite(lcd[num].pins[i],(device.buff8 >> (i-2)) & 0x01);}
						digitalWrite(lcd[num].pins[1],0); digitalWrite(lcd[num].pins[1],1); digitalWrite(lcd[num].pins[1],0); 
	if(!(mode==NIBBLE)){device.buff8=val;
						for(int i=2; i<6; i++) {digitalWrite(lcd[num].pins[i],(device.buff8 >> (i-2)) & 0x01);}
						digitalWrite(lcd[num].pins[1],0); digitalWrite(lcd[num].pins[1],1); digitalWrite(lcd[num].pins[1],0);}} 
//---------------------------------------------------------------------------------------------
lcd[num].timeDelay=RT_HW_LCD_HD44780_MICROS+time; lcd[num].stepSend=1; }//--Переход на следующий этап c ожиданием времени time;
//---ЭТАП==1-----------------------------------------------------------------------------------
if(lcd[num].stepSend==1){if(RT_HW_LCD_HD44780_MICROS> lcd[num].timeDelay) {lcd[num].stepSend=0; return 1;} else {return 0;}};
return 0;};//--sendDevice	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	1.7. Инициализация устройства(Lcd через 4-х битную шину данных)
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_LCD_HD44780:: init(uint8_t num) { 
/*
switch(lcd[num].stepInit) {default: lcd[num].stepInit=0; break;	
  case 0: lcd[num].initOk=0; lcd[num].link=1; lcd[num].timeDelay=RT_HW_LCD_HD44780_MICROS+5000; lcd[num].stepInit++;  break;
  case 1: if(RT_HW_LCD_HD44780_MICROS > lcd[num].timeDelay) {lcd[num].stepInit++;} break;
  case 2: if(sendDevice(num,B00000011,NIBBLE,4500)) {lcd[num].stepInit++;} break;	//--передача тетрады
  case 3: if(sendDevice(num,B00000011,NIBBLE,4500)) {lcd[num].stepInit++;} break;	//--передача тетрады
  case 4: if(sendDevice(num,B00000011,NIBBLE,4500)) {lcd[num].stepInit++;} break;	//--передача тетрады  
  case 5: if(sendDevice(num,B00000010,NIBBLE,2000)) {lcd[num].stepInit++;} break;	//--передача тетрады  
  case 6: if(sendDevice(num,B00100000,CMND,  2000)) {lcd[num].stepInit++;} break;	//--передача тетрады  
  case 7: if(sendDevice(num,B00000001,CMND,  2000)) {lcd[num].stepInit++;} break;	//--передача тетрады   
  case 8: if(sendDevice(num,B00000100,CMND,  2000)) {lcd[num].stepInit++;} break;	//--передача тетрады    
  //------------------------------------------------------------------------------	 
  case 9: if(sendDevice(num,B00101000,CMND,  2000)) {lcd[num].stepInit++;} break;	//--Обмен по 4-х разрядной шине, 2-линии, символ 5х8.           					
  case 10:if(sendDevice(num,B00010000,CMND,  2000)) {lcd[num].stepInit++;} break;	//--Сдвиг курсора, перемещения курсора влево
  case 11:if(sendDevice(num,B00001100,CMND,  2000)) {lcd[num].stepInit++;} break;	//--Включение дисплея, выключение курсора, тип курсора "мерцание"
  case 12:if(sendDevice(num,B00000110,CMND,  2000)) {lcd[num].stepInit++;} break;	//--Движение курсора влево, выключение дисплея
  case 13:if(sendDevice(num,B00000001,CMND,  2000)) {lcd[num].stepInit++;} break;	//--Очистка дисплея
  case 14:lcd[num].initOk=1; lcd[num].initOk=1;	 lcd[num].stepInit=0;   break; }
*/ 	
if(lcd[num].stepInit==0) {lcd[num].initOk=0; lcd[num].link=1; lcd[num].timeDelay=RT_HW_LCD_HD44780_MICROS+5000; lcd[num].stepInit++; return;}
if(lcd[num].stepInit==1) {if(RT_HW_LCD_HD44780_MICROS > lcd[num].timeDelay) {lcd[num].stepInit++;} return;}
if(lcd[num].stepInit==2) {if(sendDevice(num,B00000011,NIBBLE,4500)) {lcd[num].stepInit++;} return;}	//--передача тетрады
if(lcd[num].stepInit==3) {if(sendDevice(num,B00000011,NIBBLE,4500)) {lcd[num].stepInit++;} return;}	//--передача тетрады
if(lcd[num].stepInit==4) {if(sendDevice(num,B00000011,NIBBLE,4500)) {lcd[num].stepInit++;} return;}	//--передача тетрады  
if(lcd[num].stepInit==5) {if(sendDevice(num,B00000010,NIBBLE,2000)) {lcd[num].stepInit++;} return;}	//--передача тетрады  
if(lcd[num].stepInit==6) {if(sendDevice(num,B00100000,CMND,  2000)) {lcd[num].stepInit++;} return;}	//--передача тетрады  
if(lcd[num].stepInit==7) {if(sendDevice(num,B00000001,CMND,  2000)) {lcd[num].stepInit++;} return;}	//--передача тетрады   
if(lcd[num].stepInit==8) {if(sendDevice(num,B00000100,CMND,  2000)) {lcd[num].stepInit++;} return;}	//--передача тетрады    
 //------------------------------------------------------------------------------------------------	 
if(lcd[num].stepInit==9) {if(sendDevice(num,B00101000,CMND,  2000)) {lcd[num].stepInit++;} return;}	//--Обмен по 4-х разрядной шине, 2-линии, символ 5х8.           					
if(lcd[num].stepInit==10){if(sendDevice(num,B00010000,CMND,  2000)) {lcd[num].stepInit++;} return;}	//--Сдвиг курсора, перемещения курсора влево
if(lcd[num].stepInit==11){if(sendDevice(num,B00001100,CMND,  2000)) {lcd[num].stepInit++;} return;}	//--Включение дисплея, выключение курсора, тип курсора "мерцание"
if(lcd[num].stepInit==12){if(sendDevice(num,B00000110,CMND,  2000)) {lcd[num].stepInit++;} return;}	//--Движение курсора влево, выключение дисплея
if(lcd[num].stepInit==13){if(sendDevice(num,B00000001,CMND,  2000)) {lcd[num].stepInit++;} return;}	//--Очистка дисплея
if(lcd[num].stepInit==14){lcd[num].initOk=1; lcd[num].initOk=1;	     lcd[num].stepInit=0;  return;} //--Завершение инициализации
lcd[num].stepInit=0; 																	   return;} //--Если не верный номер шага;   	
//=====================================================================================================================
// 						II. ФУНКЦИИ ДЛЯ ВЫВОДА ПАРАМЕТРОВ В УСТРОЙСТВО
//=====================================================================================================================
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
//	2.1. Генератор переменных управления ддя запуска функций  вывода параметров в устройство
//  Вход: 1) iq -> iq.ustPeriod - период обновления lcd; рабочие регистры и флаги;	
//		  2) gen50	- внешний меандр 50 мс; 
//        4) EN 	- =1 разрешение вывода; =0 принудительная очистка поля вывода; 
//		  3) blink	- разрешение мерцание поля вывода;
//  Выход id.dir: =0 вывод запрещен; =1 -разрешен (первичный или периодический вывод); =2 -разрешена очистка;
// 		  id.fresh  - готовность вывода??
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
void RT_HW_LCD_HD44780:: genEventIQ(RT_HW_LCD_HD44780_GEN_EVENT_IQ &iq, bool gen50, bool EN, bool blink){			 
//------0.Первоначальная установка-----------------------------------------------------------------
if(iq.first) {iq.first=0; iq.fresh=1; iq.dir=1; iq.cntBlink=iq.cntPeriod=0;}
//------1.Переход на очистку при изменении EN 1->0 ------------------------------------------------
if(iq.agoEN!=EN){iq.agoEN=EN; if(EN==0){iq.cntPeriod=0; iq.dir=2; return;}} 
//------2.Запрет на работу при EN=0 (ранее будет выполнена очистка dir=2)--------------------------
if(!EN) {iq.cntPeriod=iq.cntEN=0; return;}	
//------3.Ожидание разрешения на работу при EN=1 в течении 3-тактов после перехода EN 0->1. Обеспечивает надежную предварительную очистку поля.
if( EN) {if(iq.cntEN<3){iq.cntEN++;  return;}}						
//------4.При blink=1 чередование записи и очистки lcd через 500 мс--------------------------------
if(blink){if(iq.agoBlink!=gen50){iq.agoBlink=gen50; if(++iq.cntBlink>10){iq.cntBlink=1; if(iq.vBlink){iq.vBlink=0; iq.dir=2;} else {iq.vBlink=1; iq.dir=1;} }}return;}
//------5.Обработка от таймера prt[id.num]->event.blink (меандр 50мс) c периодом в тиках =period при: при EN=1,blink=0;
if(iq.agoPeriod!=gen50) {iq.agoPeriod =gen50; if(++iq.cntPeriod> iq.ustPeriod)  {iq.cntPeriod=1; iq.dir=1;}}  }; 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	2.2. Настройка параметров для структуры  RT_HW_LCD_HD44780_PARAM_CHAR_ID
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++    							  		 
void RT_HW_LCD_HD44780::setIdChar(RT_HW_LCD_HD44780_PARAM_CHAR_ID &id, int8_t num, uint8_t col, uint8_t row, char mode, uint8_t period){
  if(num>device.qnt){id.permit=false; return;}	id.num=num;			//--Установка номера устройства;
  if(col>lcd[num].cols) {id.col=lcd[num].cols;} else id.col=col;	//--Сохранение номера колонки с ограничением диапазона;
  if(row>lcd[num].rows) {id.row=lcd[num].rows;} else id.row=row;	//--Сохранение номера строки  с ограничением диапазона;
  id.mode=mode; 
  id.step=0; id.event.ustPeriod=period;								//--Установка регистров;
  id.custom=id.permit=id.event.first=true;	};						//--Установка флагов;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	2.3. Настройка параметров для структуры  RT_HW_LCD_HD44780_PARAM_VAL_ID
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++    							  		 
void RT_HW_LCD_HD44780::setIdVal(RT_HW_LCD_HD44780_PARAM_VAL_ID &id, int8_t num, uint8_t col, uint8_t row, uint8_t len, uint8_t period){
  if(num>device.qnt) {id.permit=false; return;}	id.num=num;	//--Установка номера устройства;
  if(col>lcd[num].cols) {id.col=lcd[num].cols;} else id.col=col;	//--Сохранение номера колонки с ограничением диапазона;
  if(row>lcd[num].rows) {id.row=lcd[num].rows;} else id.row=row;	//--Сохранение номера строки  с ограничением диапазона;
  device.buff8=lcd[num].cols-id.col+1; 								//--Вычисляем максимальную длину сообщения в соответствии с номером колонки и длины доступного поля;
  id.len=len;														//--Установка длины поля;
  if(len==0) {id.len=device.buff8;}									//--Если len==0, устанавливается длина поля до конца строки;
  if(len>device.buff8) {id.len=device.buff8;} 						//--Ограничение длины поля размером до конца строки; 
  id.step=0; id.event.ustPeriod=period;								//--Установка регистров;
  id.custom=id.permit=id.event.first=true;  
  };						//--Установка флагов;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	2.4. Настройка параметров для структуры  RT_HW_LCD_HD44780_PARAM_TEXT_ID
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++    							  		 
void RT_HW_LCD_HD44780::setIdText(RT_HW_LCD_HD44780_PARAM_TEXT_ID &id, int8_t num, uint8_t col, uint8_t row, uint8_t len, uint8_t period){
  if(num>device.qnt){id.permit=false; return;}	id.num=num;			//--Установка номера устройства;
  if(col>lcd[num].cols) {id.col=lcd[num].cols;} else id.col=col;	//--Сохранение номера колонки с ограничением диапазона;
  if(row>lcd[num].rows) {id.row=lcd[num].rows;} else id.row=row;	//--Сохранение номера строки  с ограничением диапазона;
  device.buff8=lcd[num].cols-id.col+1; 								//--Вычисляем максимальную длину сообщения в соответствии с номером колонки и длины доступного поля;
  id.len=len;														//--Установка длины поля;
  if(len==0) {id.len=device.buff8;}									//--Если len==0, устанавливается длина поля до конца строки;
  if(len>device.buff8) {id.len=device.buff8;} 						//--Ограничение длины поля размером до конца строки;
  id.step=0; id.event.ustPeriod=period;								//--Установка регистров;
  id.custom=id.permit=id.event.first=true; 
  };						//--Установка флагов;  
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	2.5. Вывод символа
//Serial.print(F("printChar_01 num=")); Serial.print(id.num); Serial.print(F(" step=")); Serial.println(id.step);
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_LCD_HD44780:: printIdChar (RT_HW_LCD_HD44780_PARAM_CHAR_ID &id, bool EN, bool blink, char sign0, char sign1){	
if(id.num>device.qnt) {return;}														//--Контроль допустимости номера устройства;
if(lcd[id.num].permit==0){id.event.fresh=true; id.step=0; id.run=0; return;}; 		//--Контроль готовности устройства(не настроено, потеря связи и пр.);													
if(id.step==0)	{genEventIQ(id.event,lcd[id.num].blink,EN,blink); 					//--Анализ событий  от  генератора событий.
				 if((id.event.fresh==0) && (id.event.dir==0))  {return;} 			//--Если нет событий для выполнения, то выход;
				 if(id.event.dir==1){												//--Режима вывода данных.				 
					if(id.mode=='F') {if(id.blink) {id.buff=sign0; id.blink=0;} 			//--Если мерцание, то в буфер загружается
								else {id.buff=sign1; id.blink=1;} id.event.fresh=1;}		//   выводимый символ и взодится флаг вывода;	
					if(id.mode=='B') {if(id.boolVal){id.buff=sign1;} else {id.buff=sign0;}	//--Если вывод двоичной переменной, в буфер загружается соответствующий 0 или 1 символ;
									  if(id.boolAgo!=id.boolVal){id.boolAgo=id.boolVal; id.event.fresh=1;}} 	//--
					if(id.mode=='C') {//if(sign0<32) {sign0=RT_HW_LCD_HD44780_BLANK;}			//
									  if(id.buff!=sign0){id.buff=sign0; id.event.fresh=1;}}
					if(id.event.fresh){id.event.fresh=0; id.run=1;}}
				//-----------------------------------------------------------------------
				if(id.event.dir==2) {id.event.fresh=1; id.run=1; id.buff=RT_HW_LCD_HD44780_BLANK;}					   
				id.event.dir=0;	if(id.run) {id.step++;} else {return;}}
if(id.step==1){	if(lcd[id.num].busy) return; lcd[id.num].busy=1; id.step++;}  		//--Захват устройства (в каждый момент допускается только один вывод на lcd);	
if(id.step==2){	if(!sendDevice(id.num, pointCursor(id.col,id.row),CMND,50)) {return;} id.step++;}	//--Установка курсора;
if(id.step==3){ if(!sendDevice(id.num,id.buff,DATA,10)) {return;} 					//--Ожидание завершения вывода символа;
				lcd[id.num].busy=0; id.run=0; id.step=0; return;}	};				//--Завершение;		
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	2.6. Вывод переменной
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	 
void  RT_HW_LCD_HD44780:: printIdVal(RT_HW_LCD_HD44780_PARAM_VAL_ID  &id, bool EN, bool blink, char alig, char mode, int32_t val, char suff,  const char *head) {
if(id.num>device.qnt) {return;}														//--Контроль допустимости номера устройства;
if(lcd[id.num].permit==0){id.event.fresh=true; id.step=0; id.run=0; return;}; 		//--Контроль готовности устройства(не настроено, потеря связи и пр.);		
//------Обработка параметров от таймера------------------------------------------------------------
if(id.step==0)	{genEventIQ(id.event,lcd[id.num].blink,EN,blink); if((id.event.fresh==0) && (id.event.dir==0)) {return;} //--Если нет событий для выполнения, то выход;
				 id.dir=id.event.dir; id.event.dir=0; id.step++; return;}			//--Переход на следующий этап с выходом (для возможности контроля изменения данных по признаку id.dir==1;)
//------Анализ входных данных----------------------------------------------------------------------
if(id.step==1)	{if(id.dir==1) {if(id.event.fresh){id.event.fresh=0; id.run=1;};};		   
				 if(id.dir==2) {id.event.fresh=1; id.run=1; id.clean=1;};					   				 
				 id.dir=0;	   if(!id.run) {id.step=0;  return;} 
				 if(!id.customConst) {id.customConst=1; id.head=getLenPGM(head,id.len); id.end=0; if(suff!=RT_HW_LCD_HD44780_BREAK_CHAR){id.end=1;};}; //--Одноразовая установка длин заголовка и суффикса;	 
				 setVar_ID(id.data,mode,val);										//--Обработка переменной;		
				 id.lenControl=0; id.flag=0; id.step++; return;};  			   
//------Ожидание освобождение вывода и захват устройства-------------------------------------------
if(id.step==2)	{if(lcd[id.num].busy) return;  lcd[id.num].busy=1; id.step++;}	
//-------Установка курсора и пропуск следующего шага, если режим вывода данных (id.clean==0) 
if(id.step==3)	{if(sendDevice(id.num, pointCursor(id.col,id.row),CMND,50)) {
				 if(id.clean) {id.index=id.len; id.step=4; return;} 
				        else  {id.index=getLenBegin(id.len,(id.head+id.data.minus+id.data.len+id.data.pnt+id.end),alig); id.step=5; return;}}} 
//------Очистка поля и завершение с переходом на начало--------------------------------------------
if(id.step==4)	{if(id.index==0){id.clean=0; lcd[id.num].busy=0; id.run=0; id.step=0; return;}  
				 if(sendDevice(id.num,RT_HW_LCD_HD44780_BLANK,DATA,0)) {id.index--;}  return;} 			
//-------Вывод заполнителей------------------------------------------------------------------------
if(id.step==5) {if((id.lenControl>=id.len) || (id.index==0)){id.index=0; id.indexSrc=0;  id.step++; return;} 
				if(sendDevice(id.num, RT_HW_LCD_HD44780_BLANK, DATA,0)) {id.lenControl++; id.index--;} return;} //--Вывод заполнителей до значения; 			
//-------Вывод заголовка---------------------------------------------------------------------------
if(id.step==6) {if((id.lenControl>=id.len) || (id.index>=id.head)) {id.step++; return;} 
				if(!id.flag) {id.flag=1; id.buff=getCharLcdPGM(head,id.indexSrc);}
                if(sendDevice(id.num,id.buff,DATA,0)) {id.flag=0; id.lenControl++; id.index++;} return;};
//-------Подготовка следующих этапов---------------------------------------------------------------
if(id.step==7) {if(id.data.minus) {id.step=8;} else {id.step=9;} id.index=id.data.len-id.data.point; id.data.depth=1; return;}
//-------Вывод знака '-'. Установка количества цифр до точки---------------------------------------
if(id.step==8){if(id.lenControl>=id.len) {id.step++; return;} 
				if(sendDevice(id.num,'-',DATA,0)) {id.lenControl++; id.step++;} return;}	//--Вывод символа '-' (если требуется);
//-------Вывод цифр до точки-----------------------------------------------------------------------
if(id.step==9){if((id.lenControl>=id.len) || (id.index==0)){if(id.data.pnt) {id.index=1;} else {id.index=0;} id.step++; return;}  
				if(sendDevice(id.num,getCharVal(id.data.val,1,mode,id.data.len,id.data.depth),DATA,0)) {id.lenControl++; id.data.depth++; id.index--;} return;} //-Вывод цифр до точки;
//-------Вывод точки. Установка количества цифр после точки---------------------------------------- 
if(id.step==10){if((id.lenControl>=id.len) || (id.index==0)){id.index=id.data.point; id.data.depth=id.data.len-id.data.point+1; id.step++; return;} 
				if(sendDevice(id.num,'.',DATA,0)) {id.lenControl++; id.index--;} return;} //--Вывод точки (если требуется);			
//-------Вывод цифр после точки. Переход в зависимости от наличия суффикса-------------------------
if(id.step==11){if((id.lenControl>=id.len) || (id.index==0)){if(id.end) {id.step=12;} else {id.step=13;} return;}						
				if(sendDevice(id.num,getCharVal(id.data.val,1,mode,id.data.len,id.data.depth),DATA,0)) {id.lenControl++; id.data.depth++; id.index--;} return;} 
//-------Вывод суффикса----------------------------------------------------------------------------
if(id.step==12){if(id.lenControl>=id.len){id.step++; return;}  
				if(sendDevice(id.num,suff,DATA,0)) {id.lenControl++;  id.step++;} return;}	//--Вывод суффикса; 
//-------Вывод заполнителей после суффикса---------------------------------------------------------
if(id.step==13){if(id.lenControl>=id.len){id.step++; return;}; 
				if(sendDevice(id.num,RT_HW_LCD_HD44780_BLANK, DATA,0)) {id.lenControl++;} return;} //--Вывод заполнителей после значения.				
//-------Завершение--------------------------------------------------------------------------------
if(id.step==14){lcd[id.num].busy=0; id.run=0; id.step=0; return;}};

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	2.7. Вывод текста из PROGMEM
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	 
void  RT_HW_LCD_HD44780:: printTextPGM(RT_HW_LCD_HD44780_PARAM_TEXT_ID  &id, bool EN, bool blink, char alig, const char *head) {
if(id.num>device.qnt) {return;}														//--Контроль допустимости номера устройства;
if(lcd[id.num].permit==0){id.event.fresh=true; id.step=0; id.run=0; return;}; 		//--Контроль готовности устройства(не настроено, потеря связи и пр.);		
//------Обработка параметров от таймера------------------------------------------------------------
if(id.step==0)	{genEventIQ(id.event,lcd[id.num].blink,EN,blink); if((id.event.fresh==0) && (id.event.dir==0)) {return;} //--Если нет событий для выполнения, то выход;
				 id.dir=id.event.dir; id.event.dir=0; id.step++; return;}			//--Переход на следующий этап с выходом (для возможности контроля изменения данных по признаку id.dir==1;)
//------Анализ входных данных----------------------------------------------------------------------
if(id.step==1)	{if(id.dir==1) {if(id.event.fresh){id.event.fresh=0; id.run=1;}}		   
				 if(id.dir==2) {id.event.fresh=1; id.run=1; id.clean=1;}	
				 id.dir=0;	   if(!id.run) {id.step=0;  return;} 
				 id.head=getLenPGM(head,id.len);  	
				 //Serial.print(F("LCD len,curLen,begin,end=")); RT_HW_Base.printDEC(id.len,','); RT_HW_Base.printDEC(id.head,','); RT_HW_Base.printDEC(getLenBegin(id.len,id.head,alig),'Y'); 
				 id.lenControl=0; id.flag=0; id.step++; return;}  			   
//------Ожидание освобождение вывода и захват устройства-------------------------------------------
if(id.step==2)	{if(lcd[id.num].busy) return;  lcd[id.num].busy=1; id.step++;}	
//-------Установка курсора и пропуск следующего шага, если режим вывода данных (id.clean==0) 
if(id.step==3)	{if(sendDevice(id.num, pointCursor(id.col,id.row),CMND,50)) {
				 if(id.clean) {id.index=id.len; id.step=4; return;} else  {id.index=getLenBegin(id.len,id.head,alig); id.step=5; return;}}} 
//------Очистка поля и завершение с переходом на начало--------------------------------------------
if(id.step==4)	{if(id.index==0){id.clean=0; lcd[id.num].busy=0; id.run=0; id.step=0; return;}  
				 if(sendDevice(id.num,RT_HW_LCD_HD44780_BLANK,DATA,0)) {id.index--;}  return;} 			
//-------Вывод заполнителей------------------------------------------------------------------------
if(id.step==5) {if((id.lenControl>=id.len) || (id.index==0)){id.index=0; id.indexSrc=0; id.step++; return;} 
				if(sendDevice(id.num, RT_HW_LCD_HD44780_BLANK, DATA,0)) {id.lenControl++; id.index--;} return;} //--Вывод заполнителей до значения; 
//-------Вывод заголовка. Установка кол-ва заполнителей после данных.------------------------------
if(id.step==6) {if((id.lenControl>=id.len) || (id.index>=id.head)) {id.step++; return;} 
				if(!id.flag) {id.flag=1; id.buff=getCharLcdPGM(head,id.indexSrc);}
                if(sendDevice(id.num,id.buff,DATA,0)) {id.flag=0; id.lenControl++; id.index++;} return;};
//-------Вывод заполнителей------------------------------------------------------------------------
if(id.step==7){if((id.lenControl>=id.len) || (id.index==0)){id.step++; return;}; 
				if(sendDevice(id.num,RT_HW_LCD_HD44780_BLANK, DATA,0)) {id.lenControl++;} return;} //--Вывод заполнителей после значения.				
//-------Завершение--------------------------------------------------------------------------------
if(id.step==8){lcd[id.num].busy=0; id.run=0; id.step=0; return;}
};

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	2.8. Вывод строковой переменной
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	 
void  RT_HW_LCD_HD44780:: printTextSTR(RT_HW_LCD_HD44780_PARAM_TEXT_ID  &id, bool EN, bool blink, char alig, String &head) {
if(id.num>device.qnt) {return;}														//--Контроль допустимости номера устройства;
if(lcd[id.num].permit==0){id.event.fresh=true; id.step=0; id.run=0; return;}; 		//--Контроль готовности устройства(не настроено, потеря связи и пр.);		
//------Обработка параметров от таймера------------------------------------------------------------
if(id.step==0)	{genEventIQ(id.event,lcd[id.num].blink,EN,blink); if((id.event.fresh==0) && (id.event.dir==0)) {return;} //--Если нет событий для выполнения, то выход;
				 id.dir=id.event.dir; id.event.dir=0; id.step++; return;}			//--Переход на следующий этап с выходом (для возможности контроля изменения данных по признаку id.dir==1;)
//------Анализ входных данных----------------------------------------------------------------------
if(id.step==1)	{if(id.dir==1) {if(id.event.fresh){id.event.fresh=0; id.run=1;}}		   
				 if(id.dir==2) {id.event.fresh=1; id.run=1; id.clean=1;}	
				 id.dir=0;	   if(!id.run) {id.step=0;  return;} 
				 id.head=getLenSTR(head,id.len);  id.lenControl=0; id.flag=0; 
				 id.step++; return;}  			   
//------Ожидание освобождение вывода и захват устройства-------------------------------------------
if(id.step==2)	{if(lcd[id.num].busy) return;  lcd[id.num].busy=1; id.step++;}	
//-------Установка курсора и пропуск следующего шага, если режим вывода данных (id.clean==0) 
if(id.step==3)	{if(sendDevice(id.num, pointCursor(id.col,id.row),CMND,50)) {
				 if(id.clean) {id.index=id.len; id.step=4; return;} else  {id.index=getLenBegin(id.len,id.head,alig); id.step=5; return;}}} 
//------Очистка поля и завершение с переходом на начало--------------------------------------------
if(id.step==4)	{if(id.index==0){id.clean=0; lcd[id.num].busy=0; id.run=0; id.step=0; return;}  
				 if(sendDevice(id.num,RT_HW_LCD_HD44780_BLANK,DATA,0)) {id.index--;}  return;} 			
//-------Вывод заполнителей------------------------------------------------------------------------
if(id.step==5) {if((id.lenControl>=id.len) || (id.index==0)){id.index=0; id.indexSrc=0; id.lenControl=0; id.step++; return;} 
				if(sendDevice(id.num, RT_HW_LCD_HD44780_BLANK, DATA,0)) {id.lenControl++; id.index--;} return;} //--Вывод заполнителей до значения; if(id.vMinus) {id.index=1;} else {id.index=0;}				
//-------Вывод заголовка. Установка кол-ва заполнителей после данных.------------------------------
if(id.step==6) {if((id.lenControl>=id.len) || (id.index>=id.head)) {id.step++; return;} 
				if(!id.flag) {id.flag=1; id.buff=getCharLcdSTR(head,id.indexSrc);}
                if(sendDevice(id.num,id.buff,DATA,0)) {id.flag=0; id.lenControl++; id.index++;} return;};
//-------Вывод заполнителей------------------------------------------------------------------------
if(id.step==7){if((id.lenControl>=id.len) || (id.index==0)){id.step++; return;}; 
				if(sendDevice(id.num,RT_HW_LCD_HD44780_BLANK, DATA,0)) {id.lenControl++;} return;} //--Вывод заполнителей после значения.				
//-------Завершение--------------------------------------------------------------------------------
if(id.step==8){lcd[id.num].busy=0; id.run=0; id.step=0; return;}
};
//=====================================================================================================================
// 						III. ВСПОМОГАТЕЛЬНЫЕ ФУНКЦИИ 
//=====================================================================================================================		   
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	3.1. Получение номера символа из utf-8
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
uint8_t  RT_HW_LCD_HD44780:: convert(uint8_t charH, uint8_t charL){
if(charH==0xD0) {if (charL==0x81) {return 0x40;}  if((charL>=0x90) && (charL<=0xBF)){return (charL-0x90);}};
if(charH==0xD1) {if (charL==0x91) {return 0x41;}  if((charL>=0x80) && (charL<=0x8F)){return (charL-0x50);}}; return charL;}; 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	3.2. Возвращает кода символа для LCD (из таблицы перекодировки LCD в PROGMEM)
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
uint8_t RT_HW_LCD_HD44780:: getCodeCharLcdPGM(uint8_t code){return pgm_read_byte(RT_HW_LCD_HD44780_CONVERT_RUS+code);};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	3.3. Возвращает длину String  в формате utf-8 (с учетом блокирующего символа RT_HW_LCD_HD44780_BREAK_CHAR '~')
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
uint8_t RT_HW_LCD_HD44780:: getLenSTR(String &str, uint8_t lenMax){uint8_t buff,j,LEN; if(lenMax==0) {lenMax=40;} j=LEN=0;
while (1) {buff=str[j];	         if(buff>=0xC0) {j++; buff=str[j];}  j++;	
if((buff==0) || (buff==RT_HW_LCD_HD44780_BREAK_CHAR) || (LEN>lenMax) || (j>160)) {break;} LEN++;} return LEN;}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	2.6.4. Возвращает длину текста из PROGMEM  в формате utf-8 c с учетом символа RT_HW_LCD_HD44780_BREAK_CHAR '~'
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
uint8_t RT_HW_LCD_HD44780:: getLenPGM(const char *text, uint8_t lenMax){uint8_t buff,j,LEN; if(lenMax==0) {lenMax=40;} j=LEN=0;
while (1) {buff=pgm_read_byte(text+j);	j++;
		   if((buff==0) || (buff==RT_HW_LCD_HD44780_BREAK_CHAR) || (LEN>lenMax) || (j>160)) {break;}
		   if(buff>=0xC0) {j++; buff=pgm_read_byte(text+j);} LEN++;} return LEN;}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	2.6.5. Функция возвращает следующий код символа из String  и в параметрах возвращает указатель на следующий байт текста (с учетом utf-8)
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
uint8_t  RT_HW_LCD_HD44780:: getCharLcdSTR(String &str,      uint8_t &num){uint8_t buff1,buff2; 
buff2=0; buff1=str[num]; num++; if(buff1>=0xC0) {buff2=str[num];  num++; return getCodeCharLcdPGM(convert(buff1,buff2));} else {return buff1;}}
//buff2=0; buff1=str.charAt(num);         num++; if(buff1>=0xC0) {buff2=str.charAt(num);         num++; return getCodeCharLcdPGM(convert(buff1,buff2));} else {return buff1;}}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	2.6.6. Функция возвращает следующий код символа из текста в PROGMEM  и в параметрах возвращает указатель на следующий байт текста (с учетом utf-8)
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
uint8_t  RT_HW_LCD_HD44780:: getCharLcdPGM(const char *text,uint8_t &num){uint8_t buff1,buff2; 
buff2=0; buff1=pgm_read_byte(text+num); num++; if(buff1>=0xC0) {buff2=pgm_read_byte(text+num); num++; return getCodeCharLcdPGM(convert(buff1,buff2));} else {return buff1;}}
//=====================================================================================================================
// 						IV. ВСПОМОГАТЕЛЬНЫЕ ФУНКЦИИ 
//=====================================================================================================================	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
//                **** ВОЗВРАЩАЕТ АДРЕС ПОЗИЦИИ ПО НОМЕРУ КОЛОНКИ И НОМЕРУ СТРОКИ 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
uint8_t  RT_HW_LCD_HD44780::  pointCursor(uint8_t col, uint8_t row) {row=constrain(row,1,RT_HW_LCD_HD44780_MAX_ROWS)-1;
if(row==0) {return (0x80 | ((col-1) + 0x00));} if(row==1) {return (0x80 | ((col-1) + 0x40));} 
if(row==2) {return (0x80 | ((col-1) + 0x14));} if(row==3) {return (0x80 | ((col-1) + 0x54));} return 0x80;}  
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
//    ВОЗВРАЩАЕТ КОЛ-ВО СИМВОЛОВ ДОБАВЛЯЕМЫХ ДО ФОРМАТИРУЕМОЙ СТРОКИ  
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
uint8_t RT_HW_LCD_HD44780:: getLenBegin(uint8_t len,uint8_t src,char alig){
if(len<=src){return 0;} if(alig=='C') {return (len-src-(len-src)/2);} if(alig=='R') {return (len-src);} return 0;}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
//    ВОЗВРАЩАЕТ КОЛ-ВО СИМВОЛОВ ДОБАВЛЯЕМЫХ ПОСЛЕ ФОРМАТИРУЕМОЙ СТРОКИ  
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++    
uint8_t RT_HW_LCD_HD44780:: getLenEnd  (uint8_t len,uint8_t src,char alig){
if(len<=src){return 0;} if(alig=='C') {return (len-(len-src-(len-src)/2)-src);} if(alig=='L') {return (len-src);} return 0;} 	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	Конвертор bool в char
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
char    RT_HW_LCD_HD44780::boolToChar (bool val, char sign_0, char sign_1){if(val) return sign_1; else return sign_0;};	//--Конвертация bool в char; 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	Конвертор float в int32_t су умножением на кол-во знаков после точек
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
int32_t RT_HW_LCD_HD44780::floatToInt32 (float val, uint8_t point){ for(uint8_t i=1; i< (point+1); i++){val=val*10;}; return round(val);};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
//	 ВОЗВРАЩАЕТ ПЕРВЫЙ СВОБОДНЫЙ БИТ(=0) В 16-РАЗРЯДНОМ РЕГИСТРЕ
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
uint8_t RT_HW_LCD_HD44780:: getFistBitUint16(uint16_t &reg){for(uint8_t i=0;i<=15;i++) {if(!bitRead(reg,i)) return i;} return -1;};				
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
//  Вычисление 16-битной CRC переменной String (1 символ примерно 12 мкс AVR328)
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
uint16_t  RT_HW_LCD_HD44780:: CRC16_String(String &s){uint8_t index; uint16_t buff16;
index=s.length();	buff16=0;	for (uint8_t j=index; j>0; j--) {buff16 ^=buff16+s.charAt(j);
     for (uint8_t i=0; i<8; i++) {if (buff16 & 1) {buff16= (buff16 >> 1) ^ 0xA001;} // 0xA001 is the reflection of 0x8005
					   else {buff16 >>= 1;}}}			return buff16;};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		var_ID: Установка параметров для вывода численной переменной 
// id.mode: =0 -BIN, =1 -DEC, =2 -HEX;
// id.form: =0 -без изменения,=1 -1 знак после точки;=2 -2 знака после точки;=3 -3 знака после точки; 
//			=8 -преобразование в uint8_t; =16 -преобразование в uint16_t; =32 -преобразование в uint32_t; 
//  Выход: iv.val -нормализованное значение(без знака); iv.minus - флаг наличия знака; iv.len - длина целой части;
//         iv.point-кол-во знаков после точки;     iv.pnt -флаг наличия точки;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_LCD_HD44780:: setVar_ID(RT_HW_LCD_HD44780_VAL_ID &iv, char mode, int32_t val){uint32_t buff32; uint8_t flag;
if(!iv.len) iv.len=1;  			//--Предварительная установка id.len (для исключения неточностей1 -ситуации с val=0; 
flag=iv.minus=iv.point=iv.pnt=iv.val=0; 	//--Очистка переменных;
if(mode=='D'){iv.point=0; flag=1;}
if(mode=='0'){iv.point=0; flag=1;}
if(mode=='1'){iv.point=1; flag=1;} 
if(mode=='2'){iv.point=2; flag=1;} 
if(mode=='3'){iv.point=3; flag=1;}
if(flag==1)  {flag=2; if(val<0) {iv.val=-val; iv.minus=1;} else {iv.val=val;}}
if(mode=='8'){flag=2; iv.val= (uint8_t)val&0xff;} 	
if(mode=='F'){flag=2; iv.val= (uint16_t)val&0xffff;} 	
if(mode=='L'){flag=2; iv.val= (uint32_t)val;}
if(flag==2)  { buff32=iv.val; iv.len=0; do{buff32=buff32/10; iv.len++; if(buff32==0) break;} while(iv.len<12); //--Вычисление кол-ва значащих цифр в переменной;
			   while(iv.len<=iv.point) {iv.len++;};	if(iv.point) iv.pnt=1;} //--Получение кол-ва значащих циф
if(mode=='T') {iv.len=2;   if(val<0) {val=-val;} if(val<100) {iv.val=val;} else {iv.val=val-((val/100)*100);  iv.len=2;}}	//--Формат 2-х  значного времени;
if(mode=='B') {iv.len=8;   iv.val=val & 0xff;}			//--Формат 8-ми значного BIN-числа;	
if(mode=='b') {iv.len=4;   iv.val=val & 0x0f;}			//--Формат 4-х  значного BIN-числа;	
if(mode=='H') {iv.len=4;   iv.val=val & 0xffff;}		//--Формат 4-х  значного HEX-числа;
if(mode=='h') {iv.len=2;   iv.val=val & 0xff;}		};	//--Формат 2-х  значного HEX-числа;	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	var_ID: Возвращает цифру в символьной форме из выводимой переменной начиная со старшей позиции
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
char RT_HW_LCD_HD44780:: getCharVal(uint32_t val, uint8_t upper, char mode, uint8_t len, uint8_t n){uint32_t buff32=0;
if((n>len) || (n>10)) n=0; 
if((mode=='B') || (mode=='b')) {buff32=(val>>(len-n))&1;}
if((mode=='H') || (mode=='h')) {buff32=(val>>(len-n)*4)&0xf;}
if((mode=='D') || (mode=='0') ||(mode=='1') ||(mode=='2') || (mode=='3') || (mode=='T')) {n=len-n+1; while(n!=0) {buff32=val; val/=10; buff32=buff32-(val*10); n--;};}
if(buff32<=9) {buff32=buff32 +'0';} else {buff32=buff32-10+(upper ? 'A' : 'a') ;}
return (uint8_t)buff32;};  		   
//=================================================================================================	
RT_HW_LCD_HD44780 RT_HW_Lcd_hd44780; //--Создание объекта LCD типа HD44780
//=================================================================================================