/*** FLProg ************** Библиотека RT_HW_LCD_HD44780 ver2.1 для Arduino ***************
  Вывод на несколько LCD типа HD44780 переменных String, char, int16_t, float в указанное место указанного дисплея.
  Если задана длина (len) выводимого сообщения, то вывод осуществляется в пределах указанной длины или до конца  строки.
  Если len=0, то вывод осуществляется до конца строк. 
  Сообщение может быть отформатировано(alig) в пределах заданной длины  влево ('L'), по центру ('C'), право ('R').
    При этом к сообщению добавляются пробелы или оно обрезается. Если alig='N' (или другой отличный символ 
	кроме 'L','C','R'), то форматирование не осуществляется. При этом надо иметь ввиду что более короткое сообщение
	не затрет предыдущее более длинное сообщение. Поэтому рекомендуется явно задавать вид форматирования или использовать
    форматирование по умолчанию, которое в библиотеке ='L' (влево).
  Имеется функция вывода мерцающего символа по указанному адресу.	
	
  Библиотеки  разрабатываются с учетом особенностей использования в системе визуального программирования FLProg.
  ecoins@mail.ru 29.09.2019г.
**************************************************************************************************/
//#################################################################################################
#ifndef RT_HW_LCD_HD44780_h
#define RT_HW_LCD_HD44780_h
//-------------------------------------------------------------------------------------------------
#if defined(ARDUINO) && ARDUINO >= 100 
#include <Arduino.h>
#else
#include <WProgram.h>
#endif
//#include "RT_HW_BASE.h"
//#include <avr\pgmspace.h>
//-------------------------------------------------------------------------------------------------
#include <Wire.h>
#include <SPI.h>
//------Макросы настройки интерфейса SPI-----------------------------------------------------------

/*
#if defined(ARDUINO_ARCH_STM32)
#define RT_HW_LCD_HD44780_SPI_BEGIN                 SPI.begin()
#define RT_HW_LCD_HD44780_SPI_TRANSFER_VAL(v)       SPI.transfer(v)
#define RT_HW_LCD_HD44780_SPI_TRANSFER_BUFF(buff,n) SPI.transfer(v)
#else
#define RT_HW_LCD_HD44780_SPI_BEGIN                 SPI.begin()
#define RT_HW_LCD_HD44780_SPI_TRANSFER_VAL(v)       SPI.transfer(v)
#define RT_HW_LCD_HD44780_SPI_TRANSFER_BUFF(buff,n) SPI.transfer(v)	
#endif	
#if   defined(__AVR_ATmega328__) || defined (__AVR_ATmega328P__) || defined(__AVR_ATmega168__) || defined(__AVR_ATmega88__) 
#define RT_HW_LCD_HD44780_MAX_DEVICE  	4			//--Максимальное кол-во устройств для контроллеров AVR c RAM<=2k: 
#else 
#define RT_HW_LCD_HD44780_MAX_DEVICE    8			//--Максимальное кол-во устройств всех других контроллеров;
#endif
*/	
#define RT_HW_LCD_HD44780_SPI_BEGIN                 SPI.begin()
#define RT_HW_LCD_HD44780_SPI_TRANSFER_VAL(v)       SPI.transfer(v)
#define RT_HW_LCD_HD44780_SPI_TRANSFER_BUFF(buff,n) SPI.transfer(v)	
#define RT_HW_LCD_HD44780_MAX_DEVICE    8			//--Максимальное кол-во устройств всех других контроллеров;
//-------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------
#define RT_HW_LCD_HD44780_VERSION 		12			//--Номер версии библиотеки
#define RT_HW_LCD_HD44780_MAX_COLS  	40			//--Максимальное кол-во колонок
#define RT_HW_LCD_HD44780_MAX_ROWS  	4			//--Максимальное кол-во строк
#define RT_HW_LCD_HD44780_PERIOD_TICK  	50			//--Период системного времени в ms;
#define RT_HW_LCD_HD44780_BLANK		    ' '			//--Символ заполнения;
#define RT_HW_LCD_HD44780_BREAK_CHAR	'~'			//--Символ "Запрещающего" символа;
#define RT_HW_LCD_HD44780_CODE_BLANK	's'			//--Символ указатель, что заполнитель "пробел";
#define RT_HW_LCD_HD44780_UPPER  		1			//--=0 -нижний регистр, =1 - верхний регистр (при выводе HEX-значений);
#define RT_HW_LCD_HD44780_MILLIS		millis()	//--Время в миллисекундах;
#define RT_HW_LCD_HD44780_MICROS		micros()	//--Время в микросекундах;
#define RT_HW_LCD_HD44780_DELAY_MCS(x)	delayMicroseconds(x) //--Задержка в микросекундах;
#define _RT_HW_LCD_HD44780_BYTE_PGM(a)  const uint8_t  a[]  PROGMEM  //--Создание в PROGMEM массива int8_t c именем а произвольной длины 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//         СТРУКТУРЫ
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
struct RT_HW_LCD_HD44780_VAL_ID  {uint32_t val; uint16_t len:4,depth:4,point:2,minus:1,pnt:1;};// fresh:1;}; //--6-Структура для выводимой переменной
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//                    СТРУКТУРА ДЛЯ ХРАНЕНИЯ ПАРАМЕТРОВ УСТРОЙСТВА -12 байт;
// mode режим подключении =1 через i2c; =2 через SPI; =3 через пины по 4-х битной шине; =4 через произвольные номера пинов по 4-х битной шине; 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
struct  RT_HW_LCD_HD44780_LCD {	//=16 байт		  
uint32_t timeDelay;				//--4--Рабочие регистры для работы со временем;
uint8_t  stepInit, stepDirect;	//--2--
uint8_t  num; 					//--1--
uint8_t  stepFind:1, stepSend:1, cnt20:5, reboot:1;	//-1
uint16_t cols:6,rows:3,mode:3;	//--2--Кол-во колонок и строк дисплея, номер устройства, тип устройства(i2c,SPI,PIN6,PIN3);
uint8_t	 addr;					//--1--1)адрес для i2c; 2)ss для SPI; 3)первый пин 4-х битовой шины (RS,WR,D4,D5,D6,D7); 4)первый пин через HC595;
uint8_t  custom:1,permit:1,link:1,busy:1,initOk:1,event:1,blink:1; //--1--
uint8_t  cnt,period;			//--2--
int8_t   pins[6];				//--6-- Буфера для номера пинов при подключение через шесть пинов;
};			
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	Вспомогательная структура для блоков вывода (для работы с генератором запуска вывода) - 4 байта;
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++    							  		 
struct RT_HW_LCD_HD44780_GEN_EVENT_IQ {uint8_t ustPeriod; uint8_t cntPeriod; uint16_t cntBlink:5,cntEN:2,dir:2,fresh:1,agoPeriod:1,agoBlink:1, vBlink:1, agoEN:1, first:1;};
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	Структура для вывода переменных char (10 байт);
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
struct RT_HW_LCD_HD44780_PARAM_CHAR_ID{	
RT_HW_LCD_HD44780_GEN_EVENT_IQ	event;	//-4-
uint8_t  num,step; 					  	//-2-Номер устройства, номер шага в функциях вывода;
char     mode,buff;						//-2-mode:'F'-мерцание,'C'-вывод символа,'B'-вывод двоичного значения;
uint16_t col:6,row:3,custom:1, permit:1, blink:1, boolAgo:1, boolVal:1, run:1;    }; //-2-
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	Структура для вывода переменных val (18 байт); 
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
struct RT_HW_LCD_HD44780_PARAM_VAL_ID{	
RT_HW_LCD_HD44780_GEN_EVENT_IQ	event; 						//--4-- Параметры для периодического ввызова;
RT_HW_LCD_HD44780_VAL_ID  		data; 						//--6-- Параметры для обработки выводимого значения;
uint8_t  step,num,index,indexSrc,lenControl,buff;    		//--4-- 1.Номер шага; 2.Номер устройства;3.Рабочий индекс; 4.Доп.рабочий индекс; 5.Контрольный счетчик вывода в поле; 6.Буфер выводимого в Lcd значения;
uint16_t col:6,row:3,dir:2,flag:1, custom:1, customConst:1; //--2-- 1.Номер столбца; 2.Номер строки; 3.Состояние запуска вывода(0,1,2); 4.Рабочий флаг; 
															//      5.Флаг настройки ID; 6.Флаг настройки постоянных параметров;длина заголовка, длина суффикса;
uint16_t len:6, head:6, end:1, permit:1, run:1, clean:1;	//--2-- 1.Длина поля; 2.Длина заголовка; 3.Длина суффикса: 4.готовность; 5.Запуск вывода; 6.Флаг очистки поля;
};					//-4-переменная для вывода;
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	Структура для вывода переменных val (12 байт); 
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
struct RT_HW_LCD_HD44780_PARAM_TEXT_ID{	
RT_HW_LCD_HD44780_GEN_EVENT_IQ	event; 						//--4-- Параметры для периодического ввызова;
uint8_t  step,num,index,indexSrc,lenControl,buff;    		//--4-- 1.Номер шага; 2.Номер устройства;3.Рабочий индекс; 4.Доп.рабочий индекс; 5.Контрольный счетчик вывода в поле; 6.Буфер выводимого в Lcd значения;
uint16_t col:6,row:3,dir:2,flag:1, custom:1, customConst:1; //--2-- 1.Номер столбца; 2.Номер строки; 3.Состояние запуска вывода(0,1,2); 4.Рабочий флаг; 5.Флаг настройки ID; 6.Флаг настройки постоянных параметров;длина заголовка, длина суффикса;
uint16_t len:6, head:6, end:1, permit:1, run:1, clean:1;	//--2-- 1.Длина поля; 2.Длина заголовка; 3.Длина суффикса: 4.готовность; 5.Запуск вывода; 6.Флаг очистки поля; 
};															//-4-переменная для вывода; 	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//                    СТРУКТУРА ДЛЯ ХРАНЕНИЯ ПАРАМЕТРОВ ИСПОЛЬЗУЕМЫХ ВСЕМИ УСТРОЙСТВАМИ //-2+1+1+8=12
// set  -регистр флагов инициализации устройства. Номер бита соответствует состоянию устройства с этим номером (0-15). 
//       Устройство с номером 0 зарезервировано и не используется.
// qnt  -кол-во подключенных устройства;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
struct RT_HW_LCD_HD44780_device	{uint32_t timeStart; uint16_t cntSec:5,qnt:4,i2c_ok:1,spi_ok:1,event:1,blink:1,eventSec:1,blinkSec:1; uint8_t buff8; uint8_t buffSend[8]; int32_t ibuff32; uint16_t buff16;};								 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//                     ТАБЛИЦА КОДОВ КИРИЛЛИЦЫ LCD
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
static const unsigned char  RT_HW_LCD_HD44780_CONVERT_RUS[] PROGMEM= 
{0x41,0xA0,0x42,0xA1,0xE0,0x45,0xA3,0xA4,   // АБВГДЕЖЗ
 0xA5,0xA6,0x4B,0xA7,0x4D,0x48,0x4F,0xA8,   // ИЙКЛМНОП
 0x50,0x43,0x54,0xA9,0xAA,0x58,0xE1,0xAB,   // РСТУФХЦЧ
 0xAC,0xE2,0xAD,0xAE,0xAD,0xAF,0xB0,0xB1,   // ШЩЪЫЬЭЮЯ
 0x61,0xB2,0xB3,0xB4,0xE3,0x65,0xB6,0xB7,   // абвгдежз
 0xB8,0xB9,0xBA,0xBB,0xBC,0xBD,0x6F,0xBE,   // ийклмноп
 0x70,0x63,0xBF,0x79,0xE4,0x78,0xE5,0xC0,   // рстуфхцч
 0xC1,0xE6,0xC2,0xC3,0xC4,0xC5,0xC6,0xC7,	// шщъыьэюя
 0xA2,0xB5,0xFF,0xFF};					    // Ёё
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//                     КЛАСС УПРАВЛЕНИЯ LCD
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
class RT_HW_LCD_HD44780{
enum {CMND=0,DATA=1,NIBBLE=2}; //--коды:байт команды, байт данных, тетрада
enum {FL_EN=B00000100, BACKLIGHT_ON=0x08, BACKLIGHT_OFF=0x00, SET_CGRAM_ADDR=0x40, SET_DDRAM_ADDR=0x80};
//-------------------------------------------------------------
public:
// ****** ДАННЫЕ *********************************************
#pragma pack(push, 4)			//--Установка выравнивания до 4 байт
RT_HW_LCD_HD44780_device 		device;								//--Общие регистры для работы с устройствами;
RT_HW_LCD_HD44780_LCD		    lcd[RT_HW_LCD_HD44780_MAX_DEVICE];
#pragma pack(pop)		//--Восстановление предыдущей настройки							
// ****** МЕТОДЫ ********************************************
RT_HW_LCD_HD44780(); //--Конструктор 
char 	getCharPGM(uint8_t val); 	 						//--Получение символа из таблицы PROGMEM;
uint8_t convert(uint8_t charH, uint8_t charL);				//--Возвращает перекодированный код для Lcd с учетои utf-8 и кирилдицы;
uint8_t getCodeCharLcdPGM(uint8_t code);					//--Возвращает кода символа для LCD (из таблицы перекодировки LCD в PROGMEM);
uint8_t getLenSTR      	 (String   &str,    uint8_t lenMax);//--Получение длины строковой переменной (c учетом utf-8, блокирующего символа('~'), ограничение по длине);
uint8_t getLenPGM      	 (const char *text, uint8_t lenMax);//--Получение длины текста из PROGMEM    (c учетом utf-8, блокирующего символа('~'), ограничение по длине);
uint8_t getCharLcdSTR	 (String &str,      uint8_t &num);	//--Получение символа из строковой переменной по номеру(c учетом utf-8);
uint8_t getCharLcdPGM	 (const char *text, uint8_t &num);	//--Получение символа из текста PROGMEM       по номеру(c учетом utf-8);
//------Функции для работы с устройством;
uint8_t	setDevice(uint8_t num, char mode, int8_t adr, uint8_t cols, uint8_t rows, uint8_t period=5, 
				  int8_t pRS=-1, int8_t pWR=-1, int8_t pDB4=-1, int8_t pDB5=-1, int8_t pDB6=-1, int8_t pDB7=-1);  //--Настройка параметров устройства;
void 	genEvent();																	//--Генератор событий для периодического вывода изменяющихся парамтров; 
void  	directDevice (uint8_t num);													//--Управление устройством;
bool	findDeviceI2C(uint8_t num, uint8_t cntMax);									//--Поиск устройства на шине i2c;
bool 	sendDevice   (uint8_t num, uint8_t val, uint8_t mode=DATA, uint16_t time=0);//--Запись в устройство;
void  	init		 (uint8_t num);													//--Инициализация устройства;	
//------Функции для вывода параметров
void    genEventIQ	(RT_HW_LCD_HD44780_GEN_EVENT_IQ   &iq, bool gen50, bool EN, bool blink);						//-10-Генератор временных меток;
void    setIdChar	(RT_HW_LCD_HD44780_PARAM_CHAR_ID  &id, int8_t num, uint8_t col, uint8_t row, char mode='F', uint8_t period=10); //--Установка ID для вывода char;
void    setIdVal 	(RT_HW_LCD_HD44780_PARAM_VAL_ID   &id, int8_t num, uint8_t col, uint8_t row, uint8_t len,   uint8_t period=10);	//--Установка ID для вывода численной переменной;
void    setIdText 	(RT_HW_LCD_HD44780_PARAM_TEXT_ID  &id, int8_t num, uint8_t col, uint8_t row, uint8_t len,   uint8_t period=10);	//--Установка ID для вывода численной переменной;
void    printIdChar	(RT_HW_LCD_HD44780_PARAM_CHAR_ID  &id, bool EN, bool blink, char sign0='-', char sign1='+');					//--Вывод символа;
void    printIdVal  (RT_HW_LCD_HD44780_PARAM_VAL_ID   &id, bool EN, bool blink, char alig, char mode, int32_t val, char suff, const char *head); //--Вывод переменной (D,0,1,2,3,H,h,B,b,T);				
void    printTextPGM(RT_HW_LCD_HD44780_PARAM_TEXT_ID  &id, bool EN, bool blink, char alig, const char *text);						//--Вывод текста из PROGMEM;	
void    printTextSTR(RT_HW_LCD_HD44780_PARAM_TEXT_ID  &id, bool EN, bool blink, char alig, String     &head); 						//--Вывод строковой переменной;
//------Вспомогательные функции
char    getCharVal  (uint32_t val,  uint8_t upper, char mode, uint8_t len, uint8_t n);	//--Получение символа из числа по номеру позиции;
void	setVar_ID	(RT_HW_LCD_HD44780_VAL_ID &iv, char mode, int32_t val); //--Обработка численной переменной;
uint8_t	getFistBitUint16(uint16_t &reg);									//--Возвращает первый не занятый бит из регистра uint16_t; 
uint8_t pointCursor (uint8_t col, uint8_t row);								//--Вычисление позиции курсора в памяти Lcd	
uint8_t getLenBegin(uint8_t len,uint8_t src,char alig);						//--Возвращает кол-во символов, которые надо добавить спереди;    
uint8_t getLenEnd  (uint8_t len,uint8_t src,char alig);						//--Возвращает кол-во символов, которые надо добавить сзади;
uint16_t CRC16_String(String &s);											//--Расчет контрольной суммы строковой переменной;
char    boolToChar 	(bool val, char sign_0, char sign_1);					//--Конвертор переменной bool в char; 	
int32_t floatToInt32(float val, uint8_t point);								//--Конвертор float в int32_t с умножением на кол-во знаков после точек;
uint8_t getPointFromChar(char val){if(val=='1'){return 1;}	if(val=='2') {return 2;}	if(val=='3'){return 3;}	return 0;}

};//==============  Конец класса ==================================================================
extern RT_HW_LCD_HD44780 RT_HW_Lcd_hd44780; //--Создание объекта LCD типа HD44780
#endif
//=================================================================================================