//====================================================-
//            Тест MCP23017
// Создаваемые объекты:
//    RT_HW_MCP23017_DEVICE     z1; //--Объект на одно устройство mcp23017. Можно создавать несколько объектов с разными именами.
// Используемые функции:
// 1. Функция инициализации и контроля устройства.
//    Вызов: RT_HW_mcp23017.begin(z1);
// 2. Функция настройки устройства. Вызывается один раз. Повторные вызовы игнорируются.
//    Для повторной настройки (если вдруг понадобиться...) - предварительно сбросить флаг z1.init. 
//    Вызов: RT_HW_mcp23017.setID_device(z1, modeA, modeB, period)
//       где z1 - ранее созданный объект.
//        addr (int16_t) - адрес mcp23017 на шине I2c. Может быть установлен любой адрес. Если адрес <0, устройство не используется
//               Обычно mcp23017 выпускаются с базовым адресом 0x20(32 десятичный). Это адрес по умолчанию установлен в библиотеке как "базовый адрес".
//               Пространство адресов может меняться входами a0,a1,a2 mcp23017 - всего восемь адресов.
//               Если устанавливать адрес в диапазоне 0-7, то к этому значению будет прибавлено значение базового адреса.
//        modeA,modeB (char) - режимы работы регистров A и B mcp23017. Могут быть выбраны  шесть режимов - четыре на вход, два на выход.
//          Режим определяется символом:
//          'U' вход с подтяжкой внутренним регистром к Vcc и с инверсией;
//          'u' вход с подтяжкой внутренним регистром к Vcc без инверсии;
//          'I' вход без подтяжки  к Vcc и с инверсией;
//          'i' вход без подтяжки  к Vcc без инверсии;
//          'O' выход прямой
//          'o' выход с инверсией
//        period (int16_t) - период вызова обмена данными с mcp23017 в тиках. Один тик=50мс. 
//              Если период =-1, устанавливается значение по умодчанию (=10 - соотвествует 500 мс).
// 3. Функция установки флаг запуска для созданного объекта (выполнена для удобства работы со средой визуального программрования FLProg).
//    Этот флаг запуска используется в функции RT_HW_mcp23017.device()
//    Вызов: RT_HW_mcp23017.makeRun_device (z1, EN)
//       где z1 - ранее созданный объект;
//           EN(bool) - сигнал разрешения работы устройства.
//    Результат: С периодом =period(в тиках) взводится флаг запуска обмена с mcp23017. 
//                  На время фактического запуска может влиять сложность проекта, однако поддерживается неизменность средняя периодичности с высокой точностью.
// 4. Функция обмена с mcp23017. Обмен производится при взведенном флаге запуска обмена. После выполнении функции, она сбрасывает флаг запуска обмена.
//    Вызов: RT_HW_mcp23017.device(z1, A , B )
//        где z1 - ранее созданный объект;
//            A(byte) - записываемое значение в регистр A, если он настроен как выход
//            B(byte) - записываемое значение в регистр B, если он настроен как выход
//    По завершению функции можно получить значение регистров A=z1.inA и B=z1.inB.
//    Если какой-то регистр настроен на выход, то возвращается установленное значение.
//        
//-------------------------------------------------------------------------------------
// Алгоритм поиска устройства на шине i2c:
//    1.При инициализациии флаг z1.device=0 ("поиск устройства не оуществлялось"). 
//    2. При первом запуске осуществляется поиск устройства на шине i2c. 
//       Если устройство найдено, выполняется повторный (контрольный поиск) чере 50мс 
//         и взводится флаг z1.device=2 ("устройство обнаружено и готово к работе").
//    3. Если устройство не найдено, то делается дополнительные 4-попытки обнаружения с периодом 100мс.
//    4. Если устройство окончательно не найдено, то взводится флаг z1.device=1 ("после поиска устройство не обнаружено").
//    5. В дальнейшем осуществляется поиск устройства с периодичностью 1сек. 
//       Эти запросы на шине i2c по длительности минимально короткие и шину не перегружают. 
//===================================================================================== 
//#include "RT_HW_MAIN.h"     //--Устройство може работать автономно и без использования этой библиотеки
#include "RT_HW_MCP23017.h" //--Подключение библиотеки.

//++++++++Выбор параметров под тип контроллера (в данном тесте определяется контрольный пин для логического анализатора).
#if defined (__AVR_ATmega328__) || defined (__AVR_ATmega328P__)
  enum {pinControl=13};  
#elif  defined(__AVR_ATmega2560__)  || defined(__AVR_ATmega2561__) || defined(__SAM3X8E__)
  enum {pinControl=42};  
#elif defined(ESP8266) || defined(ESP32)
  enum {pinControl=2}; 
#elif defined(_VARIANT_ARDUINO_STM32_) 
  enum {pinControl=PA0}; 
#else
  enum {pinControl=13};   
#endif 
//=================================================== 
RT_HW_MCP23017_DEVICE     z1;       //--Создание объекта 
int16_t addr1=0x20; //0x20 шестнадцатиричный или 32 десятичный
bool blink,eventBack,eventFast;
uint16_t cntBack,cntFast;
uint32_t timeStart1,timeStart2,timeStart3;
uint8_t outA=0x55;
uint8_t outB=0xAA;
uint8_t inA=0;
uint8_t inB=0;
//void printInput(){Serial.println("device="+ String(z1.device)+" vA="+String(z1.valA,HEX)+" vB="+String(z1.valB, HEX));}
//---------------------------------------
void setup(){
pinMode(pinControl,OUTPUT); 
//Wire.begin(); 
}
//---------------------------------------
void loop() {
//======Системная часть===============================
blink=!blink; digitalWrite(pinControl,blink); 
//======Фоновая задача================================= 
outB=z1.inA; //--Устанавливается 
//++++ Блок инициализации и контроля дисплея begin(). Параметры ->lcd,id,col,row,addr;     
RT_HW_mcp23017.begin(z1);
switch(z1.dir){default: case 1: break;  case 0: RT_HW_mcp23017.setID_device(z1,addr1,'U','O',1); break;  // Регистр А - вход, регистр В - выход. 
case 2: RT_HW_mcp23017.makeRun_device (z1,5); RT_HW_mcp23017.device(z1,outA,outB); break;}
//----------------------------------------------------
} // end loop()
//====================================================
