#include "RT_HW_MCP23017.h"
//-------------------------------------------------------------------------------------------------
//          Параметры - адреса для IOCON.BANK=0
//-------------------------------------------------------------------------------------------------
#define MCP_BASE	0x20	//начало адресного пространства MCP23017 (на шине i2c)
#define MCP_IODIRA	0x00	//регистр направления пинов A: =0x00(default) - все на выход, 0xff - все на вход
#define MCP_IODIRB	0x01	//регистр направления пинов B: =0x00(default) - все на выход, 0xff - все на вход
#define MCP_IPOLA	0x02	//регистр инверсии    пинов A: =0x00(default) - все прямые,   0xff - все инверсные
#define MCP_IPOLB	0x03	//регистр инверсии    выводов регистра B: =0x00(default) - все прямые,   0xff - все инверсные
#define MCP_GPPUA	0x0C	//регистр подтяжки входов регистра A резисторами 100кОм: =0x00(default) - не подтянуты,   0xff - все подтянуты
#define MCP_GPPUB	0x0D	//регистр подтяжки входов регистра A резисторами 100кОм: =0x00(default) - не подтянуты,   0xff - все подтянуты
#define MCP_GPIOA	0x12	//регистр выводов  на пины A
#define MCP_GPIOB	0x13	//регистр выводов  на пины B
#define MCP_IOCON   0x0A	//регистр конфигурации  
//-------------------------------------------------------------------------------------------------
#define DEVICE_MAX_TIME_READ    2		//--Максимальное время на считывание регистров
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
//		КОНСТРУКТОР
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
RT_HW_MCP23017:: RT_HW_MCP23017(){
  hive.permit_bus=hive.i2c_ok=hive.spi_ok=false; 	//--Очистка флага инициализации используемых интерфейсов (i2c,SPI);
  hive.busyTick=0;	//--Очистка флага  захвата генератора тиков;
  hive.timePeriod=RT_HW_MCP23017_MILLIS;}; 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		ИНИЦИАЛИЗАЦИЯ И УПРАВЛЕНИЕ УСТРОЙСТВОМ
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_MCP23017:: direct(RT_HW_MCP23017_DEVICE &z,  uint8_t vA, uint8_t vB){
 if(z.ok.permitTick) {genTick();}			//--Вызов "Генератора", если устройство захватило его; 
 // genTick();
switch(z.dir){default: z.dir=0;
  //------Настройка параметров устройства---------------------------------------------
  case 0: if(!z.ok.custom) break;			//--Ожидание настройки параметров устройства;
		  if(z.ok.ban)     break;   		//--Если устройство не может быть использовано;
		  if(!hive.busyTick) {hive.busyTick=true; z.ok.permitTick=true;} //--Захват и инициализация "Генератора временных событий занят";
		  z.dir++;				
  //------Поиск устройства на шине i2c ------------------------------------------------
  case 1: if(!bitRead(z.tuning,6)) {z.ok.find=i2c_find(z.workFind, z.cnt, z.periodFind, z.timeFind, z.adr, 20); if(!z.ok.find) break;} 
	      z.ok.link=true; z.cnt=0; z.dir++; 
  //------Инициализация устройства-----------------------------------------------------
  case 2: init(z); 
		  if(!z.ok.link){z.dir=1; break;}	//--Если потеряна связь, переход повторный поисл;
		  z.ok.permit=true;					//--Установка флага разрешение работы устройства; 
		  z.ok.runOutA=z.ok.runOutB=true; z.cnt=1; z.ok.runDevice=1; //--Для немедленного обновления данных после  
		  z.dir++; 
  //------Рабочий режим---------------------------------------------------------------		
  case 3: if(!z.ok.link)	{z.dir=1; z.ok.permit=0;  break;}		 //Переход, если взведен флаг "Потеря связи";
		  if(z.ok.agoBlink!=hive.blink){z.ok.agoBlink=hive.blink; if(++z.cnt>z.period) {z.cnt=1; z.ok.runDevice=1;}}
		  if(z.ok.runDevice) {z.ok.runDevice=false;					//--Обмен с устройством по флагу z.ok.runDevice;
			if(z.outA!=vA) {z.outA=vA; z.ok.runOutA=true;}			
			if(z.outB!=vB) {z.outB=vB; z.ok.runOutB=true;}	
		    if(!bitRead(z.tuning,7))  {z.ok.runOutA= z.ok.runOutB=true;}
			//z.ok.runOutA= z.ok.runOutB=true;
			device(z); }			//--Обмен с данными с устройством;
		  //---
		  break;
}
}; // END direct(RT_HW_LCD_HD44780_DEVICE &z)		

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
// 			        УСТАНОВКА ID УСТРОЙСТВА 
//adr 		интерфейс i2c -адрес устройства на шине;  интерфейс SPI -адрес пина cs; 
//tuning:	bit0: режим регистра А      0/1 ->ввод/вывод;
//			bit1: режим регистра В      0/1 ->ввод/вывод;		
//			bit2: инверсия региc.А		0/1 ->без инверсии/ с инверсией;
//			bit3: инверсия регис.В 		0/1 ->без инверсии/ с инверсией;	
//			bit4: входа регистра А		0/1 ->без подтяжки/ с подтяжкой к Vcc;	
//			bit5: входа регистра В		0/1 ->без подтяжки/ с подтяжкой к Vcc;
//			bit6: интерфейс подкл.		0/1 ->i2c/SPI;	
//			bit7: режим записи 			0/1 ->в каждом цикле/по изменениям;	
// Режимы ввода:
//  'R' - ввод с инверсией и подятжкой к +Vcc;
//  'r' - прямой ввод;
//  'W' - вывод только по изменению входного значения (для разгрузки шины i2c);
//  'w' - вывод при каждом обращении;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
void RT_HW_MCP23017:: setID_device(RT_HW_MCP23017_DEVICE &z, int8_t adr, char modeBus, char modeA, char modeB, uint8_t period){
  if(z.ok.custom)  {return;} 	z.ok.custom=1;	//--Выход, если устройство инициализировано или установка флага "Параметры устройства установлены";
 z.tuning=0;
  switch(modeA){default:
	  case 'R': bitSet(z.tuning,2); bitSet(z.tuning,4); break;
	  case 'r': break;
	  case 'w': bitSet(z.tuning,0); break;
	  case 'W': bitSet(z.tuning,0); bitSet(z.tuning,7); break;	 	  
  }
   switch(modeB){default:
	  case 'R': bitSet(z.tuning,3); bitSet(z.tuning,5); break;
	  case 'r': break;
	  case 'w': bitSet(z.tuning,1); break;
	  case 'W': bitSet(z.tuning,1); bitSet(z.tuning,7); break;	 	  
  } 
  if(modeBus=='s') bitSet(z.tuning,6); 
  z.period=period;
  z.step.mode=z.tuning & 0x03;
  //--Обработка адреса
  if((adr<0) || (adr>127)) {z.ok.ban=1; return;}
  if((adr>=0) && (adr<=7)) {z.adr=MCP_BASE+adr;} else {z.adr=adr;}	//--Установка адреса: если адрес от 0-7, то это адрес в адресном пространстве MCP_BASE 
  //---Настройка интерфейса 
  if(hive.permit_bus) { 		//--проверка разрешения инициализации интерфейсов (для поддержания внешней инициализации)
  if(bitRead(z.tuning,6)) {pinMode(z.adr,OUTPUT); digitalWrite(z.adr,true); if(!hive.spi_ok){hive.spi_ok=true; SPI.begin(); }}	//--Настройка интерфейса SPI; 		
					 else {                                                 if(!hive.i2c_ok){hive.i2c_ok=true; Wire.begin();}}; //--Настройка интерфейса i2c;
  }
};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		ИНИЦИАЛИЗАЦИЯ УСТРОЙСТВА
//adr   i2c:адрес устройства на шине;  SPI:адрес пина CS; 
//mode:	bit0: режим регистра А      0/1 ->ввод/вывод;
//		bit1: режим регистра В      0/1 ->ввод/вывод;		
//		bit2: инверсия региc.А		0/1 ->без инверсии/ с инверсией;
//		bit3: инверсия регис.В 		0/1 ->без инверсии/ с инверсией;	
//		bit4: входа регистра А		0/1 ->без подтяжки/ с подтяжкой к Vcc;	
//		bit5: входа регистра В		0/1 ->без подтяжки/ с подтяжкой к Vcc;
//		bit6: интерфейс подкл.		0/1 ->i2c/SPI;	
//		bit7: режим записи 			0/1 ->в каждом цикле/по изменениям;	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_MCP23017:: init(RT_HW_MCP23017_DEVICE &z) {
                          hive.buff[0]=MCP_IODIRA; 						//--Адрес регистра, начиная с которого будут записаны четыре параметра
  if(bitRead(z.tuning,0)) {hive.buff[1]=0x00;} else {hive.buff[1]=0xff;}	//--Установка направление регистра A (ввод=0xff/вывод=0x00);
  if(bitRead(z.tuning,1)) {hive.buff[2]=0x00;} else {hive.buff[2]=0xff;}	//--Установка направление регистра В (ввод=0xff/вывод=0x00);
  if(bitRead(z.tuning,2)) {hive.buff[3]=0xff;} else {hive.buff[3]=0x00;}	//--Установка инверсии    регистра A (без инверсии=0x00/с инверсией=0xff);
  if(bitRead(z.tuning,3)) {hive.buff[4]=0xff;} else {hive.buff[4]=0x00;}	//--Установка направление регистра В (без инверсии=0x00/с инверсией=0xff);  
						  hive.qnt=5;		 if(!write(z)) {return;}    	//--Вывод и выход по ошибке;
						  hive.buff[0]=MCP_GPPUA; 						//--Адрес регистра, начиная с которого будут записаны два параметра.
  if(bitRead(z.tuning,4)) {hive.buff[1]=0xff;} else {hive.buff[1]=0x00;}	//--Установка подтяжжки входов регистра A (без подтяжки =0x00/с подтяжкой=0xff);
  if(bitRead(z.tuning,5)) {hive.buff[2]=0xff;} else {hive.buff[2]=0x00;}	//--Установка подтяжжки входов регистра B (без подтяжки =0x00/с подтяжкой=0xff);
  						  hive.qnt=3;		 if(!write(z)) {return;}		//--Вывод и выход по ошибке;
};					
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//            			 ОБМЕН С УСТРОЙСТВОМ
//mode:	bit0: режим регистра А      0/1 ->ввод/вывод;
//		bit1: режим регистра В      0/1 ->ввод/вывод;	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_MCP23017:: device(RT_HW_MCP23017_DEVICE &z){
if(!z.ok.permit) return;	//Serial.print(F("device mode=")); Serial.println(z.step.mode); 
switch(z.step.mode){
  //----Регистр А - ввод, регистр В - ввод---------------------------------------------------
  case 0: hive.buff[0]=MCP_GPIOA; hive.qnt=2; read(z); z.inA=hive.buff[0]; z.inB=hive.buff[1]; break; //--Чтение регистров A и B и выход;	
  //----Регистр A- вывод, регистр B - ввод---------------------------------------------------
  case 1: if(z.ok.runOutA){z.ok.runOutA=false;											//--Проверка готовности данных для регистра A;
		  hive.buff[0]=MCP_GPIOA; hive.buff[1]=z.outA; hive.qnt=2; write(z);}			//--Запись в регистр A;							 
		  hive.buff[0]=MCP_GPIOB; hive.qnt=1; read(z); z.inB=hive.buff[0]; break;		//--Чтение  регистра B и выход;		
  //----Регистр А- ввод, регистр B - вывод---------------------------------------------------
  case 2: if(z.ok.runOutB){z.ok.runOutB=false;											//--Проверка готовности данных для регистра B;
		  hive.buff[0]=MCP_GPIOB; hive.buff[1]=z.outB; hive.qnt=2; write(z);}			//--Запись в регистр B;							 
		  hive.buff[0]=MCP_GPIOA; hive.qnt=1; read(z); z.inA=hive.buff[0]; break;		//--Чтение  регистра A и выход;					
  //--Регистр А - вывод; Регистр В - вывод----------------------------------------------------
  case 3: if((!z.ok.runOutA) && (!z.ok.runOutB)) break; z.ok.runOutA=z.ok.runOutB=false;//--Проверка готовности данных для регистров A и/или B;				
		  hive.buff[0]=MCP_GPIOA; hive.buff[1]=z.outA; hive.buff[2]=z.outB; hive.qnt=3; //--Транзакция для вывода регистров А и В;
		  write(z); break;																//--Вывод и выход;
}};	
//-------------------------------------------------------------------------------------------------
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
// 		Запись регистров в устройство
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
bool RT_HW_MCP23017:: write(RT_HW_MCP23017_DEVICE &z){
  if(hive.qnt>RT_HW_MCP23017_SIZE_BUFF ) hive.qnt=RT_HW_MCP23017_SIZE_BUFF ; //--Защита от доступа за пределом буфера;
  if(bitRead(z.tuning,6)){//---Запись в SPI регистров из буфера hive.buff
						  digitalWrite(z.adr,0); 
						 // SPI.transfer(hive.buff,hive.qnt);  
						  digitalWrite(z.adr,0); return 1;
						  }	//SPI
					else {//---Запись в i2c регистров из буфера hive.buff 
						  Wire.beginTransmission(z.adr); Wire.write(hive.buff,hive.qnt); 										//-i2c
						  if(Wire.endTransmission()!=0) {z.ok.link=false; return 0;} return 1;}};	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
// 		Чтение региcтров из устройства
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
bool RT_HW_MCP23017:: read(RT_HW_MCP23017_DEVICE &z){ 
 z.ok.read=0;
 if(hive.qnt>RT_HW_MCP23017_SIZE_BUFF ) hive.qnt=RT_HW_MCP23017_SIZE_BUFF ; //--Защита от доступа за пределом буфера;
 if(bitRead(z.tuning,6)) 
  {//--Чтение регистров из SPI - результат в буфере hive.buff
						  //digitalWrite(z.adr,0); 
						  //SPI.transfer(hive.buff,hive.qnt); 
						  //SPI.transfer(hive.buff[0]); 
						  //digitalWrite(z.adr,0); 
  z.ok.read=1;}//SPI
  else 
  {	//--Чтение регистров из i2c - результат в буфере hive.buff
	Wire.beginTransmission(z.adr); 	//--Установка связи с устройством;
	Wire.write(hive.buff[0]); 		//--Отправка адреса первого читаемого регистра;
	hive.buff8=Wire.endTransmission();
	if(hive.buff8==0) {z.ok.read=true;} else {z.ok.read=z.ok.link=false;}; //--Выход по ошибке обмена;
	hive.buff8=0; hive.buff[0]=hive.buff[1]=0;
	if(z.ok.read){ 
	Wire.requestFrom((uint8_t)z.adr,(uint8_t)hive.qnt); //--Запрос на чтение регистров; 
	hive.timeStart=RT_HW_MCP23017_MILLIS+DEVICE_MAX_TIME_READ; //-Тайм-аут-->устройство может прислать меньше, чем запрошено;
	while(Wire.available()) {hive.buff[hive.buff8]=Wire.read(); hive.buff8++; 
							 if(hive.buff8>RT_HW_MCP23017_SIZE_BUFF)  {z.ok.read=0; break;} //--Выход, если пришло больше байт
							 if(RT_HW_MCP23017_MILLIS>hive.timeStart) {z.ok.read=0; break;} }//--Выход, если превышен тайм-аут;
  }}
//Serial.print(F("read link,ok:qnt:v1,v2=")); 
//Serial.print(z.ok.link);  	Serial.print(','); 	Serial.print(z.ok.read);  	Serial.print(':');
//Serial.print(hive.qnt);   	Serial.print(':'); 	
//Serial.print(hive.buff[0]); Serial.print(',');	Serial.print(hive.buff[1]); Serial.print(';');
//Serial.print('\n'); 													
return z.ok.read;};	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//                  ПОИСК УСТРОЙСТВА НА ШИНЕ i2c 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool RT_HW_MCP23017:: i2c_find(uint8_t &step, uint8_t &cnt, uint16_t &period, uint32_t &timeDelay,  int8_t adr, uint8_t cntMax){
switch(step){default: step=0;   
  case 0: if(adr<0)	return 0;; 										//--Выход, если не допустимый адрес;
		  cnt=0; period=0; 	timeDelay=RT_HW_MCP23017_MILLIS; step++; 	//--Сброс параметров, запоминаем время, переход;
  case 1: if(!((RT_HW_MCP23017_MILLIS-timeDelay)>period)) return 0; timeDelay=RT_HW_MCP23017_MILLIS; //--Ожидание задержки по времени;
		  cnt++; if(cnt>=cntMax) cnt=cntMax; 						//--Ограничение счетчика сверху;
		  Wire.beginTransmission(adr);								//--Запрос на шину i2c
			if(Wire.endTransmission()==0){ 							//		 и проверка кода ответа (=0 - устройство ответило);
			if(cnt <2) {period=50;			return 0;}				//--Выход для повторного поиска (для надежности) или успешное завершение;
			if(cnt==2) {step=0; 			return 1;}				//--
			if(cnt >2) {period=50;   cnt=1; return 0;}}				//--Успешное завершение;
		  period =100*cnt;					return 0;}};			//--Выход для повторного поиска
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
//	 ГЕНЕРАТОР ВРЕМЕННЫХ СОБЫТИЙ И МЕАНДРА ДЛЯ УПРАВЛЕНИЯ УСТРОЙСТВОМ 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
void RT_HW_MCP23017:: genTick(){hive.tick=hive.tick500=false;	
  if(RT_HW_MCP23017_MILLIS>=hive.timePeriod) {hive.timePeriod=RT_HW_MCP23017_MILLIS+RT_HW_MCP23017_PERIOD_TICK; 
    hive.tick  =true; hive.blink  =!hive.blink; if(++hive.cntTick> 50){hive.cntTick=1; hive.tick500  =true; hive.blink500  =!hive.blink500;}}
	};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
RT_HW_MCP23017 RT_HW_mcp23017;//--Создание внешнего объекта	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++



