/**************************************************************************************************
 *       БИБЛИОТЕКА ДЛЯ РАСШИРИТЕЛЯ ВВОДА/ВЫВОДА
 *					mcp23017   
 ***********************************************************
  Написана ecoins (ecoins@mail.ru) 
   2019-01-21: Начальная версия
   2018-08-12: Оптимизированая под работу  с FLProg
   2019-06-10: Полностью изменен подход
   2019_07_18: Добавлено чтение входов
   2019_07_30: Полностью переписана
   2019_09_30: Выполнена как автономная библиотека.
   2019_10_28: Добавлены режимы настройки регистров.
   2020.04.05: Локальные усовершенствования.
   2020.08.07: Значение в выходных регистрах обновляется при сбоях линии связи. 
 ******************************************************************/
#ifndef RT_HW_MCP23017_h
#define RT_HW_MCP23017_h

#if defined(ARDUINO) && (ARDUINO >= 100)
#include <Arduino.h>
#else
#include <WProgram.h>
#endif
//-------------------------------------------------------------------------------------------------
#include <Wire.h>
#include <SPI.h>
//-------------------------------------------------------------------------------------------------
#define RT_HW_MCP23017_VERSION 			24			//--Номер версии библиотоеки
#define RT_HW_MCP23017_MAX_DEVICE  		15			//--Максимальное кол-во устройств
#define RT_HW_MCP23017_PERIOD_TICK  	10			//--Период системного времени в ms;
#define RT_HW_MCP23017_MILLIS			millis()	//--Время в миллисекундах;
#define RT_HW_MCP23017_MICROS			micros()	//--Время в микросекундах;
#define RT_HW_MCP23017_SIZE_BUFF        6			//--Размер буфера обмена с устройствами;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	СТРУКТУРА RT_HW_MCP23017_ID_DEVICE  - параметры устройства mcp23017;
//mode:	bit0: интерфейс подключения 0/1 ->i2c/SPI;
//		bit1: режим регистра А      0/1 ->ввод/вывод;
//		bit2: режим регистра В      0/1 ->ввод/вывод;		
//		bit3: инверсия регистра А	0/1 ->без инверсии/ с инверсией;
//		bit4: инверсия регистра В 	0/1 ->без инверсии/ с инверсией;	
//		bit5: входа регистра А		0/1 ->без подтяжки/ с подтяжкой к Vcc;	
//		bit6: входа регистра В		0/1 ->без подтяжки/ с подтяжкой к Vcc;	
//		bit7: режим записи 			0/1 ->в каждом цикле/по изменениям;		
//adr:	интерфейс i2c=адрес на i2c; интейфес SPI=пин cs;  
//qnt:	кол-во предаваемых/принимаемых байт; 	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
struct  RT_HW_MCP23017_DEVICE {   //--16 байт		  
struct {uint8_t find:1, mode:3, device:3;} step; 								//--Номера шагов для программ управления дисплеем; 
struct {uint16_t custom:1,permit:1,ban:1,find:1,link:1,permitTick:1, agoBlink:1, runDevice:1, runOutA:1, runOutB:1, read:1, write:1;}ok; //--Флаги состояний;				   
uint8_t	dir, tuning, period, inA, inB, outA, outB; 	//--9 байт
int8_t	adr;	
uint8_t	 workFind, cnt; 		   
uint16_t periodFind;		//--Рабочий регистр для программы find();     
uint32_t timeFind;  
};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		КЛАСС: RT_HW_MCP23017 	- функции управления устройством mcp23017
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
class RT_HW_MCP23017{
public:
// ****** ДАННЫЕ *********************************************
#pragma pack(push, 4)			//--Установка выравнивания до 4 байт
struct { //--12 байт
	uint8_t  busyTick:1, tick:1, tick500:1, blink:1, blink500:1, permit_bus:1, i2c_ok:1, spi_ok:1;  
	uint8_t  cntTick;  
	uint8_t  buff8;
	uint8_t  qnt;
	uint8_t  buff[RT_HW_MCP23017_SIZE_BUFF ];
	uint32_t timePeriod; 
	uint32_t timeStart;
} hive;
#pragma pack(pop)		//--Восстановление предыдущей настройки		
// ****** Методы ******
RT_HW_MCP23017(); 							//--Конструктор
//void    setID_device(RT_HW_MCP23017_DEVICE &z, int8_t adr, uint8_t period); 
void 	setID_device(RT_HW_MCP23017_DEVICE &z, int8_t adr, char modeBus, char modeA, char modeB, uint8_t period);    
//------Управление устройством---------------------------------------------------------------------
void 	genTick(); 														//--Генератор временных событий и меандра для управления устройством;
bool    i2c_find(uint8_t &step, uint8_t &cnt, uint16_t &period, uint32_t &timeDelay, int8_t adr, uint8_t cntMax); //--Поиск устройства на шине i2c;	
void    direct      (RT_HW_MCP23017_DEVICE &z,  uint8_t vA, uint8_t vB);//--Управление устройством;
void    init     	(RT_HW_MCP23017_DEVICE &z);							//--Инициализация устройства;
void    device		(RT_HW_MCP23017_DEVICE &z); 						//--Обмен данными с устройством
bool    write       (RT_HW_MCP23017_DEVICE &z);
bool    read		(RT_HW_MCP23017_DEVICE &z);
};  //---END 
//==============  Конец класса ======================================================
extern RT_HW_MCP23017 RT_HW_mcp23017; //--Создание объекта LCD типа mcp23017
//===================================================================================
#endif //---RT_HW_MCP23017_h


