#include "RT_HW_HC_SR04.h"
//-------------------------------------------------------------------------------------------------
//          Параметры
//-------------------------------------------------------------------------------------------------
#define SR04_timePulse	     10	//--Длительность (mcs) генерируемого импульса на пине Trig 
#define SR04_maxBegin       700	//--Максимальное время (mcs) появления ответного импульса Echo 
#define SR04_minBegin       200	//--Минимальное время  (mcs) появления ответного импульса Echo 
#define SR04_maxDurate    25000	//--Максимальная длительность ответного импульса
#define SR04_convert        582 //--Коэффициент пересчета мкс в мм
#define SR04_coeffCmToMcs	 70 //--Коэффициент преобразования 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		Функция комплексной работы с сенсором;	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_HC_SR04:: meter(RT_HW_HC_SR04_ID &id, bool EN,int8_t Echo, int8_t Trig, uint16_t period,  bool permitIRQ, uint16_t maxLenNoIRQ){
  switch(id.dir){default:
	case 0: //--Инициализация устройства и переход на этап 1, если все нормально, или на этап 2, если устройство не может быть использовано.
			setID(id, EN, Echo, Trig, permitIRQ, maxLenNoIRQ); 
			if(id.custom) id.dir=1; else {break;}
	case 1: //--Ожидание флага запуска измерений и генерация импульса Trig id.run. Этот флаг сбрасывается после заверешения измерения.
			if(EN){if(id.agoEN!=EN) {id.agoEN=1;                            id.timePeriod=millis(); id.run=1;} 
                               else {if(millis()>=(id.timePeriod+ period)) {id.timePeriod=millis(); id.run=1;}}} else {id.run=0;} 
			if(id.run) {id.dir=2;} else {id.start=false; break;} //--Переход на измерение по готовности флага id.run;
	case 2: //--Генерация запускающего импульса--------------------------------
			id.start=id.begin=id.end=false; id.stepIRQ=0; id.codeCur=0;
			digitalWrite(id.pinTrig, LOW);				//--Пин в низкий уровень, чтобы не было помех при настраивании на выход;         
			pinMode		(id.pinTrig, OUTPUT );			//--Настраиваем как выход;	
			digitalWrite(id.pinTrig, HIGH);   			//--Устанавливаем высокий уровень (начало генерации запускающего импульса);      
			delayMicroseconds(SR04_timePulse);			//--Задержка на длительность генерируемого импульса на пине Trig; 
			digitalWrite(id.pinTrig, LOW);    			//--Завершаем запускающий импульс;
			pinMode		(id.pinEcho, INPUT_PULLUP);		//--Настраиваем пин Echo как вход с подтяжкой к Vcc;
			id.timeStart=id.timeBegin=id.timeEnd=RT_HW_HC_SR04_MICROS; 	//--Установка начальных времен для измерения параметров ответного импульса:
			if(digitalRead(id.pinEcho)){id.codeCur=2;}	//--Проверка на ошибку: высокий уровень при наличии датчика не должен появиться ранее чем через 400mcs;
			if(id.codeCur) {processErr(id); id.run=0; id.dir=1; break;} //--Выход если ошибка;		
			id.start=true;  id.dir=3;					//--Разрешение обработки на входе с прерыванием и переход на следующий этап;
			id.dir=3;									//--//--Переход на следующий этап.
	case 3: if(id.modeIRQ) RT_HW_hc_sr04.meterIRQ(id);	else RT_HW_hc_sr04.meterNoIRQ(id);	//--Вызов блока измерения параметров обратного echo-импульса;
			if(id.run) break;							//--Выход, если результат не готов;
			processErr(id);								//--Обработка на ошибки.
			id.dir++;									//--Переход на следующий этап.
	case 4:	if(timeOut(id.timeBegin,SR04_maxDurate) || (!digitalRead(id.pinEcho))) {id.dir=1;}  break; //--Ожидание завершения ответному сигналу или по тайм-ауту;	
}};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//           Инициализация устройств	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_HC_SR04:: setID(RT_HW_HC_SR04_ID &id, bool EN, int8_t Echo, int8_t Trig, bool permitIRQ, uint16_t maxLenNoIRQ) {
  if((!EN) || (Trig<0) || (Trig>=100) ) {id.custom=0; id.fault=1; id.codeErr=1; id.lenCur=maxLenNoIRQ; return;}	//-ВЫХОД, если пин Echo не определен;		  
  id.pinTrig=Trig; if(Echo<0) id.pinEcho=id.pinTrig; else id.pinEcho=Echo;				//--Установка номеров пинов Echo и Trig; 
  id.numIRQ=digitalPinToInterrupt(id.pinEcho);											//--Если пин не имеет прерывания, возвращается -1;
  if((id.numIRQ>=0) && permitIRQ) {id.modeIRQ=id.runIRQ=1; id.lenMaxCm =450;}			//--Параметры для пина c прерыванием (если работа с прерыванием разрешена); 	
                             else {id.modeIRQ=id.runIRQ=0; id.lenMaxCm =maxLenNoIRQ;}	//--Параметры для пина без прерывания; 
  id.lenMaxMcs=id.lenMaxCm * SR04_coeffCmToMcs;											//--Расчет максимального времени в mcs;
  //Serial.print("setID id.lenMaxMcs="); Serial.println(id.lenMaxMcs);
  id.custom=id.fresh=id.run=true;};											//--Установка флагов;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		Измерение параметров ответного эхо-импульса на пине echo, без аппаратного прерывание;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_HC_SR04:: meterNoIRQ(RT_HW_HC_SR04_ID &id) {				  				
  //--Ожидание переднего фронта ответного импульса Echo с тайм-аутом;                        
  do{if(digitalRead(id.pinEcho)) {id.timeBegin=RT_HW_HC_SR04_MICROS; id.begin=true; break;} // Фиксация времени переднего фронта и установка флага прихода переднего фронта
     if(timeOut(id.timeStart,SR04_maxBegin)) {break;}} while(1);	
  //--Ожидание  заднего  фронта ответного импульса Echo с тайм-аутом;    			
  if(id.begin){
  do{if(!digitalRead(id.pinEcho)){id.timeEnd  =RT_HW_HC_SR04_MICROS; id.end  =true; id.start=false; break;}	
	 if(timeOut(id.timeBegin,id.lenMaxMcs)){id.start=false; id.timeEnd=id.timeBegin+id.lenMaxMcs; id.end=true; break;}} while(1);}
  //--Завершение измерения 
  id.run=0; }; //--Выход
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		Измерение параметров ответного эхо-импульса на пине echo, без аппаратного прерывание;
//	Примечание: в скетч или в блок FLProg должны быть включены функции 
//  			IRQ_start()[Запуск обрабочтка прерывания] и IRQ_handler() [//--Обработчик прерывания];
//		void IRQ_start()  {id.runIRQ=false; attachInterrupt(id.numIRQ, IRQ_handler, CHANGE);};
//		void IRQ_handler(){if(!id.start) return;
//   					   if(!id.begin){if( digitalRead(id.pinEcho)) {id.timeBegin=RT_HW_HC_SR04_MICROS; id.begin=true;}}
//     								else{if(!digitalRead(id.pinEcho)) {id.timeEnd  =RT_HW_HC_SR04_MICROS; id.end  =true;  detachInterrupt(id.numIRQ);}}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool RT_HW_HC_SR04:: meterIRQ(RT_HW_HC_SR04_ID &id) {if((id.end) || timeOut(id.timeStart, SR04_maxDurate)) id.run=0; return id.run;};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		   Обработка результатов измерения.
//  До трех  подряд ошибочных измерений отбрасываются.
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++  
void RT_HW_HC_SR04:: processErr(RT_HW_HC_SR04_ID &id) {uint32_t g;          
  g=RT_HW_hc_sr04.getPastTime(id.timeStart,id.timeBegin);
  if(!id.codeCur) {if(!id.begin)                      {id.codeCur=3;}	//--Нет отклика от сенсора в течении
					       else {if(g<=SR04_minBegin) {id.codeCur=4;}	//--Передний фронт раньше 200 mcs после id.start;  
								 if(g >SR04_maxBegin) {id.codeCur=5;}}}	//--Передний фронт позже  700 mcs после id.start; 
  if(!id.codeCur) {if(!id.end)	                      {id.codeCur=6;}}	//--Нет отклика от сенсора в течении тайм-аута;
  //--Обработка кода ошибки и установления текущей дистанции;   
  if(id.codeCur) {if(id.cntErr <7) {id.cntErr++;} 
                  if(id.cntErr>=3) {id.codeErr=id.codeCur; id.lenCur=id.lenMaxCm; id.fault=1;}}
			else {id.cntErr=0; id.codeErr=0; id.fault=0; id.lenCurMcs=id.timeEnd-id.timeBegin; g=(uint32_t)id.lenCurMcs *10/SR04_convert; //--Сброс параметров и конвертация мкс в см.
				  if(g>id.lenMaxCm) {id.lenCur=id.lenMaxCm;} else {id.lenCur=g;}} //--Присвоение текущего значения. 
 
 // Serial.print(" flag begin,end="); Serial.print(id.begin);	Serial.print(',');	Serial.print(id.end);
 // Serial.print(" codeErr="); 		Serial.print(id.codeErr);
 // Serial.print(" id.lenCurMcs="); 	Serial.print(id.lenCurMcs);
 // Serial.print(" id.lenCur="); 		Serial.print(id.lenCur);
 // Serial.print(" g="); 				Serial.print(g);
 // Serial.print(" t1,t2="); Serial.print(id.timeBegin-id.timeStart);Serial.print(',');Serial.print(id.timeEnd-id.timeBegin);
 // Serial.print(" time:start,begin,end="); Serial.print(id.timeStart);Serial.print(',');Serial.print(id.timeBegin);Serial.print(',');Serial.print(id.timeEnd);
 // Serial.print('\n');
				  
				  
};				   	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  	Вычисление времени между двумя временными точками
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
uint32_t RT_HW_HC_SR04:: getPastTime(uint32_t &timeStart, uint32_t timeEnd){
	if(timeEnd >= timeStart) return (timeEnd-timeStart); else return (0xFFFFFFFF-timeStart+timeEnd);};  
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		Тайм-аут времени timeMax mcs относительно timeStart;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool RT_HW_HC_SR04:: timeOut(uint32_t &timeStart, uint16_t timeMax){uint32_t timeEnd; timeEnd=RT_HW_HC_SR04_MICROS;
	if(timeEnd >= timeStart) {if((timeEnd-timeStart)>timeMax) return 1; else return 0;}
	                    else {if((timeStart-timeEnd)>timeMax) return 1; else return 0;}}; 
//---------------------------------------------------------------------------------------		
RT_HW_HC_SR04 RT_HW_hc_sr04;//--Создание внешнего объекта	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++






