/******************************************************************
  Библиотека для семейства сенсоров - ультразвуковые дальномеры типа HC-SR04
Версия 3.1
ecoins@mail.ru 28.05.20 - полностью обновленная версия.
 -------------------------------------------------------------------
Измеряет расстояние до ближайшего препятствия 
  (макс. 100 или 450 см в зависимости от типа выбранного пина контроллера):
1.Возвращает на выходе расстояние в см.
2.Если сенсор не подключен или три раза подряд  ошибки измерения, то взводится флаг "Неисправность",
  расстояние на выходе устанавливается =100. После первого успешного измерения флаг сбрасывается.
3.Работа с сенсором осуществляется через два пина, к которым подключаются выводами сенсора Trig и Echo. 
   Если значение входа Echo <0 или >=100, 
   то считается, что номер  пина подключения вывода сенсора Echo 
               равен номеру пина подключения вывода сенсора Trig. 
4.Работа также возможно через один пин. 
  В этом случае вывод Echo нужно соединить с выводом Trig через резистор 470-510ом. 
  Подключение к контроллеру осуществляется через один пин, к которому подключается вывод сенсора Trig. 
  При этом считается, что номер пина подключения вывода Echo равен номеру пина подключения вывода Trig. 
5.Измерение не осуществляется:
     вход  разрешения EN=0;
	 вход  trig<0 или >=100. 
6.Если пин подключения вывода сенсора Echo поддерживает прерывание, 
  то длительность работы программы измеряется не более 100 микросекунд (в зависимости от типа контроллера). 
  Измеряемое расстояние при этом может быть до 450см.
7.Если пин подключения вывода сенсора Echo не поддерживает прерывание, 
  то длительнось работы программы зависит от расстояния до препятствия (программно ограничено 100см), 
  и может достигать 7 мс.
8. Тестировалось на контроллерах AV328(Nano,UNO), MEGA2560, Arduino DUE, ESP8266, ESP32, STM32F103x.  
   К контроллерам с Vcc=3.3в (Arduino DUE, ESP8266, ESP32) сенсор подключается через преобоазователь уровня 
   или используется сенсор с подержкой 3.3в -HC-SR04+, HC-SR05. 
9. Используемая библиотека RT_HW_HC_SR04.h поддерживает неограниченное кол-во сенсоров.   
10.При одновременном применении нескольких сенсоров с пересекаемыми зонами измерения,
   для избежания наложения эхо сигналов, рекомендуется опрос распределить во времени (примерно через 35 мс). 
   Для этого рекомендуется использовать диспетчер задач ecoins.  
-----------------------------------------------------------------
 ecoins@mail.ru 28.05.20 - полностью обновленная версия. 
 Библиотека RT_HW_HC_SR04.h  версия 3.1
******************************************************************/
#ifndef RT_HW_HC_SR04_h
#define RT_HW_HC_SR04_h

#if defined(ARDUINO) && ARDUINO >= 100 
#include <Arduino.h>
#else
#include <WProgram.h>
#endif
#define RT_HW_HC_SR04_VER 31				//--Номер версии;
#define RT_HW_HC_SR04_MICROS	micros()	//--Получение текущего времени в mcs;
#define RT_HW_HC_SR04_MILLIS	millis()	//--Получение текущего времени в ms;
//-------------------------------------------------------------------------------------------------
//		Структура данных сенсора HC-SR04 (32 байта)
//-------------------------------------------------------------------------------------------------
struct RT_HW_HC_SR04_ID {
uint8_t 	dir;
int8_t		pinEcho, pinTrig, numIRQ;
uint16_t   	custom:1, start:1, begin:1, end:1, run:1, agoEN:1;
uint16_t    fault:1, fresh:1, modeIRQ:1, runIRQ:1, stepIRQ:3, cntErr:3, codeCur:3, codeErr:3;
uint16_t 	lenCurMcs, lenCur, lenMaxCm, lenMaxMcs; 
uint32_t	timeStart, timeBegin, timeEnd, timePeriod; 
};
//-------------------------------------------------------------------------------------------------
//			             КЛАСС СЕНСОРА HC-SR04
//-------------------------------------------------------------------------------------------------
class RT_HW_HC_SR04{
public:
// ****** Методы ******
RT_HW_HC_SR04(){}; //--Конструктор	
void	meter		(RT_HW_HC_SR04_ID &id, bool EN, int8_t Echo, int8_t Trig, uint16_t period=100, bool permitIRQ=1, uint16_t maxLenNoIRQ=100);	//--Функция комплексной работы с сенсором;
void	setID		(RT_HW_HC_SR04_ID &id, bool EN, int8_t Echo, int8_t Trig,                      bool permitIRQ,   uint16_t maxLenNoIRQ); 	//--Инициализация данных устройтсва
void	meterNoIRQ	(RT_HW_HC_SR04_ID &id);			//--Измерение параметров ответного эхо-импульса на пине echo, без аппаратного прерывание;
bool	meterIRQ 	(RT_HW_HC_SR04_ID &id);			//--Измерение параметров ответного эхо-импульса на пине echo, который имеет аппаратное прерывание;
void	processErr	(RT_HW_HC_SR04_ID &id);			//--Обработка результатов измерения;	
//-------------------------------------------------------------------------------------------------
uint32_t getPastTime (uint32_t &timeStart, uint32_t timeEnd);	//--ВЫЧИСЛЕНИЕ РАЗНИЦЫ МЕЖДУ ДВУМЯ ВРЕМЕННЫМИ ТОЧКАМИ 
bool	timeOut		(uint32_t &timeStart, uint16_t timeMax); 	//--Тайм-аут времени timeMax mcs относительно timeStart;
};
//-------------------------------------------------------------------------------------------------
extern RT_HW_HC_SR04 RT_HW_hc_sr04; //--Создание объекта LCD типа HD44780
//-------------------------------------------------------------------------------------------------
#endif
//-------------------------------------------------------------------------------------------------
