#include "RT_HW_DHTx.h"
//-------------------------------------------------------------------------------------------------
//          Параметры
//-------------------------------------------------------------------------------------------------
#define DHT22_DELAY_BEFORE_METER 700 //--мкс Время притягивания входа датчика к GND перед началом измерения
#define DHTx_MAX_QNT_ERR    	   3 //--ед. Максимальное кол-во oшибок
#define DHTx_NAN_INT		   	   1 //--результат при ошибках и  отсутствие датчиклв;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//-------------------------------------------------------------------------------------------------
//                           ИНИЦИАЛИЗАЦИЯ
//-------------------------------------------------------------------------------------------------
void RT_HW_DHTx:: setID_device(RT_HW_DHTx_ID &id, int8_t pin) {
id.codeErr=id.cntErr=id.cntMeter=0;						//--Очистка регистров
id.temperature=id.humidity=DHTx_NAN_INT;				//--Установка начальных значений температуры и влажности
id.error=1; id.fresh=1; id.fault=id.agoEN=0;				//--Готовность данных для вывода
if((pin<0)|| (pin>100)) {id.custom=0;} else {id.custom=1;}	//--Установка состояние устройства: =1 -не может быть настроен; =2 -готово к работе
};
//-------------------------------------------------------------------------------------------------
//            ЧТЕНИЕ СЕНСОРА В РЕЖИМЕ RT
// 1.Линия сенсора: притягивается к земле (600mсs); переводится в режим чтени; последовательно принимается 83 изменения линии.
//	 Если нет линия не меняется более 90mcs - фиксируется ошибка с кодом 2.
//    FALLING, RISING и FALLING - это стартовый бит, затем 40 бит: 
// 2. Проверяется контрольная сумма ---> codeErr=3.
// 3. Проверяется диапазон (температура -50.0/+125.0 градусов по Цельсию, влажность 0/100.0 %);
// 4. При отсутствии ошибок запоминаются полученные результаты и при необходимости взводится флаг изменившихся данных (id.fresh);
// 5. После  ошибочных чтений, превышающих maxQntErr, результатам T и H присваивается DHTx_NAN_INT
// Коды ошибок: =0 нет ошибки; =1 не верный пин;=2 ошибка по тайм-ауту 90mcs при измерении; =3 ошибка КС; =4 выход за диапазон измерений;
//-------------------------------------------------------------------------------------------------
bool RT_HW_DHTx:: meter(RT_HW_DHTx_ID &id, bool EN, int8_t pin, uint16_t period, uint8_t maxQntErr){
  switch(id.dir){default: 
	case 0: setID_device(id, pin);
			if(id.custom) id.dir=1; else {id.codeErr=1; id.fault=1; id.dir=7; break;}
	case 1: if(EN){ if(id.agoEN!=EN) {id.agoEN=1; id.run=1; id.timePeriod=millis();} 
                    else {if(millis()>=(id.timePeriod+ period)) {id.timePeriod=millis(); if(!id.run) {id.run=1;}}}}
			if(!id.run) {break;} id.run=0;
	//--Измерение------------------------------------------------		
			digitalWrite(pin,LOW); pinMode(pin,OUTPUT); 	//--Притягивание линия устройства к земле 
			delayMicroseconds(DHT22_DELAY_BEFORE_METER);	//--Удержание фиксированное время.
			pinMode(pin,INPUT);  digitalWrite(pin,HIGH);	//--Перевод пина устройства в режим чтения
			read(id,pin,maxQntErr);							//--Чтение последовательности данных от устройства (без прерываний - примерно 4мc).
			break;											//--Завершения цикла измерения.
	//--Заглушка (когда пин <0)---------------------------------	
	case 7: break;		
} return id.fault;}; 
//-------------------------------------------------------------------------------------------------
//                     ИЗМЕРЕНИЯ НА ПИНЕ БЕЗ ПРЕРЫВАНИЙ
//-------------------------------------------------------------------------------------------------
void RT_HW_DHTx:: read(RT_HW_DHTx_ID &id, int8_t pin, uint8_t maxQntErr) {uint8_t age; uint32_t timeDelay;	int8_t i; uint16_t rawHumidity, rawTemperature, data; int16_t _temperature;
//--Начала цикла изменений на входе (83 раз) - первые три - стартовый импульс
rawHumidity=rawTemperature=_temperature=0;
data=0;	id.error=id.fresh=0; 

for( i=-3; i<80; i++) {timeDelay=RT_HW_DHTx_MICROS;
 do{age = getPastTime(timeDelay, RT_HW_DHTx_MICROS); 
 if(age >90){id.codeErr=2; id.error=1; break;}} while (digitalRead(pin) == (i & 1) ? HIGH : LOW ); //--Ожидание импульса с тайм-аутом 90мкс(ошибка тайм-аута codeErr=2)      
 if(id.error) break;
 if(i>=0 && (i&1)) {data<<=1; if(age>30){data|=1;}}		//--Проверяем изменение на линии (первые три стартовых импульса пропускаем)
 if(i==31){rawHumidity=data;}
 if(i==63){rawTemperature=data; data = 0;}}
 //--Проверка на контрольную сумму
 if(!id.error) {if( (uint8_t)(((uint8_t)rawHumidity) + (rawHumidity >> 8) + ((uint8_t)rawTemperature) + (rawTemperature >> 8)) != data ) 
					{id.codeErr=3; id.error=1; }}
 //---Проверка и обработка результата измерения
 if(id.error)  {if(id.cntErr<15) id.cntErr++; if(id.cntErr> maxQntErr){id.fault=1; id.temperature=id.humidity=DHTx_NAN_INT;}}  
          else {if (rawTemperature & 0x8000 ) { _temperature = -(int16_t)(rawTemperature & 0x7FFF);} else {_temperature=rawTemperature;}
		        if( (rawHumidity <0) || (rawHumidity > 1000) || (_temperature < -500) || (_temperature > 1250)) {id.codeErr=4; id.error=1;}}
 //--Обновление положительного рзультата		  
 if(!id.error) {id.fault=0; id.cntErr=0; id.codeErr=0; id.cntMeter++;
	            if((id.humidity !=rawHumidity) || (id.temperature !=_temperature)) {id.fresh=true; id.humidity=rawHumidity; id.temperature=_temperature; }}
		  else {id.cntMeter=0;}
};
//---------------------------------------------------------------------------------------
//  	ФУНКЦИИ ВОЗВРАЩАЕТ ПРОШЕДШЕЕ ВРЕМЯ
//--------------------------------------------------------------------------------------- 
uint32_t RT_HW_DHTx:: getPastTime(uint32_t &timeStart, uint32_t timeEnd){
	if(timeEnd >= timeStart) return (timeEnd-timeStart); else return (0xFFFFFFFF-timeStart+timeEnd);}; 
//---------------------------------------------------------------------------------------		
RT_HW_DHTx RT_HW_dhtx;//--Создание внешнего объекта	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++