/**************************************************************************************************
Датчик температуры и влажности типа DHT21. 
------------------------------------------------------------------
Не поддерживает датчик DHT11 (к нему много претензий по точности и качеству) 
Измеряет температуру и влажность воздуха:
   1. Возвращает температуру воздуха в градусах С х10 (Выход T - формат int).
      Для перевода в формат float результат нужно поделить на 10.
   2. Возвращает влажность воздуха в % х10 (Выход H - формат int).
      Для перевода в формат float результат нужно поделить на 10.
   3. По конструктивным особенностям сенсора вызовы не должны бвть чаще чем 1 раз в 2 сек.   
   4. Запущенный процесс измерения занимает примерно 5,7мс - в это время процессор занят и реагирует 
      только на аппаратные прерывания, время обработки которого не должен превышать 30-50мкс - 
      это может привести к потери сигнала с сенсора и соответственно приведет к ошибке измерения. 
   5. После нескольких подряд ошибочных измерений (определяется параметром maxQntErr - по умолчанию=3),
      устанавливается ошибка (Выход no - формат bool).), 
      которая будет держаться до первого успешного измерения. 
      При ошибке выходным значениям T и H присваивается значение =1. 
  6. Если вход EN=0 или pin<0 или pin>=100, то сенсор считается не подключенным: no=1, T=1, H=1.
-----------------------------------------------------------------
Рекомендации - устанавливать на плате, которая вызывается  с периодом >250мс - 1сек. 
-----------------------------------------------------------------
 ecoins@mail.ru 28.05.20 - полностью обновленная версия (3.1) библиотеки RT_HW_DHTx.h  
 Библиотека адаптирована для использования с FLProg.
 ***********************************************************
  Написана ecoins (ecoins@mail.ru) 
  Лист изменений
   2018-01-21: Начальная версия
   2018-08-12: Оптимизированая под работу  с библиотекой RT_Hardware
   2019-11-17: Заново переписано. Версия 2.1. 
   2020-05-23: Заново переписано. Версия 3.1.  
 ******************************************************************/
#ifndef RT_HW_DHTx_h
#define RT_HW_DHTx_h

#if defined(ARDUINO) && (ARDUINO >= 100)
#include <Arduino.h>
#else
#include <WProgram.h>
#endif

#define RT_HW_DHTx_VER 				31		//--Номер версии библиотеки
#define RT_HW_DHTx_MICROS		micros()	//--Макрос измерения времени в mcs
//-------------------------------------------------------------------------------------------------
//			           
// dir =0 нет инициализации; =1 устройство в работе; =2 устройство не может быть использовано;
//-------------------------------------------------------------------------------------------------
struct RT_HW_DHTx_ID{
uint8_t		dir;		//=0 нет инициализации; =1 устройство в работе; =2 устройство не может быть использовано; 
uint8_t 	custom:1, fresh:1, error:1, run:1, fault:1, agoEN:1;
uint8_t 	codeErr:3, cntErr: 4; //--флаг ошибки,код ошибки,счетчик ошибок при чтении(сбрасывается при ок),макс.кол-во ошибок(<=7)
uint8_t 	cntMeter;
uint16_t  	humidity; 
int16_t     temperature;	
uint32_t 	timePeriod;		
};
//++++++++++++++++++++Описание класса DHT_RT++++++++++++++++++++++++++++
class RT_HW_DHTx{
public:
RT_HW_DHTx(){}; 											//--Конструктор
void    setID_device(RT_HW_DHTx_ID &id, int8_t pin=-1);    //--Инициализация сенсора                                     
bool 	meter(RT_HW_DHTx_ID &id, bool EN, int8_t pin=-1, uint16_t period=2200, uint8_t maxQntErr=3); //--Запуск в режиме реального времени   
void 	read		(RT_HW_DHTx_ID &id, int8_t pin, uint8_t maxQntErr);
//--------------------------------------------------------------------------------------------------
uint32_t getPastTime(uint32_t &t1,uint32_t t2);		//--ВЫЧИСЛЕНИЕ РАЗНИЦЫ МЕЖДУ ДВУМЯ ВРЕМЕННЫМИ ТОЧКАМИ 
};  //---END 
//-------------------------------------------------------------
//==============  Конец класса ======================================================
extern RT_HW_DHTx RT_HW_dhtx; //--Создание объекта LCD типа HD44780
//===================================================================================
#endif //---RT_HW_DHTx_h


