#include "RT_HW_BME280.h"
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	1.1. Конструктор
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
RT_HW_BME280::RT_HW_BME280() {}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
//	1.2 Генератор меандра 50 мс и флага перезапуска устройства (periodReboot в тиках=50мс)
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
void RT_HW_BME280:: genEventID(RT_HW_BME280_ID &id){id.event=0; if(RT_HW_BME280_MILLIS>=id.timeStart) {id.event=1; id.blink=!id.blink; id.timeStart=RT_HW_BME280_MILLIS+RT_HW_BME280_PERIOD_TICK;}};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	1.3. Настройка параметров устройства
// t_sb(период опроса)  =0: 0.5ms; =1: 62.5ms; =2: 125ms; =3: 250ms; =4: 500ms; =5: 1000ms; =6: 10ms; =7: 20ms;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_BME280:: setID(RT_HW_BME280_ID &id, char interface, int8_t addr){
if(id.custom==1) {return;} 	
id.custom=1; id.codeErr=id.ban=id.permit=id.mode=id.enSPI=0;	//--Выход-устройство уже настроено; 
initVal(id); id.fresh=1;										//--Присвоение начальных значений параметров;
if (addr<0)   {id.ban=1; return;} id.addr=addr;					//--Недопустимый адрес (номер пина);
//----Настройка интерфейса I2C-------------------------------------------------
if(interface=='I') {id.interface=1; if(!device.i2c_ok) {device.i2c_ok=1; Wire.begin();}}
//----Настройка интерфейса SPI-------------------------------------------------
if(interface=='S') {id.interface=2; id.enSPI=1; digitalWrite(id.addr,1); pinMode(id.addr,OUTPUT);
			        if(!device.spi_ok) {device.spi_ok=1; SPI.begin(); digitalWrite(id.addr,0); SPI.transfer(0); digitalWrite(id.addr,1);}}
//----Настройка интерфейса - параллельный через 4 пина-------------------------
if(interface=='B') {id.interface=3; for(uint8_t i=0; i<4; i++){digitalWrite((id.addr+i),0); pinMode((id.addr+i),OUTPUT);}}
//-----------------------------------------------------------------------------
if(!id.interface)  {id.ban=1; id.codeErr=1;} //--Если интерефейс не определен -> блокирование устройства;
//----Определение значения параметров сенсора (могут быть изменены пользователем)------------------
id.mode 	=0x3;	//--Нормальный режим (циклическое измерение)
id.t_sb		=0x0;	//--Период между измерениями температуры=0.5ms;
id.filter	=0x3;	//--Коэффициент фильтрации=8;
id.osrs_t	=0x3;	//--Коэффициент передискредитации (усреднения) данных температуры=4.При включенном фильтре разрешение АЦП=20 бит.);
id.osrs_p	=0x3;	//--Коэффициент передискредитации (усреднения) данных давления=4. 	При включенном фильтре разрешение АЦП=20 бит.);
id.osrs_h	=0x3;	//--Коэффициент передискредитации (усреднения) данных влажности=4. Разрешение АЦП=16 бит.);
//----Определение периодов опроса и допустимой ошибки----------------------------------------------
id.periodTemper=10; id.periodHum=20; id.periodPress=30; id.periodErr=100;	id.err=1; //--Установка начальных параметров;
id.pressureSeaLevel=1013.25;  							//--Давление на уровне моря в кПа*100;  
id.cntTemper  =id.cntHum  =id.cntPress=id.cntErr=0; 	//--Очистка счетчиков;
id.eventTemper=id.eventHum=id.eventPress=1;				//--Установка флагов разрешения запуска изменения;
//-------------------------------------------------------------------------------------------------
};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	1.4. Управление устройством
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_BME280:: directDevice(RT_HW_BME280_ID &id, bool EN){
genEventID(id); 
//----Установка флагов запуска по времени---------------------------------------------------------
if(id.event){    if(++id.cntTemper > id.periodTemper) {id.cntTemper=1; if(id.periodTemper) {id.eventTemper =1;}}  
			     if(++id.cntHum    > id.periodHum)    {id.cntHum=1;    if(id.periodHum)    {id.eventHum    =1;}}
			     if(++id.cntPress  > id.periodPress)  {id.cntPress=1;  if(id.periodPress)  {id.eventPress  =1;}} 
				 if(!EN) {id.codeErr=7;}
				 if(id.codeErr) {id.cntErr++; 
				 if(id.cntErr    > 100)    {id.cntErr=1; id.err=1; initVal(id); id.fresh=1;}} else {id.cntErr=1;}} 
//----Проверка на признаки аварийного обмена------------------------------------------------------
if(id.codeErr) {if(id.stepDirect) {id.initBegin=0; id.stepDirect=0;}}		 
//----Вызов инициализации устройства (проверка связи, кода ID, чтение корректирующих коэффициентов, установка параметров.
if(id.stepDirect==0) {if(!EN){return;} begin(id); if(id.initBegin) {id.stepDirect++;}};
//----Ожидание события 50мс и одного флагов запросов на измерения ---------------------------------
if(id.stepDirect==1) {if(!id.event) {return;} if(!EN){return;}
					  if(id.eventTemper || id.eventHum || id.eventPress) {id.stepDirect++;} else {return;}}
//----Обработка температуры------------------------------------------
if(id.stepDirect==2) {if(id.eventTemper) {id.eventTemper=0; readTemp(id);  if(id.codeErr) {return;} else {id.stepDirect++; return;}} id.stepDirect=4;} 
if(id.stepDirect==3) {handlerTemp(id); id.stepDirect++; return;}
//----Обработка давления------------------------------------------
if(id.stepDirect==4) {if(id.eventPress)  {id.eventPress=0;  readPress(id); if(id.codeErr) {return;} else {id.stepDirect++; return;}} id.stepDirect=6;} 
if(id.stepDirect==5) {handlerPress(id); id.stepDirect++; return;}
//----Обработка влажности и высоты------------------------------------------
if(id.stepDirect==6) {if(id.eventHum)    {id.eventHum=0;    readHum(id);   if(id.codeErr) {return;} else {id.stepDirect++; return;}} id.stepDirect=8;} 
if(id.stepDirect==7) {handlerHum(id); handlerAero(id); id.stepDirect++;}	
//----Завершение------------------------------------------------------------
if(id.stepDirect==8) {id.stepDirect=1; id.fresh=1; id.codeErr=id.err=0; return;}
};//--Если перезапуск, переход на частичную

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//                  ПОИСК УСТРОЙСТВА НА ШИНЕ i2c 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_BME280:: i2c_find(RT_HW_BME280_ID &id, uint8_t cntMax){
if(id.stepFind==0) {if(id.find) {return;}; id.cntFind=0; id.periodFind=0; id.timeDelay=RT_HW_BME280_MILLIS; id.stepFind++;} 	//--Сброс параметров, запоминаем время, переход;
if(id.stepFind==1) {if((RT_HW_BME280_MILLIS-id.timeDelay)>id.periodFind) {id.timeDelay=RT_HW_BME280_MILLIS;} else {return;}		//--Ожидание задержки по времени;
					id.cntFind++; if(id.cntFind>=cntMax) id.cntFind=cntMax; 				//--Ограничение счетчика сверху;
					Wire.beginTransmission(id.addr);										//--Отправка запроса на шину i2c;
					id.errWire=Wire.endTransmission(); if(id.errWire==0) { 					//		 и проверка кода ответа (=0 - устройство ответило);
					if(id.cntFind <2) {id.periodFind=50;				return;}			//--Выход для повторного поиска (для надежности) или успешное завершение;
					if(id.cntFind==2) {id.stepFind=0; id.find=1;		return;}			//--
					if(id.cntFind >2) {id.periodFind=50; id.cntFind=1;	return;}}			//--Успешное завершение;
					id.periodFind =100*id.cntFind;			  			return;}};			//--Выход для повторного поиска		
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	1.4. Настройка устройства
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_BME280:: begin(RT_HW_BME280_ID &id){
if(id.stepBegin==0) {id.timeDelay=RT_HW_BME280_MILLIS;  id.stepBegin++;}	//--Установка времени задержки и переход на следующий этап; 
//---Пауза 50 мс для нормализации питания и переход 1)если i2c, то поиск устройства на шине i2c; 2)иначе пропуск следующего шага;
if(id.stepBegin==1) {if(((RT_HW_BME280_MILLIS-id.timeDelay)>50))  {if(id.interface==1) {id.find=id.stepFind=0; id.stepBegin=2;} else {id.stepBegin=3;}}}			 
//---Поиск устройства по адресу на шине i2c--------------------------------------------------------
if(id.stepBegin==2) {i2c_find(id,20); if(id.find) {id.link=1; id.stepBegin++;}};
//---Идентификация устройства----------------------------------------------------------------------
if(id.stepBegin==3) {id.vChipID=readByte(id,CHIP_ID);
					 if(!id.link) 		{id.codeErr=1; id.stepBegin=0; return;}
					 id.modeChip=0; 
					 if((id.vChipID==0x56) || (id.vChipID==0x57) || (id.vChipID==0x58)) {id.modeChip=1;} //--Идентификация сенсора типа BMP280;
					 if( id.vChipID==0x60) 												{id.modeChip=2;} //--Идентификация сенсора типа BME280;
					 if(id.modeChip) {id.stepBegin++; } else {id.codeErr=2; id.stepBegin=0;} return;} ;		 //--Переход на обработку если сенсор идентифицирован;
//---Чтение из устройства корректирующих коэффициентов по температуре------------------------------
if(id.stepBegin==4) {uint8_t buff[6];  readBuff(id, buff, DIG_T1_LSB, 6);
					 id.coeff.dig_T1 = ((uint16_t)(buff[1] << 8) + buff[0]);
					 id.coeff.dig_T2 = (( int16_t)(buff[3] << 8) + buff[2]);
					 id.coeff.dig_T3 = (( int16_t)(buff[5] << 8) + buff[4]);
					 id.stepBegin++; return;}					 
//---Чтение из устройства корректирующих коэффициентов по давлению--------------------------------- 												   
if(id.stepBegin==5){uint8_t buff[8];  readBuff(id, buff, DIG_P1_LSB, 8);
					 id.coeff.dig_P1 = ((uint16_t)(buff[1]  << 8) + buff[0]);
					 id.coeff.dig_P2 = (( int16_t)(buff[3]  << 8) + buff[2]);
					 id.coeff.dig_P3 = (( int16_t)(buff[5]  << 8) + buff[4]);
					 id.coeff.dig_P4 = (( int16_t)(buff[7]  << 8) + buff[6]);
					 id.stepBegin++; return;} 					 
if(id.stepBegin==6){uint8_t buff[10]; readBuff(id, buff, DIG_P5_LSB, 10);					 	
					 id.coeff.dig_P5 = (( int16_t)(buff[1] << 8) + buff[0]);
					 id.coeff.dig_P6 = (( int16_t)(buff[3] << 8) + buff[2]);					 
					 id.coeff.dig_P7 = (( int16_t)(buff[5] << 8) + buff[4]);
					 id.coeff.dig_P8 = (( int16_t)(buff[7] << 8) + buff[6]);
					 id.coeff.dig_P9 = (( int16_t)(buff[9] << 8) + buff[8]);
					 id.stepBegin++; return;} 
//---Чтение из устройства корректирующих коэффициентов по влажности--------------------------------					 									
if(id.stepBegin==7){ id.coeff.dig_H1= ((uint8_t )(readByte(id,DIG_H1_LSB)));  id.stepBegin++; return;}					
if(id.stepBegin==8){ uint8_t buff[7]; readBuff(id, buff, DIG_H2_LSB, 7);	
					 id.coeff.dig_H2 = (( int16_t)(buff[1] << 8) +   buff[0]);
					 id.coeff.dig_H3 = (( uint8_t)(buff[2]));
					 id.coeff.dig_H4 = (( int16_t)(buff[3] << 4) +  (buff[4] & 0x0F));
					 id.coeff.dig_H5 = (( int16_t)(buff[5] << 4) + ((buff[4]>> 4 ) & 0x0F));
					 id.coeff.dig_H6 = (( uint8_t)(buff[6]));	 
					 id.stepBegin++; return;}				 					 
//---Запись регистра config(     t_sb[7:5], filter[4:2], sp3w_en[0])------------------------------------
if(id.stepBegin==9) {writeByte(id, CONFIG,    calcConfig(id));   id.stepBegin++; return;}	
//---Запись регистра ctrl_mess(osrs_t[7:5], osrs_p[4:2],  mode[1:0])------------------------------------
if(id.stepBegin==10){writeByte(id, CTRL_MEAS, calcCtrlMeas(id)); id.stepBegin++; return;}
//---Запись регистра ctrl_hum(							osrs_h[2:0])------------------------------------					 
if(id.stepBegin==11){writeByte(id, CTRL_HUM,  calcCtrlHum(id));  id.stepBegin++; return;}
//---Проверка связи 
if(id.stepBegin==12) {if(!id.link) {id.codeErr=1; id.stepBegin=0;}  			  
					 id.stepBegin=0; id.initBegin=1; id.codeErr=0; id.err=0;
					 	//Serial.print(F(" End begin codeErr=")); Serial.println(id.codeErr);
					 return;}					 
} 
//------------------------------------------------------					 

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	1.5. Чтение температуры в градусах с разрешением 0,01. Выходное значение «5123» равно 51,23 град..
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_BME280::readTemp(RT_HW_BME280_ID &id){uint8_t buff[3]; 
if (id.osrs_t==0)	{return;}	
readBuff(id, buff, TEMP_MSB, 3);	if(!id.link) {id.codeErr=3; return;}
id.adc_T  = (uint32_t)buff[0] <<12;	id.adc_T |= (uint32_t)buff[1] << 4; id.adc_T |=(buff[2] >>4) & 0b00001111;}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	1.6. Обработка температуры в градусах с разрешением 0,01.
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_BME280:: handlerTemp(RT_HW_BME280_ID &id){int64_t var1, var2;
var1 =  ((((id.adc_T>>3) - ((int32_t)id.coeff.dig_T1<<1))) * ((int32_t)id.coeff.dig_T2)) >> 11;
var2 = (((((id.adc_T>>4) - ((int32_t)id.coeff.dig_T1)) * ((id.adc_T>>4) - ((int32_t)id.coeff.dig_T1))) >> 12) * ((int32_t)id.coeff.dig_T3)) >> 14;
id.t_fine=var1+var2; id.T=(float)((id.t_fine * 5 + 128)>>8)/100;}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	1.7. Получение значения влажности из сенсора 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_BME280:: readHum	(RT_HW_BME280_ID &id){uint8_t buff[2];
if (id.osrs_h==0) {return;}
readBuff(id, buff, HUM_MSB, 2);	if(!id.link) {id.codeErr=4; return;}
id.adc_H  = (uint16_t)(buff[0] << 8) | buff[1];	}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	1.8. Обработка влажности
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_BME280:: handlerHum (RT_HW_BME280_ID &id){int32_t var1;
var1 = (id.t_fine - ((int32_t)76800));
var1 = (((((id.adc_H<<14)-(((int32_t)id.coeff.dig_H4)<<20)-(((int32_t)id.coeff.dig_H5) * var1)) +((int32_t)16384))>>15) * (((((((var1 * ((int32_t)id.coeff.dig_H6)) >> 10) * 
		(((var1 * ((int32_t)id.coeff.dig_H3))>>11)+((int32_t)32768)))>>10)+((int32_t)2097152)) * ((int32_t)id.coeff.dig_H2)+8192)>>14));
var1 = (var1-(((((var1 >> 15) * (var1 >> 15)) >> 7) * ((int32_t)id.coeff.dig_H1)) >> 4));
var1 = (var1 < 0 ? 0 : var1);
var1 = (var1 > 419430400 ? 419430400 : var1);
id.H = (var1>>12)/1024.0;}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	1.7. Получение значения давления из сенсора 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_BME280::readPress(RT_HW_BME280_ID &id){uint8_t buff[3]; 
if (id.osrs_p==0) {return;}
readBuff(id, buff, PRESS_MSB, 2);	if(!id.link) {id.codeErr=5; return;}
id.adc_P = ((uint32_t)buff[0] << 12) | ((uint32_t)buff[1] << 4) | ((buff[2] >> 4) & 0x0F);}	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	1.8. Обработка давления
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_BME280::handlerPress	(RT_HW_BME280_ID &id){int64_t var1, var2, P;
var1 = ((int64_t)id.t_fine) - 128000;
var2 = var1 * var1 * (int64_t)id.coeff.dig_P6;
var2 = var2 + ((var1 * (int64_t)id.coeff.dig_P5)<<17);
var2 = var2 + (((int64_t)id.coeff.dig_P4)<<35);
var1 = ((var1 * var1 * (int64_t)id.coeff.dig_P3)>>8) + ((var1 * (int64_t)id.coeff.dig_P2)<<12);
var1 = (((((int64_t)1)<<47)+var1))*((int64_t)id.coeff.dig_P1)>>33;
if (var1 == 0){id.P=0; return;}
P = 1048576 - id.adc_P;
P = (((P << 31) - var2)*3125)/var1;
var1 = (((int64_t)id.coeff.dig_P9) * (P >> 13) * (P >> 13)) >> 25;
var2 = (((int64_t)id.coeff.dig_P8) * P) >> 19;
P = ((P + var1 + var2) >> 8) + (((int64_t)id.coeff.dig_P7)<<4);
P = P >> 8; 
id.P=(float)P/100;}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	1.9. Обработка высоты
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_BME280::handlerAero (RT_HW_BME280_ID &id){float h;
float tempOutsideKelvin = id.T+273.15+15;
h= id.P / id.pressureSeaLevel;
h = 1-pow(h, 0.190284);
h = (h * tempOutsideKelvin) / 0.0065;
id.A=h;	}
//-------------------------------------------------------------------------------------------------
//	Запись в регистр сенсора
//-------------------------------------------------------------------------------------------------
void   RT_HW_BME280::writeByte(RT_HW_BME280_ID &id, uint8_t reg, uint8_t val){
if(id.interface==1) {Wire.beginTransmission(id.addr); Wire.write(reg); Wire.write(val);   id.errWire=Wire.endTransmission(); if(id.errWire) {id.link=0;} return;} 	//--Интерфейс i2c;
//if(id.interface==2) {digitalWrite(id.addr,LOW); RT_HW_BME280_SPI_TRANSFER(    reg & 0x7F);      RT_HW_BME280_SPI_TRANSFER(val); 	digitalWrite(id.addr, HIGH); return;}		//--Интерфейс SPI;
//if(id.interface==3) {digitalWrite(id.addr,LOW); 	         spi_Transfer(id,(reg & 0x7F));     spi_Transfer(id,val);              	digitalWrite(id.addr, HIGH); return;}		//--Интефейс 4-пина (программный SPI);
}
//-------------------------------------------------------------------------------------------------
//	Чтение из регистра сенсора
//-------------------------------------------------------------------------------------------------
uint8_t RT_HW_BME280::readByte(RT_HW_BME280_ID &id, uint8_t reg){uint8_t val=0; uint8_t qnt=1;
if(id.interface==1) {Wire.beginTransmission(id.addr); Wire.write(reg); id.errWire=Wire.endTransmission(); if(id.errWire) {id.link=0; return 0;} Wire.requestFrom(id.addr,qnt); val=Wire.read();} //--Интефейс i2c;
if(id.interface==2) {digitalWrite(id.addr,LOW); //RT_HW_BME280_SPI_TRANSFER(    reg | 0x80);  val=RT_HW_BME280_SPI_TRANSFER(0);
					 digitalWrite(id.addr, HIGH); val=0;}	//--Интерфейс SPI;
if(id.interface==3) {digitalWrite(id.addr,LOW);  		     //spi_Transfer(id,(reg | 0x80)); val=spi_Transfer(id,0); 		 
					 digitalWrite(id.addr, HIGH); val=0;}	//--Интерфейс 4-пина (программный SPI);
return val;}
//-------------------------------------------------------------------------------------------------
//	Чтение из регистров в буфер 
//-------------------------------------------------------------------------------------------------
void RT_HW_BME280::readBuff(RT_HW_BME280_ID &id, uint8_t* buff, uint8_t reg, uint8_t qnt){
if(id.interface==1) {Wire.beginTransmission(id.addr); Wire.write(reg); id.errWire=Wire.endTransmission(); if(id.errWire) {id.link=0; return;} 
					 Wire.requestFrom(id.addr,qnt); 
					 for(uint8_t i=0; i<qnt; i++){buff[i]=Wire.read();}
					 } //--Интефейс i2c;
if(id.interface==2) {//digitalWrite(id.addr,LOW); RT_HW_BME280_SPI_TRANSFER(    reg | 0x80);  val=RT_HW_BME280_SPI_TRANSFER(0);digitalWrite(id.addr, HIGH); return val;
}	//--Интерфейс SPI;
if(id.interface==3) {//digitalWrite(id.addr,LOW);  		     spi_Transfer(id,(reg | 0x80)); val=spi_Transfer(id,0); 		        digitalWrite(id.addr, HIGH); return val;
}
}	//--Интефейс 4-пина (программный SPI);

//-------------------------------------------------------------------------------------------------
//	Программый SPI: SS=addr; MOSI=addr+1; MISO=addr+2;	SCK=addr+3;};
//-------------------------------------------------------------------------------------------------
uint8_t RT_HW_BME280::spi_Transfer(RT_HW_BME280_ID &id, uint8_t data)	{uint8_t val=0;
for (int i=7; i>= 0; i--) {val<<= 1; digitalWrite(id.addr, LOW); digitalWrite(id.addr+1, data & (1<<i)); digitalWrite(id.addr+3, HIGH); if (digitalRead(id.addr+2)) {val |= 1;}}
return val;}
//=================================================================================================	
RT_HW_BME280 RT_HW_bme280; //--Создание объекта 

