/*** FLProg ************** Библиотека RT_HW_BME280 ver0.2 для Arduino ***************
 После измерения сенсор выдает некомпенсированные и некалиброванные значения температуры, давления и влажности.
 Для исправления этих значений нужно считать калибровочные коэффициенты, которые прошиваются  при производстве.
 Показания влажности фиксируется на 16 битах. 	
 Разрешение показаний по давлению и температуре повышается до 20 бит внутреннии фильтр IIR (БИХ), 
 который подавляет высокочастотные колебания и повышает разрешение. Коэффициент фильтра (2, 4, 8 или 16) 
 определяет постоянную времени фильтра (см. Таблицу).
 
 BME280 запускается в следующей последовательности:
  1. Выбирается протокол связи (I2C или аппаратный SPI) и настраивается по упрощенной схеме, если он еще не настроен.
  2. Проверяется наличие связи с устройством.
  3. Из устройства считываются коэффициенты компенсации.
  4. Устанавливается настройка БИХ-фильтра* (0,2,4,8).
  5. Настройка коэффициентов передискретизации и режима работы датчика.

  
// Чрезмерная выборка сильно влияет на шум в данных (пожалуйста, обратитесь к таблице для получения дополнительной информации)
// Спецификация BME280 предоставляет рекомендации по настройке для различных приложений
// Наконец мы записываем все эти значения в соответствующие регистры
  
  // 5. Проверьте связь (спросите и проверьте идентификатор чипа)
	
  Библиотеки  разрабатываются с учетом особенностей использования в системе визуального программирования FLProg.
  ecoins@mail.ru 29.09.2019г.
**************************************************************************************************/
//#################################################################################################
#ifndef RT_HW_BME280_h
#define RT_HW_BME280_h
//-------------------------------------------------------------------------------------------------
#if defined(ARDUINO) && ARDUINO >= 100 
#include <Arduino.h>
#else
#include <WProgram.h>
#endif
//#include <avr\pgmspace.h>
//-------------------------------------------------------------------------------------------------
//#if defined(_AVR_)
//#include <util/delay.h>
//#endif
//-------------------------------------------------------------------------------------------------
#include <Wire.h>
#include <SPI.h>
//------Макросы настройки интерфейса SPI-----------------------------------------------------------
//#if defined(ARDUINO_ARCH_STM32)
//#define RT_HW_BME280_SPI_BEGIN                 	SPI.begin()
//#define RT_HW_BME280_SPI_TRANSFER(v)       		SPI.transfer(v)
//#define RT_HW_BME280_SPI_TRANSFER_BUFF(buff,n)	//SPI.transfer(v)
//#else
//#define RT_HW_BME280_SPI_BEGIN                 	SPI.begin()
//#define RT_HW_BME280_SPI_TRANSFER(v)       		SPI.transfer(v)
//#define RT_HW_BME280_SPI_TRANSFER_BUFF(buff,n) 	SPI.transfer(v)	
//#endif
//-------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------
#define RT_HW_BME280_VERSION 			01			//--Номер версии библиотеки
#define RT_HW_BME280_PERIOD_TICK  		50			//--Период системного времени в ms;
#define RT_HW_BME280_MILLIS				millis()	//--Время в миллисекундах;
#define RT_HW_BME280_MICROS				micros()	//--Время в микросекундах;
#define RT_HW_BME280_MAX_TICK_ERR		100			//--Максимальное кол-во тиков с ошибками;
//-------------------------------------------------------------------------------------------------							   
struct RT_HW_BME280_DEVICE 	  {uint16_t i2c_ok:1, spi_ok:1;};
struct RT_HW_BME280_Coefficients {uint16_t dig_T1; int16_t dig_T2, dig_T3;														//--4
								  uint16_t dig_P1; int16_t dig_P2, dig_P3, dig_P4, dig_P5, dig_P6, dig_P7, dig_P8, dig_P9;		//-18
								  int16_t  dig_H1; int16_t dig_H2, dig_H3, dig_H4, dig_H5, dig_H6;};							//-12		
struct RT_HW_BME280_ID {RT_HW_BME280_Coefficients coeff;
	uint32_t timeStart, timeDelay;
	uint8_t  addr;  uint8_t vChipID;
	uint16_t interface:2, custom:1, permit:1, ban:1, link:1, find:1, mode:3, errWire:3, codeErr:3;	//--Интерфейс: i2c=1; SPI=2; Pin=3;
	uint16_t initFind:1,  initBegin:1, event:1, blink:1, eventTemper:1, eventHum:1, eventPress:1, eventAero:1, fresh:1, err:1, modeChip:3;
	uint16_t t_sb:3, filter:3, osrs_t:3, osrs_p:3, osrs_h:3, enSPI:1;
	uint8_t  stepDirect, stepFind, stepBegin;	uint8_t  cntFind; 
	uint16_t periodFind;	
	uint8_t  cntTemper,    cntHum,    cntPress,    cntErr;  
	uint8_t  periodTemper, periodHum, periodPress, periodErr;
	float    pressureSeaLevel;
	int32_t  adc_T; uint32_t adc_P, adc_H;
	float 	 T,H,P,A;
	int32_t  buff32;
	int32_t  t_fine;
	};							
struct BME280_Coefficients {uint16_t dig_T1; int16_t dig_T2, dig_T3;
							uint16_t dig_P1; int16_t dig_P2, dig_P3, dig_P4, dig_P5, dig_P6, dig_P7, dig_P8, dig_P9;
							uint8_t  dig_H1; int16_t dig_H2; uint8_t dig_H3; int16_t dig_H4; int16_t dig_H5; int8_t dig_H6; };
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//                     КЛАСС УПРАВЛЕНИЯ LCD
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
class RT_HW_BME280{
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//         СТРУКТУРЫ
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++				 
//---------------------------------------------------------------------------------------
public: 
enum {CHIP_ID=0xD0, CTRL_HUM=0xF2, CTRL_MEAS=0xF4, CONFIG=0xF5, PRESS_MSB=0xF7, PRESS_LSB=0xF8, PRESS_XLSB=0xF9, TEMP_MSB=0xFA, TEMP_LSB=0xFB, TEMP_XLSB=0xFC, HUM_MSB=0xFD, HUM_LSB=0xFE}; //--Номера регистров сенсора;
enum {DIG_T1_LSB=0x88,  DIG_P1_LSB=0x8E, DIG_P5_LSB=0x96,  DIG_H1_LSB=0xA1, DIG_H2_LSB=0xE1};

RT_HW_BME280_DEVICE 		device;			//--
RT_HW_BME280_Coefficients	coeff;
RT_HW_BME280();										//--Конструктор;
void	genEventID	(RT_HW_BME280_ID &id);
void	setID       (RT_HW_BME280_ID &id, char mode, int8_t adr);
void 	directDevice(RT_HW_BME280_ID &id, bool EN=1);
void    i2c_find    (RT_HW_BME280_ID &id, uint8_t cntMax);
void 	begin       (RT_HW_BME280_ID &id);
//----------------
uint8_t calcConfig  (RT_HW_BME280_ID &id){uint8_t v=0; v=id.enSPI  & 0x1; v |=(id.filter << 2) & 0b00011100; v |=(id.t_sb   << 5)& 0b11100000; return v;}; //--Формирование регистра config;
uint8_t calcCtrlMeas(RT_HW_BME280_ID &id){uint8_t v=0; v=id.mode   & 0x3; v |=(id.osrs_p << 2) & 0b00011100; v |=(id.osrs_t << 5)& 0b11100000; return v;}; //--Формирование регистра ctrl_meas;
uint8_t calcCtrlHum (RT_HW_BME280_ID &id){uint8_t v=0; v=id.osrs_h & 0x3; 																	   return v;}; //--Формирование регистра ctrl_hum;
void    initVal		(RT_HW_BME280_ID &id){id.T=20.5; id.H=40.5; id.P=1000.5; id.A=35.5;};
void 	readTemp	(RT_HW_BME280_ID &id);
void 	readHum		(RT_HW_BME280_ID &id);
void 	readPress	(RT_HW_BME280_ID &id);
void 	handlerTemp	(RT_HW_BME280_ID &id);
void    handlerHum 	(RT_HW_BME280_ID &id);
void 	handlerPress(RT_HW_BME280_ID &id);
void    handlerAero	(RT_HW_BME280_ID &id);
uint8_t readByte    (RT_HW_BME280_ID &id, uint8_t reg);
void    readBuff    (RT_HW_BME280_ID &id, uint8_t* buff, uint8_t reg, uint8_t num);
uint8_t spi_Transfer(RT_HW_BME280_ID &id, uint8_t data);
void 	writeByte   (RT_HW_BME280_ID &id, uint8_t reg, uint8_t value);
};//==============  Конец класса ==================================================================
extern RT_HW_BME280 RT_HW_bme280; //--Внешняя ссылка на объект;
#endif
