#include "RT_HW_MAX6675.h"
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	1.1. Конструктор
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
RT_HW_MAX6675::RT_HW_MAX6675() {}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
//	1.2 Генератор меандра 50 мс и флага перезапуска устройства (periodReboot в тиках=50мс)
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
void RT_HW_MAX6675:: genEventID(RT_HW_MAX6675_ID &id){id.event=0; if(RT_HW_MAX6675_MILLIS>=id.timeStart) {id.event=1; id.blink=!id.blink; id.timeStart=RT_HW_MAX6675_MILLIS+RT_HW_MAX6675_PERIOD_TICK;}};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	1.3. Настройка параметров устройства
// t_sb(период опроса)  =0: 0.5ms; =1: 62.5ms; =2: 125ms; =3: 250ms; =4: 500ms; =5: 1000ms; =6: 10ms; =7: 20ms;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_MAX6675:: setID(RT_HW_MAX6675_ID &id, char interface,   uint8_t pinCS, uint8_t pinSCK, uint8_t pinSO){
if(id.custom==1) {return;} id.custom=1; id.cs=(int8_t)pinCS; id.sck=(int8_t)pinSCK; id.so=(int8_t)pinSO;
if(interface=='H') {id.modeInterface=1;} else {id.modeInterface=0;}
//----Начальная установка параметров
id.permit=0; id.codeErr=7; id.T=-0.1;	id.err=1; id.fresh=1;	
//----Определение периодов опроса и допустимой ошибки--------------------------
id.cntTemper=id.cntErr=0; 		//--Очистка счетчиков;	
//----Настройка интерфейса SPI-------------------------------------------------
if(id.modeInterface) {
	if(id.cs<0) {id.codeErr=5; return;}
	digitalWrite(id.cs,1); pinMode(id.cs,OUTPUT); if(!device.spi_ok) {device.spi_ok=1; SPI.begin();}}
//----Настройка интерфейса - программный SPI через 3 пина----------------------
else {
	if((id.cs<0) || (id.sck<0) || (id.so<0)) {id.codeErr=5; return;}
	digitalWrite(id.cs,1); pinMode(id.cs,OUTPUT); digitalWrite(id.sck,0); pinMode(id.sck,OUTPUT); pinMode(id.so,INPUT_PULLUP);}	
//----Завершение  настройки----------------------------------------------------
id.eventTemper=1;				//--Установка флага разрешения запуска;	
id.permit=1; 					//--Установка разрешения работы с устройством;				  
};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	1.4. Управление устройством
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_MAX6675:: directDevice(RT_HW_MAX6675_ID &id, uint8_t period){
genEventID(id); 
//----Обработка каждый тик (50 мс)-------------------------------------------------------------------------
if(id.event){ if(period < 5) {period =5;} 																								
			  if(++id.cntTemper > period) {id.cntTemper=1; id.eventTemper =1;}	//--Установка события запуска измерения (id.eventTemper);
			}
//----Ожидание флага запуска измерения id.eventTemper----------------------------------------------
if(!id.eventTemper) {return;}			//--Выход, если нет события запуска измерения;
id.eventTemper=0;						//--Сброс события запуска измерения;
if(!id.permit) 		{return;} 			//--Выход, если устройство не готово;
//----Обработка ошибок;
if(id.codeErr) {id.cntErr++;} 			//--Если предыдущее измерение с ошибкой, то увеличение счетчика, ине
if(id.cntErr >= RT_HW_MAX6675_MAX_ERR)   {id.cntErr=0; id.err=1; id.fresh=1;}
//----Чтение с устройства------------------------------------------------------
id.rawData=0;
//-------------------Аппаратный SPI--------------------------------------------
if(id.modeInterface) {
	SPI.beginTransaction(SPISettings(4000000, MSBFIRST, SPI_MODE0)); //up to 4MHz, read MSB first, SPI mode 0, see note
	digitalWrite(id.cs,0);  id.rawData = SPI.transfer16(0x0000);  digitalWrite(id.cs,1); SPI.endTransaction(); 
} 
//-------------------Программный SPI-------------------------------------------					  
else {
	digitalWrite(id.cs,0);                        
	if(RT_HW_MAX6675_ENABLE_BLOCK_INTERRUPT) {noInterrupts();}         //--Запрет прерываний
	for (int8_t i=16; i>0; i--){digitalWrite(id.sck,1); id.rawData = (id.rawData<< 1)| digitalRead(id.so); digitalWrite(id.sck,0);}
	if(RT_HW_MAX6675_ENABLE_BLOCK_INTERRUPT) {interrupts();}                                
	digitalWrite(id.cs,1);
}	
//----Обработка результатов измерение------------------------------------------
if(id.rawData==0xffff)		{id.codeErr=1; return; }	//--Датчик не подключен;
if(id.rawData==0) 			{id.codeErr=2; return; }	//--Ошибочное измерение;
if(bitRead(id.rawData,2))	{id.codeErr=3; return;}		//--Обрыв термопары;														 
if(!id.err) {id.T= (float)(id.rawData>>3) * RT_HW_MAX6675_RESOLUTION;}//--Получение температуры c защитой от случайного значения, если была ошибка;  
id.codeErr=id.cntErr=0; id.err=0; id.fresh=1;			//--Сброс ошибок, установка флага новых данных; 	
};				 			 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	1.5. Запуск преобразования
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_MAX6675::readRawData(RT_HW_MAX6675_ID &id){id.rawData=0;
if(id.modeInterface) {SPI.beginTransaction(SPISettings(4000000, MSBFIRST, SPI_MODE0)); //up to 4MHz, read MSB first, SPI mode 0, see note
					  digitalWrite(id.cs,0);  id.rawData = SPI.transfer16(0x0000);  digitalWrite(id.cs,1); SPI.endTransaction(); } 
                else {digitalWrite(id.cs,0);                        
					  if(RT_HW_MAX6675_ENABLE_BLOCK_INTERRUPT) {noInterrupts();}         //--Запрет прерываний
					  for (int8_t i=16; i>0; i--){digitalWrite(id.sck,1); id.rawData = (id.rawData<< 1)| digitalRead(id.so); digitalWrite(id.sck,0);}
					  if(RT_HW_MAX6675_ENABLE_BLOCK_INTERRUPT) {interrupts();}                                
					   digitalWrite(id.cs,1);}	}
//=================================================================================================	
RT_HW_MAX6675 RT_HW_max6675; //--Создание объекта 

