/*** FLProg ************** Библиотека RT_HW_MAX6675 ver0.1 для Arduino ***************
Измерение температуры  К-термопары с компенсацией холодного спая с разрешением 12 разрядов АЦП.
Интерфейс SPI 4 мГц, максимальная 4Гц.
    - Термопары типа K имеют абсолютную точность около ± 2 ° C.
    - Диапазон измерения температуры 0 ° C ... + 1024 ° C с разрешением 0,25 ° C
    - Диапазон компенсации холодного спая -20 ° C ... + 85 ° C
    - Следует поддерживать одинаковую температуру холодного спая термопары типа K и MAX6675
    - Следует избегать. размещения тепловыделяющих устройств или компонентов рядом с преобразователем.
    - Рекомендуется добавить керамический конденсатор для поверхностного монтажа 10 нФ / 0,01 мФ, расположенный поперек
      контакты T + и T- для фильтрации шума на линиях термопары.
   Пины аппаратного SPI: 
   Board:                                    MOSI        MISO        SCLK         SS, don't use for CS   Level
   Uno, Mini, Pro, ATmega168, ATmega328..... 11          12          13           10                     5v
   Mega, Mega2560, ATmega1280, ATmega2560... 51          50          52           53                     5v
   Due, SAM3X8E............................. ICSP4       ICSP1       ICSP3        x                      3.3v
   Leonardo, ProMicro, ATmega32U4........... 16          14          15           x                      5v
   Blue Pill, STM32F103xxxx boards.......... PA17        PA6         PA5          PA4                    3v
   NodeMCU 1.0, WeMos D1 Mini............... GPIO13/D7   GPIO12/D6   GPIO14/D5    GPIO15/D8*             3v/5v
   ESP32.................................... GPIO23/D23  GPIO19/D19  GPIO18/D18   x                      3v	  					 
  ecoins@mail.ru 08.08.2020г.
**************************************************************************************************/
//#################################################################################################
#ifndef RT_HW_MAX6675_h
#define RT_HW_MAX6675_h
//-------------------------------------------------------------------------------------------------
#if defined(ARDUINO) && ARDUINO >= 100 
#include <Arduino.h>
#else
#include <WProgram.h>
#endif
//-------------------------------------------------------------------------------------------------
#include <SPI.h>
//-------------------------------------------------------------------------------------------------
#define RT_HW_MAX6675_VERSION 					02			//--Номер версии библиотеки
#define RT_HW_MAX6675_PERIOD_TICK  				50			//--Период системного времени в ms;
#define RT_HW_MAX6675_MILLIS					millis()	//--Время в миллисекундах;
#define RT_HW_MAX6675_RESOLUTION      			0.25   		//--Разрешение измерения в градусах С;
#define RT_HW_MAX6675_SIZE_BUFF      			4  			//--Размер массива для усреднения;
#define RT_HW_MAX6675_MAX_ERR      				3  			//--Максимальное допустимое кол-во ошибок (мах=7);
#define RT_HW_MAX6675_ENABLE_BLOCK_INTERRUPT	0			//-- =1 - Разрешена блокировка прерываний при измерения; 
//-------------------------------------------------------------------------------------------------							   
struct RT_HW_MAX6675_DEVICE 	  {uint16_t spi_ok:1;};								
struct RT_HW_MAX6675_ID {
	uint32_t timeStart;
	int8_t  cs, sck, so; uint8_t cntTemper;
	uint8_t codeErr:3, cntErr:3, err:1; 
	uint8_t custom:1, permit:1, modeInterface:1, event:1, blink:1, eventTemper:1, fresh:1;
	uint16_t rawData;
	float T;
	};							
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//                     КЛАСС УПРАВЛЕНИЯ LCD
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
class RT_HW_MAX6675{
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//         СТРУКТУРЫ
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++				 
//---------------------------------------------------------------------------------------
public:
 //void	setID       	(RT_HW_MAX6675_ID &id, char interface='S', uint8_t period=10,  float correct=0.0);
RT_HW_MAX6675_DEVICE 		device;			//--
RT_HW_MAX6675();										//--Конструктор;
void	genEventID		(RT_HW_MAX6675_ID &id);
void    setID			(RT_HW_MAX6675_ID &id, char interface, uint8_t pinCS, uint8_t pinSCK=-1, uint8_t pinSO=-1);
void	directDevice	(RT_HW_MAX6675_ID &id, uint8_t period);
void 	readRawData 	(RT_HW_MAX6675_ID &id);
//void    filterAdcData	(RT_HW_MAX6675_ID &id);
//void 	filterMiddleData(RT_HW_MAX6675_ID &id); 
};//==============  Конец класса ==================================================================
extern RT_HW_MAX6675 RT_HW_max6675; //--Внешняя ссылка на объект;
#endif
