/***************************************************************************************************
    Тест библиотеки сенсора MAX6675 (температура 0-1024град.С)
    Библиотека ориентирована для работы в системах реальногов времени.
    Библиотека адаптирована для работы с системой визуального програмиирования FLProg.
1. Поддерживает подключение через аппаратный SPI и программный SPI (пина).
2. Поддерживает одновременную работу с несколькими сенсорами (в т.ч. и с разными интерфейсами подключения);
3. Точность измерения  0.25 градусов С.
4. Периодичность вызова определяется параметр period(default=20) в тиках.
   Если период опроса  меньше 5, ему присваивается значение 5(соответствует 250мс). Максимальное значение 255. 1 тик=50мс. 
6. При троекратной ошибке измерения, взводится флаг id.err и сохраняется последнее значение температуры.
7. При обновлении данных взводится флаг id.fresh, который сбрасывается пользователем.
8. Длительность цикла опроса для AV328:  аппаратный SPI=60 мкс; программный SPI=350мкс;
9. Коды ошибок (id.codeErr):
    =0 - нет ошибок;
    =1 - устройство не подключено. 
    =2 - ошибка измерения;
    =3 - обрыв термопары;
    =5 - недопустимый номер пина;  
10. Контрольные пины тест соответствуют MEGA2560.
   Для тестирования с другими контроллерами требуется изменить строки 34-37.  
   ==================================================================================================
    - Термопары типа K имеют абсолютную точность около ± 2 ° C.
    - Диапазон измерения температуры 0 ° C ... + 1024 ° C с разрешением 0,25 ° C
    - Диапазон компенсации холодного спая -20 ° C ... + 85 ° C
    - Следует поддерживать одинаковую температуру холодного спая термопары типа K и MAX6675
    - Следует избегать размещения тепловыделяющих устройств или компонентов рядом с преобразователем.
    - Рекомендуется добавить керамический конденсатор для поверхностного монтажа 10 нФ / 0,01 мФ, расположенный поперек
      контакты T + и T- для фильтрации шума на линиях термопары.
****************************************************************************************************/
//***************************************************************************************************
#include "RT_HW_MAX6675.h"
//--------------------------------------------------------------------------------------------------
RT_HW_MAX6675_ID id;            //--Создание объекта с данными под сенсор;
int8_t pinControl=38;          //--Номер контрольного пина;
uint8_t pinCS=22;
uint8_t pinSCK=9;
uint8_t pinSO=10;
uint8_t period=20;;
float max_T;
bool  max_err;
byte  max_codeErr;
uint16_t max_rawData;
//-------------------------------------------------------------------------------------------------
bool blink;
uint32_t timeStart;
//=================================================================================================
void setup() {Serial.begin(115200);
while(!Serial){;};                          //--Ожидание готовности монитора;
pinMode(pinControl,OUTPUT);                 //--Настройка контрольного пина;
Serial.println(F("Start test MAX6675***")); //--Вывод стартового сообщения
}
//=================================================================================================
void loop() {
blink=!blink; digitalWrite(pinControl,blink);  //--Вывод контрольного сигнала; 
if((millis()-timeStart)>2000){timeStart=millis(); 
Serial.print(F("MAX6675-> mode=")); Serial.print(id.modeInterface); 
Serial.print(F(" err=")); Serial.print(max_err);
Serial.print(F(" code=")); Serial.print(max_codeErr);
Serial.print(F(" rawData=")); Serial.print(max_rawData);
Serial.print(F(" T=")); Serial.print(max_T,2);
Serial.print('\n');
} 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
if(!id.custom){RT_HW_max6675.setID(id,'S', pinCS, pinSCK, pinSO);} //--Настройка: 'P' - программный SPI; 10-период опроса 500мс(50мс*10); 5-пин cs; 4-пин sck; 6-пин so; -1.0 -корректировка значения;
if(1) {RT_HW_max6675.directDevice(id, period);}
if(id.fresh){id.fresh=0;  max_T=id.T;  max_err=id.err; max_codeErr=id.codeErr; max_rawData=id.rawData;}                        
//-------------------------------------------------------------------------------------------------
}
//------------------------------------------------------------------------------------------------
