#include "RT_HW_HTU21.h"
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		1.1. Конструктор
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
RT_HW_HTU21::RT_HW_HTU21() {};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		1.2. Установка параметров
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_HTU21:: setID(RT_HW_HTU21_ID &id, uint16_t periodTemp, uint16_t periodHum){
if(id.custom) {return;}  			//--Проверка настройки устройства (настраивается один раз);
id.custom=1;						//--Установка флага"Устройство настроено";
id.permit=0;						//--Очистка флага " Готовность устройства;
id.addr=RT_HW_HTU21_ADDRESS;		//--Установка адреса устройства;
id.ban=0;							//--Флаг запрета работы с устройством;
setVal(id);							//--Установка измерений по умочанию;
id.cntErrLink=id.cntErrMeter=0;		//--Счетчики многократные измерений и инциализациий
id.periodTmp=periodTemp; if((id.periodTmp!=0) && (id.periodTmp<1000)) {id.periodTmp=1000;}	//--Установка периода(тики) опроса температуры;
id.periodHum=periodTemp; if((id.periodHum!=0) && (id.periodHum<1500)) {id.periodHum=1500;}	//--Установка периода(тики) опроса влажности;
id.timeTmp=id.timeHum=RT_HW_HTU21_MILLIS;
};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		1.3. Установка измерений по умолчанию
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_HTU21:: setVal(RT_HW_HTU21_ID &id){id.valTmp=20.1; id.valHum=90.2;};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		Управление устройством
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_HTU21:: direct(RT_HW_HTU21_ID &id, bool EN){
//----Установка флагов запуска измерения-----------------------------------------------------------
//id.eventTmp=0; id.eventHum=0;
if(RT_HW_HTU21_MILLIS>=id.timeTmp){id.eventTmp=1; id.timeTmp+=id.periodTmp;}
if(RT_HW_HTU21_MILLIS>=id.timeHum){id.eventHum=1; id.timeHum+=id.periodHum;}
//----Контроль повторяющихся ошибок и установка измерений по умолчанию при повторяющихся ошибок при перезапуске; 
if(id.errLink) {id.errLink =0; if(++id.cntErrLink >3) {id.cntErrLink =0; setVal(id); id.dir=0; id.err=1;}} //--Обработка ошибки связи;
if(id.errMeter){id.errMeter=0; if(++id.cntErrMeter>3) {id.cntErrMeter=0; setVal(id); id.dir=0; id.err=1;}} //--Обработка ошибки измерения;
//-------------------------------------------------------------------------------------------------
//----Вызов инициализации устройства (проверка связи, установка параметров)
if(id.dir==0)	{if(!EN){return;} id.step=0;  		 id.dir++;}
if(id.dir==1)	{begin(id);       if(id.okBegin){id.dir++;} return;}
//----Ожидание флагов запросов на измерение--------------------------------------------------------
if(id.dir==2)	{if((id.eventTmp) || (id.eventHum)) {if(EN){id.dir++;}} return;};
//----Измерение  температуры-----------------------------------------------------------------------
if(id.dir==3) 	{if(id.eventTmp) {id.eventTmp=0; id.step=0; id.dir++;} else {id.dir=5;}}
if(id.dir==4)	{readTemp(id);    if(id.okTmp){id.dir++;} }
//----Измерение  влажности-------------------------------------------------------------------------
if(id.dir==5) 	{if(id.eventHum) {id.eventHum=0; id.step=0; id.dir++;} else {id.dir=7;}}
if(id.dir==6)	{readHum(id);     if(id.okHum){id.dir++;} } 
//----Успешное завершение--------------------------------------------------------------------------	
if(id.dir==7)	{id.err=0;
				 if(id.errLink) {id.err=1;} else {id.cntErrLink=0;} 
				 if(id.errMeter){id.err=1;} else {id.cntErrMeter=0;} 
				 id.dir=2;}	
};	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		Инициализация устройства
//  Вход: id.step=0
//	1.Если шина i2c не настроена, выполняется настройка командой Wire.begin();
//  2.Осуществляется поиск устройства на шине i2c. 
//  3.Осуществляется инициализация устройства (для HTU21 не выполняются действия).
//  Выход: Если все успешно, id.okBegin=1.
//		   Если устройство не обнаружено, то поиск осуществляется с последовательно увеличивющим 
//		   периодом от 100мс до 100мс х RT_HW_HTU21_FIND_MAX=2000мс.  	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_HTU21:: begin(RT_HW_HTU21_ID &id){
//----Однократная инициализация шины i2c (если hive.i2c_state)-------------------------------------
if(id.step==0)	{id.okBegin=0;  if(id.ban) {return;} 
				 if(!hive.i2c_state) {hive.i2c_state=1; Wire.begin(); } 	//--Иницализация шины i2c;
				 id.findErr=id.findOk=0; id.step++;}						//--Сброс флагов для функции i2c_find;				 
//----Поиск уcтройства на шине i2с------------------------------------------------------------------
if(id.step==1)	{if(i2c_find(id.timeDelay,id.findErr,id.findOk,id.addr,RT_HW_HTU21_FIND_MAX)){id.findErr=0; id.findOk=0; id.step++;} return;} 
				// if(id.findOk >0) 					  {id.code=1; return;}	//--Выход для продолжения успешного поиска;
				// if(id.findErr>=RT_HW_HTU21_FIND_MAX) {id.code=2; return;}  //--Выход для продолжения поиска по ошибке;
				//								       id.code=3; return;}
//----Программный сброс сенсора----------------------------------------------------------------------
if(id.step==2)	{if(writeCmnd(id.addr, RT_HW_HTU21_SOFT_RESET)) {id.errLink=1; return;} id.step++;}	
//----Завершение инциализации:				 
if(id.step==3)	{id.eventTmp=id.eventHum=1; id.timeTmp=id.timeHum=RT_HW_HTU21_MILLIS; id.okBegin=1; id.step=0; return;}
//-----Обработка несуществующего номера шага;
id.step=0;};	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		Чтение температуры
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_HTU21:: readTemp(RT_HW_HTU21_ID &id){
//----Запрос на чтение регистра температуры--------------------------------------------------------
if(id.step==0)	{if(writeCmnd(id.addr,RT_HW_HTU21_TRIG_TMP_NOHOLD)){id.errLink=1; id.okTmp=1; return;} 		 
				 id.okTmp=0; id.timeDelay=RT_HW_HTU21_MILLIS+60; id.step++;}
//----Пауза для преобразования в сенсоре------------------------------------------------------------				 
if(id.step==1)	{if(RT_HW_HTU21_MILLIS < id.timeDelay) {return;} id.step++;}
//----Чтение данных-----------------------------------------------------------				 
if(id.step==2)	{if(readData(id.timeDelay,id.step2,id.addr,3))       {id.errMeter=1; id.okTmp=1; return;}						
				 id.msb=Wire.read(); id.lsb=Wire.read(); id.crc=Wire.read();          id.step++; return;} 	
//----Обработка измерений--------------------------------------------------------------------------
if(id.step==3) {uint16_t rawTmp = ((uint16_t)id.msb << 8) | (uint16_t)id.lsb;
				if(check_crc(rawTmp,id.crc)!=0){id.errMeter=1; id.okTmp=1; return;}  //--Завершение по ошибке CRC;	
				rawTmp &=0xFFFC; float valTmp=rawTmp/(float)65536; id.valTmp= -46.85 + (175.72 * valTmp); 	
				id.step=0; id.okTmp=1; return;}};				
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		Чтение влажности
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_HTU21:: readHum(RT_HW_HTU21_ID &id){
//----Запрос на чтение регистра температуры--------------------------------------------------------
if(id.step==0)	{if(writeCmnd(id.addr,RT_HW_HTU21_TRIG_HUM_NOHOLD)){id.errLink=1; id.okHum=1; return;} 		 
				 id.okHum=0; id.timeDelay=RT_HW_HTU21_MILLIS+60; id.step++;}
//----Пауза для преобразования в сенсоре-----------------------------------------------------------				 
if(id.step==1)	{if(RT_HW_HTU21_MILLIS < id.timeDelay){return;} id.step++;}
//----Чтение данных-----------------------------------------------------------				 
if(id.step==2)	{if(readData(id.timeDelay,id.step2,id.addr,3))    {id.errMeter=1; id.okHum=1; return;}						
				 id.msb=Wire.read(); id.lsb=Wire.read(); id.crc=Wire.read(); id.step++; return;} 
//----Обработка измерений--------------------------------------------------------------------------
if(id.step==3) {uint16_t rawHum = ((uint16_t)id.msb << 8) | (uint16_t)id.lsb;
				if(check_crc(rawHum,id.crc)!=0){id.errMeter=1; id.okHum=1; return;}  //--Завершение по ошибке CRC;	
			    rawHum &= 0xFFFC;  float valHum=rawHum/(float)65536; id.valHum= -6 + (125 * valHum); 	
				id.step=0; id.okHum=1; return;}};			
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		Поиск устройства на шине i2c 
//  Вход: cnt=0;
//  Если устройство с заданныим адресом не найдено, то период поиска изменяется от 100мс до (100 х cntMax).
//	Если устройство найдено, то делаются дополнительные запросы через 50мс (всего должно быть три успешных попытки подряд);
//  Выход: успешный выход 		 ->cnt=0, функция возвращает 1;
//		   устройство не найдено ->cnt>1, функция возвращает 0;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool RT_HW_HTU21::i2c_find(uint32_t &timeDelay, uint8_t &findErr, uint8_t &findOk, int8_t addr, uint8_t cntMax){
if((findErr==0) && (findOk==0)) {timeDelay=RT_HW_HTU21_MILLIS;} //--Сброс и установка параметров;
if(RT_HW_HTU21_MILLIS < timeDelay) {return 0;}  				//--Ожидание по времени;
Wire.beginTransmission(addr);									//--Отправка запроса на шину i2c;
//----Обработка успешной транзакции-------------------------------
if(Wire.endTransmission()==0){if(findOk>=3)      {return 1;}	//--Успешный выход после трехкратного успешного поиска;	 
  else{findOk++; timeDelay=RT_HW_HTU21_MILLIS+50; return 0;}}	//--Повторный поиск (для проверки устойчивости связи);	
//----Обработка ошибочной транзакции-----------------------------
findOk=0; if(cntMax==0){cntMax=20;} if(findErr<cntMax){findErr++;} 
timeDelay=(RT_HW_HTU21_MILLIS+(100*findErr)); return 0;}  			
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		Отправка команды
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
uint8_t RT_HW_HTU21:: writeCmnd(int8_t addr, uint8_t reg){Wire.beginTransmission(addr); Wire.write(reg); return Wire.endTransmission();} 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		Получение данных после отправки команды на чтение параметра;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
uint8_t RT_HW_HTU21:: readData(uint32_t &timeDelay, uint8_t &step, int8_t addr, int8_t qnt){
step=0; timeDelay=RT_HW_HTU21_MILLIS+RT_HW_HTU21_TIME_OUT_READ;	step=0;	//--Запуск тайм-аута;	
do{if(RT_HW_HTU21_MILLIS>timeDelay) {return 1;} 						//--Выход по тайм-ауту;
   if(step==0){if(Wire.available()>0) {Wire.read();} else {step=1;}}	//--Очистка приемного буфера;
   if(step==1){Wire.requestFrom(addr,qnt); step=2;}						//--Запрос на чтение регистров; 
   if(Wire.available()==3) {return 0;}} while(1);						//--Нормальный выход;
};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		Проверка контрольной суммы
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
uint8_t RT_HW_HTU21::check_crc(uint16_t message_from_sensor, uint8_t check_value_from_sensor){
uint32_t divisor   = (uint32_t)RT_HW_HTU21_SHIFTED_DIVISOR;	//--Запоминаем 3-х байтовый полином;
uint32_t remainder = (uint32_t)message_from_sensor << 8;	//--Сдвиг измеренного значения на один байт влево (для работы с 3-байтовым полиномом); 
         remainder|= check_value_from_sensor; 				//--Добавление в младший байт crc из сенсора;
for (uint8_t i=0;i<16;i++){if(remainder & (uint32_t)1<<(23-i)) remainder^= divisor; divisor>>=1;} //--Расчет остатка после проверки crc;
return (uint8_t)remainder;}	
//=================================================================================================	
RT_HW_HTU21 RT_HW_htu21; //--Создание объекта 

