/*** FLProg ************** Библиотека RT_HW_HTU ver0.1 для Arduino ***************


При длине проводов более 10см, рекомендуется проложить VDD и/или GND между SCK и SDA,
или использовать экранированные провода.

 
 BME280 запускается в следующей последовательности:
  1. Выбирается протокол связи (I2C или аппаратный SPI) и настраивается по упрощенной схеме, если он еще не настроен.
  2. Проверяется наличие связи с устройством.
  3. Из устройства считываются коэффициенты компенсации.
  4. Устанавливается настройка БИХ-фильтра* (0,2,4,8).
  5. Настройка коэффициентов передискретизации и режима работы датчика.

  
// Чрезмерная выборка сильно влияет на шум в данных (пожалуйста, обратитесь к таблице для получения дополнительной информации)
// Спецификация BME280 предоставляет рекомендации по настройке для различных приложений
// Наконец мы записываем все эти значения в соответствующие регистры
  
  // 5. Проверьте связь (спросите и проверьте идентификатор чипа)
	
  Библиотеки  разрабатываются с учетом особенностей использования в системе визуального программирования FLProg.
  ecoins@mail.ru 29.09.2019г.
**************************************************************************************************/
//#################################################################################################
#ifndef RT_HW_HTU21_h
#define RT_HW_HTU21_h
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//							Глобальные параметры и функции
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#if defined(ARDUINO) && (ARDUINO >= 100) 
  #include <Arduino.h>
#else
  #include <WProgram.h>
#endif
#include <Wire.h>
#define RT_HW_HTU21_MILLIS			millis()	//--Время в миллисекундах;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//							Системные параметры 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#define RT_HW_HTU21_VERSION 			01			//--Номер версии библиотеки
#define RT_HW_HTU21_PERIOD_TICK  		50			//--Период системного времени в ms;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//							Параметры для работы с устройством
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#define RT_HW_HTU21_ADDRESS 			0x40  		//--Адрес сенсора на шине i2c
#define RT_HW_HTU21_MAX_NUM_ERR			3			//--Максимальное кол-во тиков с ошибками;
#define RT_HW_HTU21_FIND_MAX			20			//--Максимальное кол-во циклов при поиске на шине i2c;
#define RT_HW_HTU21_TIME_OUT_READ    	2			//--Максимальное время на считывание 3-x байт;
#define RT_HW_HTU21_TRIG_TMP_NOHOLD	0xF3
#define RT_HW_HTU21_TRIG_HUM_NOHOLD		0xF5
#define RT_HW_HTU21_SOFT_RESET			0xFE		//--Код программного сброса сенсора
#define RT_HW_HTU21_SHIFTED_DIVISOR 	0x988000	//--Полином 0x0131 = x^8 + x^5 + x^4 + 1;													
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//							Структура для поиска устройства на шине i2c
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
struct RT_HW_HTU21_I2C_ID {uint8_t step:2, find:1, cntFind:4; uint8_t cnt;  uint32_t timeStart;};//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//							Структура для работы с устройством HTU21
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++  													
struct RT_HW_HTU21_ID {
 uint32_t timeTmp; uint16_t periodTmp;  float valTmp;
 uint32_t timeHum; uint16_t periodHum;  float valHum; 
 uint32_t timeDelay;
 uint16_t permit:1, ban:1, link:1, err:1, eventTmp:1, eventHum:1, okBegin:1, okFind:1, okTmp:1, okHum:1; 
 uint8_t  cntErrLink, cntErrMeter;
 uint8_t  addr, findErr, findOk, step2;
 uint8_t  custom, dir, step, errLink, errMeter;		
 uint8_t  msb,lsb,crc;			//--Кол-во принятых байт; Старший и младший байты данных; Контрольная сумма; 

	};	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//							Структура для работы класса RT_HW_HTU21 (20 байт)
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++  
struct RT_HW_HTU21_HIVE {
	uint16_t  i2c_state:1;  //--Состояние i2c(=1-настроен);
};	
//=================================================================================================
//                     КЛАСС УПРАВЛЕНИЯ LCD
//=================================================================================================
class RT_HW_HTU21{
public: 
//----Данные---------------------------
RT_HW_HTU21_HIVE	hive;	//--Данные с рабочимми регистрами и параметрами.
//----Функции--------------------------------------------------------------------------------------
RT_HW_HTU21();										//--Конструктор;
void setID    (RT_HW_HTU21_ID &id, uint16_t periodTemp=1000, uint16_t periodHum=1500); //--Установка параметров;
void setVal	  (RT_HW_HTU21_ID &id);					//--Установка измерений по умолчанию;
void direct   (RT_HW_HTU21_ID &id, bool EN);		//--Управление устройством;
void begin    (RT_HW_HTU21_ID &id);					//--Инициализация устройства;		
void readData (RT_HW_HTU21_ID &id, uint8_t qnt); 	//--Получение данных после отправки команды на чтение параметра;
void checkCRC (RT_HW_HTU21_ID &id); 				//--Проверка контрольной суммы;
void readTemp (RT_HW_HTU21_ID &id);					//--Чтение температуры и обработка температуры;
void readHum  (RT_HW_HTU21_ID &id);					//--Чтение влажности;
bool    i2c_find (uint32_t &timeStart, uint8_t &cnt, uint8_t &cntFind, int8_t addr, uint8_t cntMax);//--Поиск уст-ва на шине i2c;
uint8_t writeCmnd(int8_t addr, uint8_t reg);									//--Отправка команды;
uint8_t readData (uint32_t &timeDelay, uint8_t &step, int8_t addr, int8_t qnt);	//--Чтение данных;
uint8_t timeOut  (uint32_t &timeBegin, uint32_t timeEnd, uint16_t timeMax);		//--Расчет тайм-аута; 
uint8_t	check_crc(uint16_t data_sensor, uint8_t crc_sensor); 					//--Проверка контрольной суммы;
};//==============  Конец класса ==================================================================
extern RT_HW_HTU21 RT_HW_htu21; //--Внешняя ссылка на объект;
#endif
