//***************************************************************************************************
//  Тест библиотеки сенсора BME280 (температура, влажность, давление)
//  (для работы в системах реальногов времени).
//  Библиотека адаптирована для работы с системой визуального програмиирования FLProg.
//  1. Поддерживает подключение через i2c(проверено), SPI(в работе), через 4-пина - программный SPI (в работе).
//  2. Поддерживает одновременную работу с несколькими сенсорами (в т.ч. и с разными интерфейсами подключения);
//  3. Точность измерения  20 разрядов АЦП для температуры и давления и 16 разрядов АЦП для влажности.
//  4. Измерение производится через программные прерывания - длительность цикла не более 300 мкс при скорости шины i2c 400кГц.
//  5. Измерения каждого параметра (T,H,P) занимает 2 программных вхождения. 
//     В терминах FLProg/EIS рекомендуется использовать вызовы как "свободные" задачи.
//  6. Периодичность вызова определяется параметрами id.periodTemper(default=20 - 1сек), id.periodHum(default=40 -2 сек), id.period.Press(default=60 -3 сек)-
//       значения в тиках. Максимальное значение 255. 1 тик=50мс. Если значение параметра=0, соответствующий параметр не опрашивается.
//  7. При обрыве связи на время более 5сек (параметр id.periodErr=100) взводится флаг id.err и устанавливаются значения по умолчанию
//      id.T=20.5 (градусы С); id.H=40.5(%); id.P=1000.5(кПа); id.A=35.5(м);  
//  8. При восстановление связи повторно считываются из сенсора и записываются в сенсор коректирующие коэффициенты и настроечные параметры
//     и производятся измерения.
//  9. При обновлении данных взводится флаг id.fresh, который сбрасывается пользователем.
//  10. Подробное использование параметров приведено  в datasheet BME280 BOSH.      
//      id.mode =0,1,2,3 - (default 0x3 -нормальный режим (циклическое измерение)
//      id.t_stb =0-7    - (default 0x3 - период между измерениями температуры=250ms)
//      id.filter=0-4    - (default 0x3 - коэффициент фильтрации =8);
//      id.osrs_t=(0-7)  - (default 0x3 - коэффициент передискредитации (усреднения) данных температуры. При включенном фильтре разрешение АЦП=20 бит.);
//      id.osrs_p=(0-7)  - (default 0x3 - коэффициент передискредитации (усреднения) данных давления.    При включенном фильтре разрешение АЦП=20 бит.);
//      id.osrs_h=(0-7)  - (default 0x3 - коэффициент передискредитации (усреднения) данных влажности.                          Разрешение АЦП=16 бит.);  
//  9. Тест адаптирован для AVR328, MEGA2560,DUE.
//     Для тестирования с другими контроллерами требуется закомментировать или изменить строки 22,27,33,44.
//***************************************************************************************************
#include "RT_HW_BME280.h"
RT_HW_BME280_ID id;         //--Создание объекта с данными под сенсор.
uint8_t pinControl=10;      //--Номер контрольного пина - 27AVR328, MEGA2560,DUE;
bool blink;
//=================================================================================================
void setup() {Serial.begin(115200);
while(!Serial){;};                         //--Ожидание готовности монитора;
pinMode(pinControl,OUTPUT);               //--AVR328, MEGA2560,DUE;
Serial.println(F("Start ***"));
}
//=================================================================================================
void loop() {
blink=!blink; digitalWrite(pinControl,blink);  //--AVR328, MEGA2560,DUE;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
if(!id.custom) {RT_HW_bme280.setID(id,'I',0x76);
            id.periodTemper=20; id.periodHum=40; id.periodPress=80;
            Wire.setClock((400*(uint32_t)1000));  //--AVR328, MEGA2560,DUE;
            Serial.println(F("-------------------------------------------------------------"));
            Serial.print(F("chip_ID=0x")); Serial.print(id.vChipID,HEX);  Serial.print('\n'); 
            Serial.print(F("x.coeff T")); Serial.print(F(" T1=")); Serial.print(id.coeff.dig_T1); Serial.print(F(" T2=")); Serial.print(id.coeff.dig_T2); Serial.print(F(" T3=")); Serial.print(id.coeff.dig_T3); Serial.print('\n');
            Serial.print(F("x.coeff P")); Serial.print(F(" P1=")); Serial.print(id.coeff.dig_P1); Serial.print(F(" P2=")); Serial.print(id.coeff.dig_P2); Serial.print(F(" P3=")); Serial.print(id.coeff.dig_P3);
                                          Serial.print(F(" P4=")); Serial.print(id.coeff.dig_P4); Serial.print(F(" P5=")); Serial.print(id.coeff.dig_P5); Serial.print(F(" P6=")); Serial.print(id.coeff.dig_P6);
                                          Serial.print(F(" P7=")); Serial.print(id.coeff.dig_P7); Serial.print(F(" P8=")); Serial.print(id.coeff.dig_P8); Serial.print(F(" P9=")); Serial.print(id.coeff.dig_P9); Serial.print('\n');
            Serial.print(F("x.coeff H")); Serial.print(F(" H1=")); Serial.print(id.coeff.dig_H1); Serial.print(F(" H2=")); Serial.print(id.coeff.dig_H2); Serial.print(F(" H3=")); Serial.print(id.coeff.dig_H3);
                                                  Serial.print(F(" H4=")); Serial.print(id.coeff.dig_H4); Serial.print(F(" H5=")); Serial.print(id.coeff.dig_H5); Serial.print(F(" H6=")); Serial.print(id.coeff.dig_H6); Serial.print('\n'); 
            Serial.print(F("t_sb="));    Serial.print(id.t_sb);           
            Serial.print(F(" filter=")); Serial.print(id.filter);
            Serial.print(F(" osrs_t=")); Serial.print(id.osrs_t);                             
            Serial.print(F(" osrs_p=")); Serial.print(id.osrs_p);
            Serial.print(F(" osrs_h=")); Serial.print(id.osrs_h);  
            Serial.print('\n');
            Serial.println(F("-------------------------------------------------------------"));             
}
//------------------------------------------------------------------------------------------------
RT_HW_bme280.directDevice(id,1);
if(id.fresh){id.fresh=0;  Serial.print(F(" T=")); Serial.print(id.T,2); 
                          Serial.print(F(" H=")); Serial.print(id.H,2);
                          Serial.print(F(" P=")); Serial.print(id.P,2);
                          Serial.print(F(" A=")); Serial.print(id.A,2); 
                          if(id.err) {Serial.print(F(" Error"));} else {Serial.print(F(" Ok"));}  
                          Serial.print('\n'); }
}
//------------------------------------------------------------------------------------------------
