#ifndef OneWire_h
#define OneWire_h

#include <stdint.h>

#if defined(__AVR__)
#include <util/crc16.h>
#endif

#if ARDUINO >= 100
#include <Arduino.h>       // for delayMicroseconds, digitalPinToBitMask, etc
#else
#include "WProgram.h"      // for delayMicroseconds
#include "pins_arduino.h"  // for digitalPinToBitMask, etc
#endif
//=====================================================================================================
#ifndef ONEWIRE_CRC
#define ONEWIRE_CRC 1			//--Разрешение проверки CRC(=1);
#endif
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Макросы для работы с GPIO для различных контроллеров
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#include "util/OneWire_direct_regtype.h"
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		КЛАСС 1-WIRE
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
class OneWire
{
private:
//---Скрытые данные
IO_REG_TYPE bitmask;
volatile IO_REG_TYPE *baseReg;
//---Данные для функции search
    unsigned char ROM_NO[8];
    uint8_t LastDiscrepancy;
    uint8_t LastFamilyDiscrepancy;
    bool LastDeviceFlag;
//----------------------------------------
public:
int8_t pin;
OneWire() { }
OneWire(uint8_t pin) { begin(pin); }
void begin(uint8_t pin);				//--Инициализация шины;
uint8_t reset(void); 					//--Возвращает 1, если устройство ответило (>250mcs)
void select(const uint8_t rom[8]);		//--Команда выбора 1-Wire ROM
void skip(void);						//--Пропуск команды поиска
void write(uint8_t v, uint8_t power =0);//--Запись байта. Если power =1, шина удерживается в высоком состоянии
void write_bytes(const uint8_t *buf, uint16_t count, bool power = 0); //--Запись нескольких байт;
uint8_t read(void);						//--Чтение байта;
void read_bytes(uint8_t *buf, uint16_t count); //--Чтение нескольких байт в массив;
void write_bit(uint8_t v);				//--
uint8_t read_bit(void);					//--
void depower(void);						//--Прекратить подачу питания на шину;

void reset_search(); 					//--Очистка состояние поиска
void target_search(uint8_t family_code);//--Поиск по коду семейства
bool search(uint8_t *newAddr, bool search_mode = true); //--Поиск следующего устройства

static uint8_t crc8  (const uint8_t *addr, uint8_t len); //--Вычисление 8-битной CRC Dallas

static bool check_crc16(const uint8_t* input, uint16_t len, const uint8_t* inverted_crc, uint16_t crc = 0);
static uint16_t crc16(const uint8_t* input, uint16_t len, uint16_t crc = 0);
};
//--Предотвращение утечки имени в Arduino
#ifdef IO_REG_TYPE
#undef IO_REG_TYPE
#endif
#endif // OneWire_h
