#include "RT_HW_CAR.h"
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
//		КОНСТРУКТОР
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
RT_HW_CAR:: RT_HW_CAR(){}; 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
//		Инициализация устройства;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_CAR:: setID (RT_HW_CAR_DRIVE_ID &id, uint16_t period, uint8_t factor, uint16_t powerMax, uint16_t powerMin){
if(id.dir==0){id.dir=1;
id.delta.stop=500; 		//--Шаг выхода на остановку моторов;
id.delta.drive=250;		//--Шаг выхода на включение моторов; 
id.delta.small=200;		//--
id.power.cur=id.power.add=0; id.cmnd.cur='S';
if(factor  <20) {id.power.factor =75;} 	else {id.power.factor =factor;}; //--Норм."factor" - коэф.сниж.мощости при поворотах).
if(period  <10) {id.period=10;} 	   	else {id.period=period;};
if(powerMax<100){id.power.Max=4000;}   	else {id.power.Max=powerMax;}; if(id.power.Max>0xfff){id.power.Max=0xfff;}
if(powerMin<100){id.power.Min=2000;}	else {id.power.Min=powerMin;}; if(id.power.Min>0xfff){id.power.Min=0xfff;}
id.power.space=id.power.Max-id.power.Min;	//--Диапазон управления мощностью моторов;
id.timeStart=RT_HW_MILLIS;	
id.step=0;	
}};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  Управление моторами с плавным управлением мощности-----
//	Вход: 	1)	команда "Движение" - cmnd (S,F,B,R,L,G,I,H,J); 2) скорость (%) - speed;
//			3)  флаг "аварийный стоп" block;	4)флаг "режим форсажа" - fast;
//			2)	установочная мощность speed (0-100)
//	Выход:	1)	текущая  команда  cmnd.cur (S,F,B,R,L,G,I,H,J)
//			2)  текущая  мощность моторов power.cur (=0 или в диапазоне power.delta); 
//			3)  текущая  мощность левого и правого моторов - EnA,EnB;
//			4)  вектор   управления направления работы моторов (bit3-bit0->IN4,IN2,IN3,IN4)
// ----------------Алгоритм управления: 
//	Если команда изменилась, то:
//		1) в начале скорость ускоренно уменьшается;
//		2) устанавливается команда СТОП;
//		3) устанавливается новая команда;
//		4) плавно выводится на новую скорость;
//		5) по достижению скорости переход в режим стабилизации скорости
//   По достижению заданной выходной мощности поддерживается пропорционально входной скорости
//====================================================
//	Вывод на режим с определением cmndCur, powerCur, PowerDown
//====================================================
void RT_HW_CAR:: direct(RT_HW_CAR_DRIVE_ID &id, char cmnd, uint8_t speed,  bool block, bool fast){
if(RT_HW_Base.getPastMs(id.timeStart) < id.period) {return;}
id.timeStart=RT_HW_MILLIS;
if(block){id.cmnd.set='S';} else {id.cmnd.set=cmnd;} 							//--Уставка команды;
if(id.cmnd.set=='S'){id.power.set=0;} 											//--Если "СТОП", сброс уставки мощности;
 else				{id.power.set=map(speed,0,100,id.power.Min,id.power.Max);};	//--Расчет уставки мощности;
//-------------------------------------------------------------------------------------------------
convert(id, fast);
id.power.left=id.power.right=id.power.cur;
id.power.add=(id.power.cur * (uint32_t) id.power.factor )/100;  
if(id.power.add < id.power.Min) {id.power.add=id.power.Min;}
//-----------------------------------------------------------------------------------------
if(id.cmnd.cur=='S'){id.power.motor=B0000;}	 							//--СТОП;
if(id.cmnd.cur=='F'){id.power.motor=B1010;}								//--Движение вперед;
if(id.cmnd.cur=='L'){id.power.motor=B1001;}								//--Поворот налево;
if(id.cmnd.cur=='B'){id.power.motor=B0101;}								//--Поворот налево;   
if(id.cmnd.cur=='R'){id.power.motor=B0110;}								//--Поворот налево; 
if(id.cmnd.cur=='I'){id.power.motor=B0110; id.power.left =id.power.add;} //--Вперед с плавным правым поворотом;
if(id.cmnd.cur=='G'){id.power.motor=B0110; id.power.right=id.power.add;} //--Вперед с плавным правым поворотом;
if(id.cmnd.cur=='H'){id.power.motor=B1001; id.power.right=id.power.add;} //--Назад с плавным левым поворотом;
if(id.cmnd.cur=='J'){id.power.motor=B0110; id.power.left =id.power.add;} //--Вперед с плавным левым поворотом;
};
//-------------------------------------------------------------------------------------------------
//	Вычисление текущей мощности.
//	Вход: cmnd.cur, power.cur, power.set; Выход: cmnd.cur, power.cur:
//  Если команда изменилась, то:
//	 1)в начале ускоренно уменьшается мощность; 2)устанавливается команда СТОП;
//	 3)устанавливается новая команда;			4)выполняетcя плавный вывод на новую мощность;
//	 5)по достижению мощности переход в режим стабилизации мощности;
//-------------------------------------------------------------------------------------------------
void RT_HW_CAR:: convert(RT_HW_CAR_DRIVE_ID &id, bool fast){
if(id.step==0) {id.step=1; return;}
//--Ускоренное снижение мощности до нуля  
if(id.step==1) {if(id.power.cur > id.power.Min) {if(id.power.cur > id.delta.stop){id.power.cur=id.power.cur-id.delta.stop;} else{id.power.cur=id.power.Min;}}
				if(id.power.cur <=id.power.Min) {id.power.cur = id.power.Min; id.step=2;} 
				return;}
//--Фиксация команды стоп
if(id.step==2) {id.cmnd.cur='S'; id.power.cur=0; id.step=3; return;}				
//--Ожидание команды движения;
if(id.step==3) {if(id.cmnd.set!='S'){id.cmnd.cur=id.cmnd.set; id.power.cur=id.power.Min; id.step=4;} return;}  
//--Вывода на установочную мощность скорость
if(id.step==4) {if(fast){if(id.cmnd.cur!=id.cmnd.set) {id.power.cur=id.power.Min; id.step=2; return;}
												else  {id.power.cur=id.power.set; id.step=5; return;} return;}
				if(id.cmnd.cur!=id.cmnd.set) {id.step=1; return;} //--Если команда изменилась, то переход на остановку;
				if(id.power.set > id.power.cur) {id.power.cur=id.power.cur + id.delta.drive;}  
				if(id.power.set <=id.power.cur) {id.power.cur=id.power.set; id.step=5;} return;}
//--------Режим стабилизации
if(id.step==5) {if(fast) {if(id.cmnd.cur!=id.cmnd.set) {id.power.cur = id.power.Min; id.step=2;} return;}
				if(id.cmnd.cur!=id.cmnd.set) {id.step=1; return;} return;}            
};	
//-------------------------------------------------------------------------------------------------
//	Преобразование символа от программы Remote Сontrol в вектор управления.
//-------------------------------------------------------------------------------------------------
void RT_HW_CAR:: convertRemoteControl(RT_HW_CAR_REMOTE_CONTROL_ID &id, char sign){
bitWrite(id.direct,15,1); bitWrite(id.direct,14,1); //--Установка флагов "Связь есть" и "Пульт включен"
id.cmnd='S';								//--Предварительная установка rклманды "Стоп";
if(sign=='S') {id.cmnd='S';	bitWrite(id.direct,0,0); bitWrite(id.direct,1,0); bitWrite(id.direct,2,0); bitWrite(id.direct,3,0); return;}	//--Стоп
if(sign=='F') {id.cmnd='F';	bitWrite(id.direct,0,1); bitWrite(id.direct,1,0); bitWrite(id.direct,2,0); bitWrite(id.direct,3,0); return;}	//--Вперед
if(sign=='L') {id.cmnd='L';	bitWrite(id.direct,0,0); bitWrite(id.direct,1,1); bitWrite(id.direct,2,0); bitWrite(id.direct,3,0); return;}	//--Влево
if(sign=='B') {id.cmnd='B';	bitWrite(id.direct,0,0); bitWrite(id.direct,1,0); bitWrite(id.direct,2,1); bitWrite(id.direct,3,0); return;}	//--Назад
if(sign=='R') {id.cmnd='R';	bitWrite(id.direct,0,0); bitWrite(id.direct,1,0); bitWrite(id.direct,2,0); bitWrite(id.direct,3,1); return;}	//--Вправо
if(sign=='G') {id.cmnd='G';	bitWrite(id.direct,1,1); bitWrite(id.direct,1,1); bitWrite(id.direct,2,0); bitWrite(id.direct,3,0); return;}	//--Вперед-влево
if(sign=='I') {id.cmnd='I';	bitWrite(id.direct,1,1); bitWrite(id.direct,1,0); bitWrite(id.direct,2,0); bitWrite(id.direct,3,1); return;}	//--Вперед-вправо
if(sign=='H') {id.cmnd='H';	bitWrite(id.direct,1,0); bitWrite(id.direct,1,1); bitWrite(id.direct,2,1); bitWrite(id.direct,3,0); return;}	//--Назад-влево
if(sign=='J') {id.cmnd='J';	bitWrite(id.direct,1,0); bitWrite(id.direct,1,0); bitWrite(id.direct,2,1); bitWrite(id.direct,3,1); return;}	//--Назад-вправо 
if(sign=='V') {bitWrite(id.direct,4,1); return;}							//--Звук включить
if(sign=='v') {bitWrite(id.direct,4,0); return;}							//--Звук выключить
if(sign=='W') {bitWrite(id.direct,5,1);	bitWrite(id.direct,13,1); return;}	//--Включить  передние фары и форсаж.
if(sign=='w') {bitWrite(id.direct,5,0);	bitWrite(id.direct,13,0); return;}	//--Выключить передние фары и форсаж.
if(sign=='U') {bitWrite(id.direct,6,1);	return;}							//--Задние фары включить
if(sign=='u') {bitWrite(id.direct,6,0); return;}							//--Задние фары выключить
if(sign=='X') {bitWrite(id.direct,7,1);	bitWrite(id.direct,12,1); return;}	//--Включить  Аварийку и блокировку сонара.
if(sign=='x') {bitWrite(id.direct,7,0);	bitWrite(id.direct,12,0); return;}	//--Выключить Аварийку и блокировку сонара.
if(sign=='0') {id.speed= 0; return;}										//--Скорость  0%
if(sign=='1') {id.speed=20; return;}										//--Скорость 20%
if(sign=='2') {id.speed=30; return;}										//--Скорость 30%
if(sign=='3') {id.speed=40; return;}										//--Скорость 40%
if(sign=='4') {id.speed=50; return;}										//--Скорость 50%
if(sign=='5') {id.speed=60; return;}										//--Скорость 60%
if(sign=='6') {id.speed=70; return;}										//--Скорость 70%
if(sign=='7') {id.speed=80; return;}										//--Скорость 80%
if(sign=='8') {id.speed=90; return;}										//--Скорость 90%
if(sign=='9') {id.speed=100; return;}										//--Скорость 100%
if(sign=='q') {id.speed=100; return;}										//--Скорость 100%
if(sign=='D') {id.cmnd='S'; id.speed=0; 									//--Полный стоп
			   bitWrite(id.direct,0,0); bitWrite(id.direct,1,0); bitWrite(id.direct,2,0); bitWrite(id.direct,3,0);return;}	
}
//-------------------------------------------------------------------------------------------------
//	Чтение и обработка команд из UART
//-------------------------------------------------------------------------------------------------
void RT_HW_CAR:: readRemoteControl(RT_HW_CAR_REMOTE_CONTROL_ID &id, int8_t uart, uint16_t timeOut){
if(id.dir==0) {if(uart==7) {RT_HW_Base.uartBegin(uart,9600);} else {RT_HW_Base.uartBegin(uart,115200);}  
			   if(RT_HW_Base.uartSerial(uart)) {bitSet(RT_HW_Base.device.uart.ready,0); id.dir++;}}; 		
if(id.dir==1) {do{ if(RT_HW_Base.uartAvailable(uart)>0)  {id.timeStart=RT_HW_MILLIS;  id.sign=RT_HW_Base.uartRead(uart); id.cnt++; 
				  convertRemoteControl(id, id.sign); if(id.cnt>10){id.cnt=0;}} else {id.cnt=0;} } while(id.cnt>0); 
			   if(RT_HW_Base.timeOutMs(id.timeStart, timeOut )) {id.direct=0; id.speed=0; id.cmnd='S';}}		  
}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
RT_HW_CAR RT_HW_Car;//--Создание внешнего объекта	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++



